(*********************************************************************)         
(*                                                                   *)         
(*COPYRIGHT: (c) 1989, 1990                                          *)         
(*           Gesellschaft fuer Mathematik und Datenverarbeitung mbH  *)         
(*           Institut fuer Informationstechnische Infrastrukturen    *)         
(*           Bereich Bonn                                            *)         
(*           D-5300 Bonn 2                                           *)         
(*                                                                   *)         
(*           (c) 1990, 1991                                          *)         
(*           EARN Association, Paris                                 *)         
(*                                                                   *)         
(*                                                                   *)         
(*     This program is written to support the EARN network and its   *)         
(*   associated networks like BITNET and NetNorth. It can be freely  *)         
(*   used and copied in the network and any other non commercial     *)         
(*   network as long as this header remains intact. It may not       *)         
(*   be used to support military activities in any way.              *)         
(*                                                                   *)         
(*     The program may not be sold without permission.               *)         
(*                                                                   *)         
(*DISCLAIMER:                                                        *)         
(*     Although the program has been tested by the author and other  *)         
(*   people, no warranty is made by the author, by GMD, or the       *)         
(*   EARN Association for the correct function of the program.       *)         
(*   The author, GMD, or the EARN Association cannot be held for     *)         
(*   any damage the program may cause.                               *)         
(*                                                                   *)         
(*     Maintenance or developments may or may not be available. No   *)         
(*   promise of such support or enhancement can be deduced anyhow.   *)         
(*                                                                   *)         
(*                                                                   *)         
(*NAME:                                                              *)         
(*                                                                   *)         
(*       G R ...                                                     *)         
(*                                                                   *)         
(*                                                                   *)         
(*FUNCTION/OPERATION:                                                *)         
(*                                                                   *)         
(* This program generates ROUTING tables for various subsystems      *)         
(* that are used in EARN and cooperating networks:                   *)         
(*                                                                   *)         
(* The major protocol used in this network is NJE. NJE is            *)         
(* implemented in several operating systems and it requires          *)         
(* different format of routing tables. Currently the major           *)         
(* format are for RSCS, JES2, JES3, JNET, UREP, NJEF, NTF.           *)         
(* GR will generate tables that can be used to add                   *)         
(* network definitions to a local environment.                       *)         
(*                                                                   *)         
(*                                                                   *)         
(* On top of the basic NJE layer there is a layer of RFC822 style    *)         
(* electronic mail.  GR will generate data that simplifies           *)         
(* the routing process of the mail layer.  Known implementations     *)         
(* for IBM environments are:                                         *)         
(*                                                                   *)         
(*   VM Network Mailer (also known as Crosswell Mailer or XMAILER)   *)         
(*   UCLA/Mail (and its derivatives like UCLA/Mail400).              *)         
(*                                                                   *)         
(* GR uses a data set commonly known at BITEARN NODES as a base      *)         
(* of data containing all information to generate routing tables.    *)         
(*                                                                   *)         
(* There are currently two versions of BITEARN NODES that must be    *)         
(* supported.  A general definition of the new format can be         *)         
(* found in the EARN technical document about NEWTAGS DESCRIPT.      *)         
(* The contents of the old file is documented in OLDTAGS DESCRIPT.   *)         
(*                                                                   *)         
(* The missing piece in both documents is a general                  *)         
(* description about how routing tables are generated from the       *)         
(* contents of BITEARN NODES. This is the place to fill this         *)         
(* hole.                                                             *)         
(*                                                                   *)         
(*                                                                   *)         
(*                                                                   *)         
(* Old format of BITEARN NODES.                                      *)         
(*                                                                   *)         
(*  For this old format we try to interprete the directory           *)         
(*  information as it was done by the predecessor of GR namely       *)         
(*  GENROUTS. It is obvious that errors of the old program are       *)         
(*  not copied (if they were detected). We will list the major       *)         
(*  differences between the interpretations, and the differences     *)         
(*  between the new and the old format of BITEARN NODES.             *)         
(*                                                                   *)         
(*  GR has no requirement for the order of entries in the            *)         
(*  data base. The file is treated as a flat list of entries.        *)         
(*  The first tag name in the file will be used as separator         *)         
(*  for entries. GENROUTS required the tag 'node'.                   *)         
(*  The only obvious requirement is that the tag name be different   *)         
(*  from any other tag name. The similarity between BITEARN NODES    *)         
(*  and a standard names file may suggest to use 'nick'.             *)         
(*  In the following we will only use the wording "attribute node".  *)         
(*                                                                   *)         
(*  Version Entry:                                                   *)         
(*                                                                   *)         
(*     This is the first entry in the file although there            *)         
(*     are no requirement to have this. The first node entry should  *)         
(*     have an attribute GENROUTS, so the GR can check whether       *)         
(*     it can use the file.                                          *)         
(*                                                                   *)         
(*                                                                   *)         
(*  Attribute GENROUTS:                                              *)         
(*                                                                   *)         
(*     The value of this attribute should contain a character        *)         
(*     string.  It is compared to a time stamp in GR of the          *)         
(*     form YY/MM/DD. If the value of the attributes is larger,      *)         
(*     GR will stop running. This indicates that an obsolete         *)         
(*     version of GR is being used. This check is also in            *)         
(*     older versions of GENROUTS, at least those that               *)         
(*     work at all on the file have this check.                      *)         
(*                                                                   *)         
(*  Other attributes of the version entry are treated as for         *)         
(*  node entries as this entry also creates a pseudo node in         *)         
(*  the generated routing tables.                                    *)         
(*                                                                   *)         
(*  Node Entries:                                                    *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute NODE:                                               *)         
(*                                                                   *)         
(*       This attribute gives the name for this entry in the file.   *)         
(*                                                                   *)         
(*       The name will be used as given in routing tables for        *)         
(*       NJE implementations. Therefore certain restrictions         *)         
(*       apply to the name space in this case.                       *)         
(*                                                                   *)         
(*       GENROUTS will check the validity of names if routing        *)         
(*       tables are produced.                                        *)         
(*                                                                   *)         
(*       (Some remark of nodename.BITNET for mailer tables???).      *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute NODENUM:                                            *)         
(*                                                                   *)         
(*       This is an integer to create unique node numbers in JES2    *)         
(*       routing tables.                                             *)         
(*                                                                   *)         
(*       The number is use for the tie break numbering scheme.       *)         
(*       (see best path algorithm).                                  *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute INFO:                                               *)         
(*                                                                   *)         
(*       This attribute contains a list of keywords to direct        *)         
(*       the generation of routing tables. GENROUTS used a keyword   *)         
(*       JES2-nnn where nnn is a number between 1 and ??? that       *)         
(*       used to generate NJE node numbers for JES2 routing tables.  *)         
(*       The value of nnn is added to the nodenum/aliasnum attribute.*)         
(*                                                                   *)         
(*       The info attribute will no longer be supported by GR,       *)         
(*       it is neccesssry to use the new routtab attribute to get    *)         
(*       the same effect.                                            *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute COUNTRY:                                            *)         
(*                                                                   *)         
(*       The old GENROUTS uses the value of this attributes when     *)         
(*       routing tables for all nodes in a country should be         *)         
(*       produced. This feature is no longer available.              *)         
(*       The attribute will eventually be used for hiearchical       *)         
(*       routing.                                                    *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute ALIAS:                                              *)         
(*                                                                   *)         
(*       This attribute is used as if there is a directly connected  *)         
(*       node having this name.                                      *)         
(*       In the new format this attribute is no longer available     *)         
(*       but support will be continued.                              *)         
(*                                                                   *)         
(*       The attribute is copied also into XMAILER NAMES.            *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute ALIASNUM:                                           *)         
(*                                                                   *)         
(*       This attribute is used as if there is a directly connected  *)         
(*       node having this nodenum. It is only used if an ALIAS       *)         
(*       attribute exists, too.                                      *)         
(*       In the new format this attribute is no longer available     *)         
(*       but support will be continued.                              *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute NETSOFT:                                            *)         
(*                                                                   *)         
(*       This attribute determines the routing table format for      *)         
(*       the NJE system used at the node represented by this node    *)         
(*       entry. It is required when an NJE routing table is          *)         
(*       generated for this node entry.                              *)         
(*                                                                   *)         
(*       The value of this attribute always start with a 4           *)         
(*       character name that describes an NJE implementation.        *)         
(*       Different tables for RSCS, JES2 (three versions), JES3,     *)         
(*       JNET are supported.  The next token (= the next word        *)         
(*       separated by blank) must contain a version for JES2.        *)         
(*                                                                   *)         
(*       In order to allow a migration to the new format GENROUTS    *)         
(*       will always look for the existance of the routtab           *)         
(*       attribute first.                                            *)         
(*                                                                   *)         
(*       Currently ownership of the netsoft attribute is shared      *)         
(*       between GENROUTS and the VM Network Mailer generation       *)         
(*       tool for MG uses the value JNET (and UREP?)  in order to    *)         
(*       create a default of userid truncation.                      *)         
(*                                                                   *)         
(*       There is a rumour that :netsoft. will be phased out and     *)         
(*       the VM Network Mailer Version 2.04 will no longer use       *)         
(*       the field and assume truncate as a default.                 *)         
(*                                                                   *)         
(*       This means that userids for NJE systems can now be          *)         
(*       always longer than 8 characters.                            *)         
(*                                                                   *)         
(*       The attribute is copied also into XMAILER NAMES.            *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute MAILER:                                             *)         
(*                                                                   *)         
(*       This value is copied as is into the XMAILER NAMES file.     *)         
(*       It is also interpreted when the UCLA/Mail routing table     *)         
(*       is generated. The authors of the VM network mailer claim    *)         
(*       that the syntax of this attribute is owned by them.         *)         
(*       As long as we simply copy this value into XMAILER NAMES     *)         
(*       we are forced to support that view.                         *)         
(*                                                                   *)         
(*       Since we started designing this program several people      *)         
(*       have written a program to extract required information      *)         
(*       for UCLA/Mail. Therefore there is no need anymore to        *)         
(*       support UCLA/Mail sites.                                    *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute NET:                                                *)         
(*                                                                   *)         
(*       GENROUTS as well as GR copy the attribute into the XMAILER  *)         
(*       NAMES file.                                                 *)         
(*                                                                   *)         
(*       The attribute will be used in hierachical routing.          *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute GMTOFF:                                             *)         
(*                                                                   *)         
(*       This attributes was used by the old GENROUTS                *)         
(*       for RSCS V1 routing tables in the comment part that         *)         
(*       describes the direct connections to this node.              *)         
(*       GR ignores this attribute.                                  *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute ABBR:                                               *)         
(*     Attribute SITE:                                               *)         
(*     Attribute SYSTEM:                                             *)         
(*     Attribute CONTACT:                                            *)         
(*                                                                   *)         
(*       These attributes were used to form comments in the routing  *)         
(*       tables or were used in the INFO file generated by the old   *)         
(*       GENROUTS.                                                   *)         
(*                                                                   *)         
(*       These tags do no longer exist in the new format             *)         
(*       or are replaced by a different hierarchical attributes.     *)         
(*                                                                   *)         
(*       GR ignores these attributes.                                *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute LINKIGN:                                            *)         
(*                                                                   *)         
(*       This attribute was used by the old GENROUTS in the          *)         
(*       best path algorithm in order to bypass non desirable        *)         
(*       indirect routes. The implementation was always wrong.       *)         
(*                                                                   *)         
(*       GR does no longer use the attribute, even                   *)         
(*       for the old BITEARN NODES format.                           *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute VIA:                                                *)         
(*                                                                   *)         
(*       This attribute contained the name of a directly connected   *)         
(*       node that is on the best path towards the first BITNET      *)         
(*       node CUNYVM. The value of this tag is used as the           *)         
(*       ADJNODES with a type of 9L if there are NO adjnodes given.  *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Tags ADJNODESnn:                                              *)         
(*     Tags ADJTYPESnn:                                              *)         
(*                                                                   *)         
(*       We use the wording 'tags' here because the interpretation   *)         
(*       as attributes is not one to one. There is a multiple value  *)         
(*       attribute that consists of a node name of a directly        *)         
(*       connected node and some information value to create best    *)         
(*       paths.                                                      *)         
(*                                                                   *)         
(*       The adjtypesnn tags currently may only contain a list of    *)         
(*       the following values: 2L 4L 9L ZL CC. They represent        *)         
(*       lines with speeds of 2.4kb, 4.8kb, 9.6kb, 64kb, or a CTC    *)         
(*       connection. This list is used to create cost values in      *)         
(*       the best path algorithm used. The values of this tag do     *)         
(*       not necessarily correspond with an actual line speed.       *)         
(*       GR may require different values to be used in order         *)         
(*       to select a desired path between otherwise identically      *)         
(*       rated indirect paths. (see also LINKIGN).                   *)         
(*                                                                   *)         
(*       The values of these tags are interpreted as a special       *)         
(*       encoding of LINKSnn entries. If there are LINKSnn entries   *)         
(*       in one node, all adjnodesnn/adjtypes are ignored for this   *)         
(*       node. This was done in order to have GENROUTS and GR        *)         
(*       to run on the same data base. Meanwhile history has         *)         
(*       created the situation that GENROUTS doesn't seem to work    *)         
(*       anymore, and another program uses both the new and old      *)         
(*       definitions.                                                *)         
(*                                                                   *)         
(* New format of BITEARN NODES:                                      *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Version Entry:                                                *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute GENROUTS:                                           *)         
(*                                                                   *)         
(*       Checked to see whether the version of GR can operate on     *)         
(*       the directory.                                              *)         
(*                                                                   *)         
(*     Other attributes of the version entry are treated as for      *)         
(*     node entries.                                                 *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute NODE:                                               *)         
(*                                                                   *)         
(*       This attribute gives the name for this entry in the file.   *)         
(*                                                                   *)         
(*       The name will be used as given in routing tables for        *)         
(*       NJE implementations. Therefore certain restrictions         *)         
(*       apply to the name space in this case.                       *)         
(*                                                                   *)         
(*       GR will check the validity of names if routing              *)         
(*       tables are produced.                                        *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute ROUTTAB:                                            *)         
(*                                                                   *)         
(*       I assume that the beginning of the attribute up to the      *)         
(*       first occurence of an open paranthesis is owned by GR; in   *)         
(*       the following this part of the routtab attribute will be    *)         
(*       called grip (GR Infomation Parameter).                      *)         
(*                                                                   *)         
(*       The routtab attribute must exist in a node entry although   *)         
(*       the description in BITEARN NODES classifies this            *)         
(*       as a role thus people might conclude that the site          *)         
(*       and member reference are might be inspected. GR does        *)         
(*       not follow these references. This description is only       *)         
(*       describes the association of the grip to a node, the        *)         
(*       other parameters may be structured as a role.               *)         
(*                                                                   *)         
(*       I have heard that some people are taking some format and    *)         
(*       do some post processing on it. This is strongly             *)         
(*       discouraged. They are invited to send me a specification    *)         
(*       of a table format.                                          *)         
(*                                                                   *)         
(*       In a more formal way this attribute is:                     *)         
(*       routtab : grip <<" "*>  "(" other-parameters >              *)         
(*                                                                   *)         
(*       grip : RSCS  | JES3  | JNET  | UREP                         *)         
(*              | JES2 <" "* version > <" "* jes2parm* >             *)         
(*              | NJEF <" "* node "=" lid>*                          *)         
(*              | NTF                                                *)         
(*              | PATH                                               *)         
(*              | NONE                                               *)         
(*                                                                   *)         
(*       version  : V1 | V2 | V3 | V4                                *)         
(*       jes2parm : offset | sort                                    *)         
(*       offset   : "O=" <number>                                    *)         
(*       sort     : "S=" <NUM"                                       *)         
(*       node     : name or number                                   *)         
(*       lid      : three characters                                 *)         
(*                                                                   *)         
(*       A detailed description of the meaning of the grip is        *)         
(*       given now:                                                  *)         
(*                                                                   *)         
(*       RSCS:                                                       *)         
(*          The old GENROUTS had two formats for RSCS.               *)         
(*          This is no longer supported as the format only           *)         
(*          differed in the comments. So there are no optional       *)         
(*          parameters                                               *)         
(*                                                                   *)         
(*       JES2:                                                       *)         
(*          There are currently three different formats of           *)         
(*          NJE initialization files generated by GENROUTS.          *)         
(*                                                                   *)         
(*          The three different versions are indicated               *)         
(*          by the first positional optional parameter which         *)         
(*          is the character V and one digit:                        *)         
(*                                                                   *)         
(*          V1: the assembler style syntax of pre-sp.1.3.6/2.1.5     *)         
(*               systems,                                            *)         
(*          V2: the pre-sp3.x.x syntax (pli style)    (default)      *)         
(*          V3: sp3.x.x syntax                                       *)         
(*          V4: currently the same as V3, it is just here because    *)         
(*              people like to use the version of JES2 as a value.   *)         
(*                                                                   *)         
(*          If GR would not generate comments then the               *)         
(*          first two formats would even be identical.               *)         
(*                                                                   *)         
(*          The second positional token may be the character O=      *)         
(*          followed by a number. This number will be                *)         
(*          to all JES2 node number in order to allow a range of     *)         
(*          free numbers for local purposes. Note that the CONNECT   *)         
(*          statements for the own node and its neighbour only exist *)         
(*          as a comment. Therefore only the own node can have       *)         
(*          number which is only locally defined.                    *)         
(*                                                                   *)         
(*          S=NUM indicates that the routing table should be         *)         
(*          sorted in ascending by ascending JES2 node numbers.      *)         
(*          S= is the default and indicated that the table will      *)         
(*          contain the nodes as they appear in BITEARN NODES.       *)         
(*                                                                   *)         
(*       JES3:                                                       *)         
(*          There is only one JES3 format                            *)         
(*                                                                   *)         
(*       JNET:                                                       *)         
(*          There is only one JNET format.                           *)         
(*                                                                   *)         
(*       NJEF:                                                       *)         
(*                                                                   *)         
(*          There is a default logic to create Lid names for         *)         
(*          NJEF tables. This default behaviour can be modified      *)         
(*          a list of node=lid associations.                         *)         
(*                                                                   *)         
(*       UREP:                                                       *)         
(*          There is only one UREP format.                           *)         
(*                                                                   *)         
(*       NTF:                                                        *)         
(*          There are several versions of NTF around. The            *)         
(*          of how to generate tables for NTF is somewhat obscure.   *)         
(*          experiences have shown that the contents of the file     *)         
(*          that is generated with the CTRYSTAT directive is         *)         
(*          always sufficient to generate tables for CDC environment.*)         
(*          Therefore we'll just create the same table.              *)         
(*                                                                   *)         
(*       PATH:                                                       *)         
(*          This creates a table as in a pathalias output.           *)         
(*                                                                   *)         
(*       NONE:                                                       *)         
(*          No table is generated at all.                            *)         
(*                                                                   *)         
(*       Any other value for the format will create an RSCS table.   *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute NODERESIST:                                         *)         
(*                                                                   *)         
(*       The value of this attribute is added to all resistances     *)         
(*       of outgoing connections.                                    *)         
(*                                                                   *)         
(*       The default used by GR is .001 unless the program finds     *)         
(*       a noderesist value in the first node entry which has to     *)         
(*       be named 'VERSxxxx', too.                                   *)         
(*       This is done to get compatible results for old and          *)         
(*       new BITEARN NODES formats, and to simulate the effect that  *)         
(*       the old GENROUTS used a path with a smaller number of       *)         
(*       nodes if two paths have the same total resistance.          *)         
(*       The attribute is only used when LINKSnn attrbutes exist.    *)         
(*                                                                   *)         
(*     Attribute LINKSnn:                                            *)         
(*                                                                   *)         
(*        The names of adjacent nodes and the resistance value       *)         
(*        (derived or explicetely given), and the NODERESIST         *)         
(*        attribute will be used in a shortest path algorithm        *)         
(*        which is based on the classical Dijkstra algorithm.        *)         
(*                                                                   *)         
(*                                                                   *)         
(*     Attribute SERVERSnn:                                          *)         
(*                                                                   *)         
(*        The first occurence of an serverattribute with type MAIL   *)         
(*        is used to create a :mailer. tag in the XMAILER names      *)         
(*        file. The three values BSMTP, RFC822 and OUTONLY are       *)         
(*        mapped to the parameters BSMTP 3, BITNET 2, and            *)         
(*        DEFRT 1 TRUNCATE.                                          *)         
(*        A warning is issued if there is more than one server       *)         
(*        attribute with type MAIL, or when there is an additional   *)         
(*        attribute mailer.                                          *)         
(*        For entries that do no have a TYPE NJE we will generate    *)         
(*        a mailer value of ? NOMAIL 2 TYPE.                         *)         
(*                                                                   *)         
(*     Attribute TYPE:                                               *)         
(*                                                                   *)         
(*        If the value of this attribute contains the keyword        *)         
(*        NJE then the node attribute will be included in            *)         
(*        an NJE initialization statement file. GR refuses           *)         
(*        generate an NJE initialization statement                   *)         
(*        file for a non NJE entry.                                  *)         
(*                                                                   *)         
(*     Attribute COUNTRY:                                            *)         
(*     Attribute NET:                                                *)         
(*                                                                   *)         
(*       These attribute are used to regionalize the internal        *)         
(*       topology.                                                   *)         
(*                                                                   *)         
(*       Paths are checked for the minimum number of                 *)         
(*       network/country changes along the path (in that order.)     *)         
(*       (This feature is not yet implemented.)                      *)         
(*                                                                   *)         
(*       See also: Best path algorithm below.                        *)         
(*                                                                   *)         
(*       The values of the tags will be included in the routing      *)         
(*       tables for RSCS and JNET.                                   *)         
(*                                                                   *)         
(*                                                                   *)         
(* Best Path Algorithm:                                              *)         
(*                                                                   *)         
(*                                                                   *)         
(*  There is NO standard algorithm for any kind of network.  The     *)         
(*  classical techniques to find some shortest path can be used      *)         
(*  as basic techniques.  The limitations of the existing NJE        *)         
(*  systems lead to some requirements. Furthermore any technique     *)         
(*  used for routing must be implementatable with the existing       *)         
(*  NJE systems (open for extensions if NJE systems will             *)         
(*  eventually be enhanced).                                         *)         
(*                                                                   *)         
(*  The classical Dijkstra shortest path algorithm finds             *)         
(*  one (of several) shortest path trees in a graph where it         *)         
(*  may happen that direct links are not used if an indirect path    *)         
(*  is shorter. In addition no symmetrie is guaranteed when          *)         
(*  two equally weighted paths exist.                                *)         
(*                                                                   *)         
(*  The routing tables generated by our tools should have the        *)         
(*  following properties:                                            *)         
(*                                                                   *)         
(*  - A direct connection should always be considered as best        *)         
(*    path.                                                          *)         
(*                                                                   *)         
(*  - Paths should be transitive: The best path from one node        *)         
(*    to another contains the best paths between all nodes           *)         
(*    on the path (in one direction).                                *)         
(*                                                                   *)         
(*  - Paths should be symmetric if the graph is symmetric.           *)         
(*    This requirement can be solved by a global tie break rule.     *)         
(*                                                                   *)         
(*  - Paths should be as short as possible.                          *)         
(*                                                                   *)         
(*  One can implement this in two ways:  It is either a network      *)         
(*  management problem, i.e.  one could ensure that the input        *)         
(*  topology never has a indirect path with better costs, or one     *)         
(*  implements this rule in the routing algorithm. The latter is     *)         
(*  preferred because with the old style BITEARN NODES correct       *)         
(*  geometry cannot always be easily guaranteed.                     *)         
(*                                                                   *)         
(*  For the modification of the shortest path algorithm this         *)         
(*  means that whenever a "shorter" path is a candidate it is        *)         
(*  only taken when nowhere on path from the origin node there is    *)         
(*  a direct link to the last node on that path. (E.g.  a 9.6kb      *)         
(*  connection will be always preferred over two indirect 64kb       *)         
(*  lines.).                                                         *)         
(*  One has to ensure that all subpaths are contained in the new     *)         
(*  candidate. For a network that has proper geopmetric behavior     *)         
(*  this is a simple consequence of the path length but in           *)         
(*  our case we have to verify this explicitely in all those cases   *)         
(*  where a geometry violation occurs, i.e. where we have not        *)         
(*  a shorter path.                                                  *)         
(*                                                                   *)         
(*  A tentative algorithm generating these  pbp's (a term invented   *)         
(*  by david Lippke) has been implemented.                           *)         
(*                                                                   *)         
(*  The second point is to ensure that when two paths are equally    *)         
(*  weighted, then from both ends the same path is selected.         *)         
(*  This can be implemented if some extra measure/partial order      *)         
(*  is defined on the list of all paths. Whenever two paths have     *)         
(*  identical costs the partial order on the paths is used for       *)         
(*  selection.  On an symmetric network this ensures symmetric       *)         
(*  routes.                                                          *)         
(*  The first step of defining a partial order is to count the       *)         
(*  number of hops, and take the one with less hops. Then we remain  *)         
(*  we a class of paths with equal number of hops.                   *)         
(*                                                                   *)         
(*  A possible solution is to assign numbers to all nodes, and       *)         
(*  select the path with the lowest number on the separate parts     *)         
(*  of the two paths.  The numbering scheme should be stable with    *)         
(*  the existance of the network, i.e. a long term number like       *)         
(*  the JES2 node number should be used as it is done in GR.         *)         
(*                                                                   *)         
(*                                                                   *)         
(* Hierarchical routing: (still a proposal)                          *)         
(*                                                                   *)         
(*     The following variation is here only for discussion and not   *)         
(*     implemented.                                                  *)         
(*                                                                   *)         
(*     Links that cross network and country boundaries are always    *)         
(*     treated as longer than any other path. The requirements to    *)         
(*     have correct network/country definitions will be              *)         
(*     "internalized" (into each "domain").                          *)         
(*                                                                   *)         
(*     If there is only one entry point for a network/country        *)         
(*     then GR does not use the topology at all if a                 *)         
(*     NJE routing table is generated. It will simply assume         *)         
(*     that all nodes are connected to the entry node of the         *)         
(*     other country.                                                *)         
(*                                                                   *)         
(*     Hierarchical path determination could be simulated by         *)         
(*     choosing very high resistance values for "critical"           *)         
(*     connections, but this will only work if there are no          *)         
(*     errors in the data base.                                      *)         
(*                                                                   *)         
(*     It is assumed that the "effects" of this regionalization      *)         
(*     are intended:                                                 *)         
(*     With hierachical routing, only the nodes in the same          *)         
(*     domain will be affected by errors in the connectivity         *)         
(*     definitions.  GR will simply assume connectivity              *)         
(*     inside another domain.                                        *)         
(*     E.G. If there is only ONE path into a domain, then all        *)         
(*     nodes inside the domain will be assumed to be directly        *)         
(*     connected to the entry node.                                  *)         
(*     A more concrete example: in order to generate routing         *)         
(*     tables for EARN systems the only information of               *)         
(*     BITNET nodes that need to be correct is the node name         *)         
(*     and the JES2 node number.                                     *)         
(*                                                                   *)         
(*     The definition of a "domain" is not done. The combination     *)         
(*     of country/network used here is only an example.              *)         
(*                                                                   *)         
(*     It may be useful to divide the US component of BITNET into    *)         
(*     several routing domains. Maybe the administration hierarchy   *)         
(*     of members/sites also can be used to define a hierarchy.      *)         
(*                                                                   *)         
(*     The main benefit is to be able to recover from errors in      *)         
(*     the data base in all areas that are not close to the          *)         
(*     point of error. The eventual speedup for the shortest         *)         
(*     path algorithm is irrelevant. We have tested a completely     *)         
(*     degenerated 'best' path algorithm where all nodes are         *)         
(*     treated as direct neighbours (i.e. the spanningtree routine   *)         
(*     in this program is basically a noop). This was about one      *)         
(*     percent of the code (the major cpu eater is definetely the    *)         
(*     PASCAL IO system).                                            *)         
(*                                                                   *)         
(*                                                                   *)         
(*ENTRY POINTS:                                                      *)         
(*                                                                   *)         
(*    GR                                                             *)         
(*    The program can be called as a CMS command processor (VM),     *)         
(*    or as an MVS-"main-program". The "TSO" directive is only       *)         
(*    usable if the program is called under TSO.                     *)         
(*                                                                   *)         
(*    The external name of the program will be either GRIBM or       *)         
(*    GRANSI83.                                                      *)         
(*                                                                   *)         
(*    GR was chosen because I have learned that the laziness of      *)         
(*    -nix programmers is a feature. Needless to say that the        *)         
(*    name also allows me to append all kinds of nice suffixes       *)         
(*    like IBM and ANSI83 in order to distinguish the different      *)         
(*    versions of the program.                                       *)         
(*                                                                   *)         
(*INPUT:                                                             *)         
(*                                                                   *)         
(* All processing of GR is directed by the contents                  *)         
(* of the file associated with the ddname COMMANDS.                  *)         
(* The contents must be a list of directives. Parameters are         *)         
(* separated by at least one blanks. The <> brackets indicate        *)         
(* optional parameters. If there are multiple optional               *)         
(* parameters an asterisk * can be used a placeholder, e.g.          *)         
(* a filespecification of * is identical to no filespecification.    *)         
(* Some parameters are only usable in the IBM version.               *)         
(*                                                                   *)         
(*                                                                   *)         
(* GETNODES <filespecification>                                      *)         
(*                                                                   *)         
(*      The contents of a file is read and an internal topology      *)         
(*      will be created. If no filespecification is given then       *)         
(*      the data definition name / filedef DATABASE is used.         *)         
(*      The ANSI83 does not support a file specification.            *)         
(*      The IBM version allows an operand that is used in a          *)         
(*      PASCAL statement Reset(database,filespecification)           *)         
(*                                                                   *)         
(*      Several GETNODES directives are possible. The tag values     *)         
(*      of the first occurence of a node entry are taken.            *)         
(*      All GETNODES directives must occur before the first          *)         
(*      directive that generates a routing table (e.g. GENROUTE).    *)         
(*                                                                   *)         
(* XMAILER <filespecification>                                       *)         
(*                                                                   *)         
(*      This directive can be given before the GETNODES directive.   *)         
(*      I can be considered as a special form of parameterizing      *)         
(*      the GETNODES processing. While reading the database,         *)         
(*      some attributes are copied into the file associated with     *)         
(*      the data defintion statement/filedef XMAILER.                *)         
(*      The filespecification can be used to modify the file         *)         
(*      association.                                                 *)         
(*                                                                   *)         
(*      The current version copies the attributes                    *)         
(*                                                                   *)         
(*      node, mailer, net, netsoft, and alias.                       *)         
(*      For node a tagname :nick. is generated.                      *)         
(*                                                                   *)         
(*      If an attribute doesn't exist in the database XMAILER NAMES  *)         
(*      will get an empty tag.                                       *)         
(*      No validation of contents is done.                           *)         
(*                                                                   *)         
(*      If is unclear to me how I am supposed to handle the server   *)         
(*      tag. I can either just copy the data, or only select those   *)         
(*      tags with type MAIL.                                         *)         
(*                                                                   *)         
(*                                                                   *)         
(* CMS command                                                       *)         
(*                                                                   *)         
(*      The given command will be given to the CMS                   *)         
(*      environment. This directive is not supported in the          *)         
(*      ANSI83 version.                                              *)         
(*      This directive should only be used in a CMS environment,     *)         
(*      otherwise the result is unpredictable.                       *)         
(*                                                                   *)         
(* TSO command                                                       *)         
(*                                                                   *)         
(*      The given command will be executed as a TSO command or       *)         
(*      CLIST. This directive is not supported in the                *)         
(*      ANSI83 version.                                              *)         
(*      This directive should only be used in a TSO environment,     *)         
(*      otherwise the result is unpredictable.                       *)         
(*                                                                   *)         
(* GENROUTE node <filespecification> <grip>                          *)         
(*                                                                   *)         
(*      A routing table for the given node will be generated and     *)         
(*      written to a file. If no filespecification is given then     *)         
(*      the file associated with the data definition statement /     *)         
(*      filedef ROUTETAB is used. The filespeciafication allows      *)         
(*      modifications. It is used in a PASCAL instruction            *)         
(*      Rewrite(routetab,filespecification). The file specification  *)         
(*      is not available in the ANSI83 version.                      *)         
(*                                                                   *)         
(*      In the IBM version the file is also closed at the end        *)         
(*      of this directive, in order to allow CMS and TSO             *)         
(*      directive to manipulate the file. ANSI83 has no Close        *)         
(*      routine.                                                     *)         
(*                                                                   *)         
(*      The optional <grip> (gr information parameter> allows        *)         
(*      to specify the routing table format and options for the      *)         
(*      generation of the routing table. The format of this          *)         
(*      parameter is identical to what we have described under the   *)         
(*      section "routtab attribute".                                 *)         
(*                                                                   *)         
(*      Note that the <grip> must be preceeded by a                  *)         
(*      <filespecification>, use an asterisk ( * ) to indicate       *)         
(*      an "empty" parameter.                                        *)         
(*                                                                   *)         
(*      From the description above it is obvious that the ANSI83     *)         
(*      version of GR would not be able to generate more than        *)         
(*      one routing table in one run. Only the last table would      *)         
(*      in the file routetab. Therefore the ANSI83 will not          *)         
(*      use the PASCAL rewrite function to open the routetab file    *)         
(*      except for the first GENROUTE directive. Instead of that     *)         
(*      GR will write 80 equal sign characters as a separator.       *)         
(*                                                                   *)         
(*      Rules for overriding:                                        *)         
(*                                                                   *)         
(*       The NJE format is required so it is obvious that the        *)         
(*       given format overrides the data of the routetab tag.        *)         
(*                                                                   *)         
(*       The JES2 version is optional, if not given the              *)         
(*       information of the routetab remains active.                 *)         
(*                                                                   *)         
(*       The JES2 offsets and sort are left as is if not specified.  *)         
(*       If the introducing characters S= and O= are specified,      *)         
(*       the default values (sort as input and offset 0) will        *)         
(*       be used.                                                    *)         
(*                                                                   *)         
(*       If at least one node=lid association is given for NJEF      *)         
(*       tables, all associations of the routetab will be ignored.   *)         
(*                                                                   *)         
(*                                                                   *)         
(* CTRYSTAT node <filespec>                                          *)         
(*                                                                   *)         
(*      This is a directive to generate a table to be used by        *)         
(*      line usage statistics generators.                            *)         
(*      For the standard accounting information created by NJE       *)         
(*      systems not containing sufficient information about          *)         
(*      which node belong to which country/network and which line    *)         
(*      is normally used, GR creates the following list that be      *)         
(*      used by such programs:                                       *)         
(*                                                                   *)         
(*      The list contains all nodes of the network.                  *)         
(*      The similarity to the RSCS routing table is intended in      *)         
(*      order to allow that existing statistic tools can be used     *)         
(*      without modification.                                        *)         
(*                                                                   *)         
(*      Comments are indicated by a asterisk in column 1 of a line.  *)         
(*                                                                   *)         
(*      Non comment lines contain contain a list of information      *)         
(*      tokens separated by one or more blanks. The total number of  *)         
(*      tokens is undefined, it may be extended in future releases.  *)         
(*                                                                   *)         
(*      Currently the following tokens are created:                  *)         
(*                                                                   *)         
(*      1 type      : This is one of the strings                     *)         
(*                    LOCAL   defines the local node                 *)         
(*                    ALIAS   defines an alias to the local node     *)         
(*                    LINK    defines a neighbour node               *)         
(*                    ROUTE   defines a non neighbour node.          *)         
(*                                                                   *)         
(*      2 node      : The NJE node name for which the following      *)         
(*                    information is generated.                      *)         
(*                                                                   *)         
(*      3 neighbour : The name of an NJE neighbour which is used     *)         
(*                    as a route to the node.                        *)         
(*                    The own node will have a * indicating no       *)         
(*                    that no neighbour exists. This line will       *)         
(*                    always be the first line in the table.         *)         
(*                    An alias of the as to the OWN node also has    *)         
(*                    the own node as neighbour!                     *)         
(*                    The information of all neighbours, i.e. where  *)         
(*                    node = neighbour will be after the first       *)         
(*                    line.                                          *)         
(*                                                                   *)         
(*      4 network   : The network which the node is associated with. *)         
(*                    This is either a variable length name, or the  *)         
(*                    character ?.                                   *)         
(*                                                                   *)         
(*      5 country   : The country which the node is associated with. *)         
(*                    This is either a two letter ISO country code,  *)         
(*                    or ?? if no data are found in the nodes table. *)         
(*                                                                   *)         
(*      The beginning of the file contains a line with the           *)         
(*      following text:                                              *)         
(*                                                                   *)         
(*      '* ROUTING STATEMENTS FOR NODE nodename'                     *)         
(*                                                                   *)         
(*      This is done to allow the CTRYEXEC to be used unmodified.    *)         
(* QUIET                                                             *)         
(*      Do not produce directive listings on SYSPRINT, and           *)         
(*      do not create debugging information.                         *)         
(*                                                                   *)         
(* SERMON                                                            *)         
(*      Create a detailed processing log (list of directives).       *)         
(*      The is the default setting.                                  *)         
(*                                                                   *)         
(* DEBUG                                                             *)         
(*      Create some information for debugging.                       *)         
(*                                                                   *)         
(* GENPATHS node                                                     *)         
(*      This is a directive for debugging purposes.                  *)         
(*                                                                   *)         
(*                                                                   *)         
(* Filespecifications:                                               *)         
(*                                                                   *)         
(* The possible values can be found in the PASCAL application        *)         
(* manual. For TSO a useful example is "DDNAME=ddname", for          *)         
(* CMS a "NAME=Fn.Ft.Fm" or (similar to TSO): "DDNAME=filedef".      *)         
(*                                                                   *)         
(* If a file specification is given for one directive, and the       *)         
(* same file is used in subsequent directives, e.g. two              *)         
(* GENROUTE directives, and if the second directive has no           *)         
(* explicit filespecification, the effect is undefined.              *)         
(*                                                                   *)         
(* Therefore the following technique is recommended:                 *)         
(*                                                                   *)         
(*    GENROUTE mynode DDNAME=file1                                   *)         
(*    GENROUTE other  DDNAME=file2                                   *)         
(*                                                                   *)         
(* For the ANSI83 no file specifications are possible. In order      *)         
(* to create a good result if more than one GENROUTE directive       *)         
(* is given, the different tables are separated by 80 equal signs    *)         
(* (=). This allows easy separation by any kind of post processor    *)         
(* that might be desired.                                            *)         
(*                                                                   *)         
(* The main control file routine logic will be very simple:          *)         
(* Some procedures in GR are called which map one to one to the      *)         
(* directives in the control file and some obvious operands.         *)         
(* Porting this feature or part of it should not create severe       *)         
(* problems.  It is obvious that the CMS and TSO directives can      *)         
(* simply be removed or replaced by something else.                  *)         
(*                                                                   *)         
(* The CMS/TSO command feature was added in order to allow           *)         
(* server machines to spin off generated routing tables.             *)         
(* Note that GR may terminate later, so a recovery                   *)         
(* mechanism cannot be a simple rerun on the same input!             *)         
(*                                                                   *)         
(*                                                                   *)         
(*OUTPUT:                                                            *)         
(* If running in "sermon" mode, a log of all actions is              *)         
(* created on file SYSPRINT, if running in "quiet" mode,             *)         
(* error messages are written to file SYSPRINT.                      *)         
(*                                                                   *)         
(* NJE routing tables:                                               *)         
(*                                                                   *)         
(*     For the syntax of the routing table I will try to use         *)         
(*     robust specifications. I will not use abbreviations           *)         
(*     if some official documents just says something like:          *)         
(*     Abbreviation is possible as long as a unique prefix is        *)         
(*     used.                                                         *)         
(*                                                                   *)         
(*     There will be no definitions for neighbours in the            *)         
(*     file, i.e. you have to define it by yourself. BUT: The node   *)         
(*     numbers of neighbours may exist in JES2 CONNECT statements.   *)         
(*     Although the neighbours have to be separately defined         *)         
(*     it is necessary to use the node numbers created by GR.        *)         
(*                                                                   *)         
(*     For some systems the files can be used as is (e.g.  by        *)         
(*     concatenating this member to others), for RSCS the file       *)         
(*     has to be inserted as a whole in the middle of an RSCS        *)         
(*     CONFIG file.                                                  *)         
(*                                                                   *)         
(*     The routing tables may have "special comments" in order       *)         
(*     to allow to create automatic "rerouting/backup" tools         *)         
(*     (especially for RSCS). The current version of the program     *)         
(*     has the network and country name as comments for RSCS and     *)         
(*     JNET tables.                                                  *)         
(*                                                                   *)         
(*     Beyond this no assumption should be made about the            *)         
(*     contents of comments in the routing tables. GR may            *)         
(*     be changed without warning.                                   *)         
(*                                                                   *)         
(*     All routing tables except the one generated for the           *)         
(*     PATH format consist of two sections.                          *)         
(*                                                                   *)         
(*     - An introductory section containing a description of         *)         
(*       the version of the file and about the local                 *)         
(*       configuration. The text will be identical for               *)         
(*       ALL formats. The different comment syntaxes require         *)         
(*       different table syntaxes.                                   *)         
(*                                                                   *)         
(*       The old GENROUTS had created information that seemed to be  *)         
(*       usable to define the neighbours by just removing the        *)         
(*       comment characters.  This will no longer be possible.       *)         
(*                                                                   *)         
(*     - The second part is a list containing connection             *)         
(*       information for nodes that are more than one hub            *)         
(*       away. For each node there is at least one line              *)         
(*       containing a kind of a ROUTE definition as for RSCS.        *)         
(*                                                                   *)         
(*       This part may also contain some comments at those places    *)         
(*       where the neighbour would have been defined.                *)         
(*                                                                   *)         
(*       In this program no assumption should be made about the      *)         
(*       order of output statements. Authors of programs that        *)         
(*       use the routing table for other purposes TAKE CARE!!!       *)         
(*                                                                   *)         
(*                                                                   *)         
(*       RSCS:                                                       *)         
(*                                                                   *)         
(*        For each node a ROUTE statement of the form                *)         
(*                                                                   *)         
(*        'ROUTE nodename linkname network country'                  *)         
(*                                                                   *)         
(*        will be created.                                           *)         
(*                                                                   *)         
(*       The network and country "parameters" are not used by        *)         
(*       the current version of RSCS. Users of the routing table     *)         
(*       should be aware that the exact position of this             *)         
(*       comment part might eventually change in case when IBM       *)         
(*       introduces more parameters. Programmers that use the        *)         
(*       RSCS routing tables should not hard code the position       *)         
(*       into their programs.                                        *)         
(*                                                                   *)         
(*       JES3:                                                       *)         
(*                                                                   *)         
(*        For each non adjacent node a statement                     *)         
(*                                                                   *)         
(*        'NJERMT NAME=nodename,MAXLINE=0,PATH=linkname'             *)         
(*                                                                   *)         
(*        will be generated. The semantics is as for RSCS.           *)         
(*                                                                   *)         
(*       JNET:                                                       *)         
(*                                                                   *)         
(*        For each non adjacent node a statement                     *)         
(*                                                                   *)         
(*        ' DEFINE nodename /ROUTE=linkname ! network country '      *)         
(*                                                                   *)         
(*        will be generated. The semantics is as for RSCS.           *)         
(*                                                                   *)         
(*        The parameters are generated at fixed position in          *)         
(*        order to use fortram FORMAT statement for "parsing"        *)         
(*                                                                   *)         
(*       UREP:                                                       *)         
(*                                                                   *)         
(*        For each non adjacent node a statement                     *)         
(*                                                                   *)         
(*        'nodename linkname *'                                      *)         
(*                                                                   *)         
(*        will be generated. The semantics is as for RSCS.           *)         
(*                                                                   *)         
(*       The UREP 3.3 documentation says that there must be          *)         
(*       three operands with the last being an operating system      *)         
(*       type, and that the last field is not used by the code.      *)         
(*                                                                   *)         
(*       GR will create a "not often use" of the third field.        *)         
(*                                                                   *)         
(*                                                                   *)         
(*       NJEF:                                                       *)         
(*                                                                   *)         
(*        For each non adjacent node a statement                     *)         
(*                                                                   *)         
(*        'FAMILY,nodename,NODnum,Lid,linkname.'                     *)         
(*                                                                   *)         
(*        will be generated. The semantics is as for RSCS.           *)         
(*        num is the JES2 node number of the nodename, lid is        *)         
(*        create as follows:                                         *)         
(*                                                                   *)         
(*        The decimal JES2 node number of the neighbour              *)         
(*        is added to the base-36 "number" NJE using the characters  *)         
(*         0-9,A-Z as digits. The result is always as three          *)         
(*        "digit" base-36 "number".                                  *)         
(*                                                                   *)         
(*        This default can be changes by giving a list of            *)         
(*        node=lid associations as a parameter to the GENROUTE       *)         
(*        directive or in the :routtab. tag. Node can either         *)         
(*        be the name of a node or its JES2 node number.             *)         
(*                                                                   *)         
(*       JES2:                                                       *)         
(*                                                                   *)         
(*        For each node a statement of the form                      *)         
(*                                                                   *)         
(*        'Nnnnn NAME=name,NONETATH,NODEVATH,NOJOBATH,NOSYSATH,SNA'  *)         
(*                                                                   *)         
(*        will be created for the non SP 3.x.x formats.              *)         
(*        For SP 3.x.x a statement                                   *)         
(*                                                                   *)         
(*        'NODE(nnnn) NAME=name'                                     *)         
(*                                                                   *)         
(*        is generated. The AUTH parameter will not be used          *)         
(*        and should be defined elsewhere:                           *)         
(*                                                                   *)         
(*        'NODE(1-x) AUTH=(...)'                                     *)         
(*                                                                   *)         
(*        (The x should be an asterisk * which I cannot use in       *)         
(*        this description because it would mean end of comment      *)         
(*        in Pascal.)                                                *)         
(*                                                                   *)         
(*        For non adjecent connections of the minimal spanning       *)         
(*        tree statement of the form                                 *)         
(*                                                                   *)         
(*        'CONNECT NODEA=xxxx,NODEB=yyyy,MEMBA=1,MEMBB=1,REST=2      *)         
(*                                                                   *)         
(*        will be generated, where xxxx and yyyy are nodenumbers     *)         
(*        of adjacent nodes corrected by the offset defined          *)         
(*        in the routtab attribute.                                  *)         
(*                                                                   *)         
(*        For the format PATH the routing table is a table that      *)         
(*        looks like the output of the more or less known program    *)         
(*        pathalias. It contains lines that start with the           *)         
(*        nodename in question, followed by a horizontal tab         *)         
(*        character, followed by a list of nodes that form the       *)         
(*        path to that node. The list does not contain the own       *)         
(*        node but all direct neighbours.                            *)         
(*                                                                   *)         
(*DD-STATEMENTS/DATA SETS:                                           *)         
(*                                                                   *)         
(* SYSPRINT: will contain some processing log.                       *)         
(* COMMANDS: contains program directives.                            *)         
(* DATABASE: will contain a node information file                    *)         
(* ROUTETAB: will contain a routing table or CNTYSTAT list.          *)         
(* XMAILER : will contain an XMAILER NAMES file                      *)         
(*                                                                   *)         
(*                                                                   *)         
(*NORMAL EXIT:                                                       *)         
(* Standard PASCAL Return                                            *)         
(*                                                                   *)         
(*                                                                   *)         
(*ERROR EXIT:                                                        *)         
(*NONE                                                               *)         
(*                                                                   *)         
(*                                                                   *)         
(*RETURN CODES:                                                      *)         
(* 0     Function successful                                         *)         
(* 1     The GETNODES directive detected an invalid data base.       *)         
(* 2     An old GR version is used.                                  *)         
(* 3     The data base contains an invalid alias.                    *)         
(* 4     Node hash table overflow                                    *)         
(* 5     Data base containes too many networks.                      *)         
(* 6     An unknown directive was requested.                         *)         
(* 7     Missing directive handler (this should never occur).        *)         
(* 8     An invalid node was given on a directive.                   *)         
(* 9     A GETNODES directive was not used.                          *)         
(* 10    Only one GETNODES directive may be currently used.          *)         
(* 11    A topology error was detected.                              *)         
(* 12    There are too many value for the :type. attribute.          *)         
(* 13    An invalid Routtab tag was specified.                       *)         
(* 14    A GETNODES directive was given after a routing table gen.   *)         
(* 16    A PASCAL runtime error occured. Check your SYSPRINT.        *)         
(*                                                                   *)         
(*EXTERNAL ROUTINES:                                                 *)         
(* The standard PASCAL IO interface.                                 *)         
(* The TSO and CMS command interface routines.                       *)         
(*                                                                   *)         
(* The MVS version need the TSOLNK routine if the source is          *)         
(* compiled. This will probably found in SYS1.LPALIB.                *)         
(*                                                                   *)         
(* When you create the CMS module version you can ignore             *)         
(* error messages about the missing TSOLNK routine.                  *)         
(*                                                                   *)         
(*RELATED MATERIAL:                                                  *)         
(*                                                                   *)         
(* References for NJE routing tables:                                *)         
(*                                                                   *)         
(*    2 books for RSCS                                               *)         
(*    3 bools for JES2                                               *)         
(*                                                                   *)         
(*    MVS/Extended Architecture System Programming Library:          *)         
(*    JES2 Initialization and Tuning:                                *)         
(*    - MVS/System product - JES2 Version 2 Release 1.5              *)         
(*      SC23-0065-4                                                  *)         
(*    - MVS/System product - JES2 Version 2 Release 2.0              *)         
(*      SC23-0065-5                                                  *)         
(*                                                                   *)         
(*    MVS/SP-JES2                                                    *)         
(*    Version 3 Release 1.1                                          *)         
(*    JES2 Installation Considerations                               *)         
(*    GG24-3327-00                                                   *)         
(*                                                                   *)         
(*    1 book for JES3                                                *)         
(*                                                                   *)         
(*    VAX/VMS Jnet NJE manager's Guide, V3.3                         *)         
(*    Fourth Edition, May 1988                                       *)         
(*    Joiner Associates Inc, Madison Wisconsin                       *)         
(*    (I have pages 20-1 to 20-28).                                  *)         
(*                                                                   *)         
(*    Unix RSCS Emulation Progran (UREP)                             *)         
(*    Software Release 3.3                                           *)         
(*    Installation Instructions                                      *)         
(*                                                                   *)         
(* References for data base contents (BITEARN NODES):                *)         
(*                                                                   *)         
(*       EARN technical document NODETAGS DESCRIPT.                  *)         
(*       EARN document NODESFMT DESCRIPT (not yet finished).         *)         
(*                                                                   *)         
(*                                                                   *)         
(*STORAGE:                                                           *)         
(*  PROGRAM:  > one byte                                             *)         
(*  DYNAMIC:  Not yet calculated                                     *)         
(*                                                                   *)         
(*                                                                   *)         
(*LOAD MODULE CREATION:                                              *)         
(* SOURCE LANGUAGE      :  VSPASCAL / PASCALVS (ANSI83).             *)         
(*                                                                   *)         
(*  There are at least two PASCAL versions of the program. One       *)         
(*  is pure ANSI83 standard and another one uses a few extensions    *)         
(*  of the IBM PASCAL (to implement the TSO and CMS directives       *)         
(*  and the file specifications.)                                    *)         
(*                                                                   *)         
(*  I intend to port the PASCAL version to C using an automatic      *)         
(*  porting tool.                                                    *)         
(*                                                                   *)         
(* DESTINATION LIBRARIES:                                            *)         
(*                                                                   *)         
(*   The program and information is available in NETSERV:            *)         
(*                                                                   *)         
(*   GR       DESCRIPT      The program header                       *)         
(*   GRIBM    MODULE        A version for IBM VM CMS                 *)         
(*   GRIBM    OBJ           A version for IBM MVS                    *)         
(*   GRIBM    PASCAL        The source of the routines above         *)         
(*   GRANSI83 PASCAL        The ANSI 83 conformant subset.           *)         
(*   GR       C             A version of the program in C.           *)         
(*                                                                   *)         
(*                                                                   *)         
(* The MODULE and OBJ files are generated with the PASCAL/VS         *)         
(* compiler and runtime system.                                      *)         
(*                                                                   *)         
(* PROGRAM GENERATING:                                               *)         
(*                                                                   *)         
(*   GRIBM-MVS:                                                      *)         
(*                                                                   *)         
(*   If you want to generate your own MVS version from the source    *)         
(*   this can ne done with the following job control:                *)         
(*                                                                   *)         
(*      //GRIBM EXEC  PASCCL,                                        *)         
(*      //       GOSET='your.GR.LOAD',                               *)         
(*      //       GOMEM=GRIBM,                                        *)         
(*      //       LOPTS='LET,AMODE=31,RMODE=ANY,MAP'                  *)         
(*      //PASC.SYSIN DD DISP=SHR,DSN=your.GRIBM.PASCAL               *)         
(*      //LKED.SYSIN DD *                                            *)         
(*       INCLUDE TSOLNK(TSOLNK)                                      *)         
(*      //LKED.TSOLNK DD DISP=SHR,DSN=SYS1.LPALIB                    *)         
(*                                                                   *)         
(*   GRIBM-VM:                                                       *)         
(*                                                                   *)         
(*   Under VM the program can be generated by using the following    *)         
(*   CMS commands with VSPASCAL Rel. 2 (at least the book says so):  *)         
(*                                                                   *)         
(*      VSPASCAL GRIBM (OBJECT(GR)                                   *)         
(*      PASCMOD GR (NAME GRIBM                                       *)         
(*                                                                   *)         
(*   The reason for the OBJECT and NAME is that the pascal PROGRAM   *)         
(*   name is GR.                                                     *)         
(*                                                                   *)         
(*   The author has tried the following commands for VSPASCAL Rel.1: *)         
(*                                                                   *)         
(*      VSPASCAL GRIBM                                               *)         
(*      RENAME GRIBM TEXT A GR TEXT A                                *)         
(*      PASCMOD GR (NAME GRIBM                                       *)         
(*                                                                   *)         
(*   GRANSI83:                                                       *)         
(*                                                                   *)         
(*   I have no idea how the program can be generated on non IBM      *)         
(*   machines, on VM and MVS it's the same procedure as with         *)         
(*   GRIBM.                                                          *)         
(*                                                                   *)         
(*STATUS:                                                            *)         
(*                                                                   *)         
(*    Current version is 91-11-19                                    *)         
(*                                                                   *)         
(*    Jan.1990:                                                      *)         
(*        beta testing version.                                      *)         
(*                                                                   *)         
(*    Feb.1990:                                                      *)         
(*                                                                   *)         
(*        Bugs removed:                                              *)         
(*                                                                   *)         
(*          GR was looking for :noderest. instead of :noderesist.    *)         
(*          Error message numbers were undefined.                    *)         
(*          The restistance values of the old line types have        *)         
(*          been adjusted to the new numbers. This included          *)         
(*          an incompatible change for a CC.                         *)         
(*          JES3 table had blanks between name and ,                 *)         
(*          Network was not set for an alias.                        *)         
(*          A bug fix bug removed.                                   *)         
(*          JES2 comments were too long.                             *)         
(*                                                                   *)         
(*        New features:                                              *)         
(*                                                                   *)         
(*          Asterisk as placeholder for optional parameters.         *)         
(*          <grip> parameter added to GENROUTE directive.            *)         
(*          SERMON directive revised, DEBUG directive added.         *)         
(*          TSO and CMS directives recoded.                          *)         
(*          GRIBM now issues return codes.                           *)         
(*          NJEF lid logic added.                                    *)         
(*          SYSPRINT output revised (version ids added).             *)         
(*          IBM return codes added.                                  *)         
(*          JES2 number sort for JES2 tables added.                  *)         
(*          CNTYSTAT directive added.                                *)         
(*          Version test done only for the "digits".                 *)         
(*          Program name changed to GR.                              *)         
(*          More identification at the routing table start.          *)         
(*                                                                   *)         
(*    Mar.1990:                                                      *)         
(*                                                                   *)         
(*        Bugs removed:                                              *)         
(*                                                                   *)         
(*          The table generation info comment was too lomg for       *)         
(*          JES2.                                                    *)         
(*          The routtab syntax checks are changed. The changes       *)         
(*          revealed some invalid data in BITEARN NODES.             *)         
(*                                                                   *)         
(*        New features:                                              *)         
(*                                                                   *)         
(*          The routtab parsing now requires correct specification   *)         
(*          of formats. We recognized RSCS2.3 or JES2215 V2          *)         
(*          If invalid formats are given the result is undefined:    *)         
(*          Currently an RSCS table is generated.                    *)         
(*          The gettag routine checks for blanks in tag names.       *)         
(*          The CNTYSTAT directive output was revised to have        *)         
(*          more compatoibility with old programs.                   *)         
(*                                                                   *)         
(*        Known bugs:                                                *)         
(*                                                                   *)         
(*          The gettag routine doesn't seem to correctly unfold      *)         
(*          tags.                                                    *)         
(*          :type.NJE is not checked at all.                         *)         
(*                                                                   *)         
(*    Apr.1990:                                                      *)         
(*                                                                   *)         
(*        Bugs removed:                                              *)         
(*                                                                   *)         
(*          NJEFtable scanning thru LID associations was wrong.      *)         
(*          JES2 comments showed wrong node.                         *)         
(*                                                                   *)         
(*        New features:                                              *)         
(*                                                                   *)         
(*          The CNTYSTAT directive has been removed.                 *)         
(*          A new directive CTRYSTAT has been added.                 *)         
(*                                                                   *)         
(*    May.1990:                                                      *)         
(*                                                                   *)         
(*        Bugs removed:                                              *)         
(*                                                                   *)         
(*          If an invalid netsoft is given, a JES2 table was         *)         
(*          generated, changed to generate an RSCS table.            *)         
(*                                                                   *)         
(*          JES2 tables shown wrong node in comment.                 *)         
(*                                                                   *)         
(*          A via. to a node did not create a direct neighbour       *)         
(*          if there was an alternate path.                          *)         
(*                                                                   *)         
(*          The C version ported using the Glockenspiel T2C          *)         
(*          processor had lines longer than 255.                     *)         
(*          (The changes do not correct a bug.)                      *)         
(*                                                                   *)         
(*          Noderesist was not properly parsed.                      *)         
(*                                                                   *)         
(*    Jun.1990:                                                      *)         
(*                                                                   *)         
(*          JNET and RSCS tables format changed:                     *)         
(*                                                                   *)         
(*          All "rscs type" table are now produce the version        *)         
(*          entries followed by all entries in the same country      *)         
(*          followed by other nodes.                                 *)         
(*                                                                   *)         
(*          RSCS and JNET tables have network and country name       *)         
(*          in the ROUTE statements.                                 *)         
(*                                                                   *)         
(*          UREP tables were wrong.                                  *)         
(*                                                                   *)         
(*          And the "latest" loop. :-)                               *)         
(*                                                                   *)         
(*    Aug.1990:                                                      *)         
(*                                                                   *)         
(*          The maximum number of connections was too low.           *)         
(*          Some people tell that the network isn't growing.         *)         
(*                                                                   *)         
(*    Oct.1990:                                                      *)         
(*                                                                   *)         
(*          Tie break logic with node numbers added (to ensure       *)         
(*          symmetry of routes).                                     *)         
(*          Direct path preferes logic was wrong.                    *)         
(*          (No problem with the real network.)                      *)         
(*                                                                   *)         
(*          Program description revised.                             *)         
(*                                                                   *)         
(*    Mar.1991:                                                      *)         
(*                                                                   *)         
(*          When an adjtypes does not follow an adjnodes then        *)         
(*          some links are deleted. The 9102 BITEARN NODES files     *)         
(*          had four occurences of this.                             *)         
(*                                                                   *)         
(*          Direct path prefered rule still wrong. Can be bypassed   *)         
(*          in real network. Research ongoing.                       *)         
(*                                                                   *)         
(*          Table format PATH added.                                 *)         
(*                                                                   *)         
(*          Multiple GETNODES directives allowed for the IBM         *)         
(*          version, first occurence of an entry taken now.          *)         
(*          (Good luck, BITNET-2.)                                   *)         
(*                                                                   *)         
(*    Apr.1991:                                                      *)         
(*                                                                   *)         
(*          Multiple GETNODES don't work under CMS.                  *)         
(*          An uninitialized variable ==> GR assumed routtab NONE.   *)         
(*          Tagnames may now be in mixed case.                       *)         
(*          Multiple GENROUTE did no longer work.                    *)         
(*          Some changes on the PATH map.                            *)         
(*                                                                   *)         
(*    Jul.1991:                                                      *)         
(*                                                                   *)         
(*          Error message for node with no links and type not nje    *)         
(*          suppressed.                                              *)         
(*                                                                   *)         
(*          Support for Server type mail implemented.                *)         
(*                                                                   *)         
(*    --- released to NETSERV ---                                    *)         
(*                                                                   *)         
(*    Aug.1991:                                                      *)         
(*                                                                   *)         
(*          XMAILER NAMES with ? NOMAIL values for all non NJE       *)         
(*          node entries (still to be discussed                      *)         
(*                                                                   *)         
(*    Sep.1991:                                                      *)         
(*                                                                   *)         
(*          JES2 S=NUM was erroneous.                                *)         
(*          JES2 V4 added identical to V3.                           *)         
(*                                                                   *)         
(*    Oct.1991:                                                      *)         
(*                                                                   *)         
(*          Some bugs in the handling of noderesist and linkresist   *)         
(*          Default resistancies taken from version entry now.       *)         
(*                                                                   *)         
(*    Nov.1991:                                                      *)         
(*                                                                   *)         
(*          For CDC NTF the CTRYLIST file is generated.              *)         
(*                                                                   *)         
(*    --- released to NETSERV ---                                    *)         
(*                                                                   *)         
(*    Users on different systems are welcome to make tests,          *)         
(*    suggestions, report errors etc. to the author.                 *)         
(*                                                                   *)         
(*    Different routing table formats can be added if an             *)         
(*    appropriate specification is available.                        *)         
(*                                                                   *)         
(*                                                                   *)         
(*    Good luck.                                                     *)         
(*                                                                   *)         
(*NOTES:                                                             *)         
(*                                                                   *)         
(*    For MVS no invocation tools are available. The number of       *)         
(*    required actions is trivial compared to the possible           *)         
(*    variations to do that: It seems superfluous to me              *)         
(*    to write a CLIST, a JCL-PROC with a set of variables.          *)         
(*                                                                   *)         
(*    Under MVS the program may be called as a main program in       *)         
(*    batch mode if the directive TSO is not used.                   *)         
(*                                                                   *)         
(*    Caution: When you RECEIVE the GRIBM OBJ file                   *)         
(*    with TSO/E, don't forget to specify a parameter that           *)         
(*    matches the maximum BLKSIZE for the MVS linkage editor.        *)         
(*    (3200 is the value, isn't it.)                                 *)         
(*                                                                   *)         
(*    NETSERV does not send a BLKSIZE NETDATA tag in the             *)         
(*    file and the TSO/E RECEIVE default is a bad value.             *)         
(*                                                                   *)         
(*    I shall not create an invocation environment for CMS.          *)         
(*    This is left as a simple exercise to a CMS expert              *)         
(*    (as is done with UPDNODES).                                    *)         
(*    It seems that someone already did this, see NETSERV.           *)         
(*                                                                   *)         
(*EXAMPLES:                                                          *)         
(*                                                                   *)         
(*    Here is an MVS job step to invoke GR for one node:             *)         
(*                                                                   *)         
(*       //GR        EXEC  PGM=LOADER,PARM='NOMAP,NOLIST/'           *)         
(*       //SYSLIN    DD    DISP=SHR,DSN=prefix.GRIBM.OBJ             *)         
(*       //DATABASE  DD    DISP=SHR,DSN=prefix.BITEARN.NODES         *)         
(*       //SYSLOUT   DD    SYSOUT=*                                  *)         
(*       //SYSPRINT  DD    SYSOUT=*                                  *)         
(*       //ROUTETAB  DD    DISP=SHR,DSN=prefix.mynode.NETINIT        *)         
(*       //COMMANDS  DD *                                            *)         
(*       GETNODES                                                    *)         
(*       GENROUTE mynode                                             *)         
(*       //                                                          *)         
(*                                                                   *)         
(*    A primitive way to do the same under CMS would be to use       *)         
(*    the following way:                                             *)         
(*                                                                   *)         
(*    Use your favorite CMS editor and create a file                 *)         
(*    that contains the two lines                                    *)         
(*                                                                   *)         
(*       GETNODES                                                    *)         
(*       GENROUTE mynode                                             *)         
(*                                                                   *)         
(*    and name this file mynode COMMANDS, then the commands          *)         
(*                                                                   *)         
(*       FILEDEF SYSPRINT TERMINAL                                   *)         
(*       FILEDEF DATABASE DISK BITEARN NODES                         *)         
(*       FILEDEF ROUTETAB DISK mynode NETINIT                        *)         
(*       FILEDEF COMMANDS DISK mynode COMMANDS                       *)         
(*       GRIBM                                                       *)         
(*                                                                   *)         
(*    should give you the desired result (if BITEARN NODES is        *)         
(*    available!)                                                    *)         
(*                                                                   *)         
(*    Other people are smart, they don't need an example ?           *)         
(*                                                                   *)         
(*                                                                   *)         
(*AUTHOR:                                                            *)         
(*                                                                   *)         
(*    Peter Sylvester                                                *)         
(*                                                                   *)         
(*    Gesellschaft fuer Mathematik und Datenverarbeitung mbH         *)         
(*    Riemenschneiderstrasse 11                                      *)         
(*    W-5300 Bonn 2                                                  *)         
(*    Germany                                                        *)         
(*                                                                   *)         
(*   or                                                              *)         
(*                                                                   *)         
(*    EARN Office Paris                                              *)         
(*    CIRCE BP 167                                                   *)         
(*    91403 Orsay CEDEX                                              *)         
(*    France                                                         *)         
(*                                                                   *)         
(*    BITNET:    GRZ027@DBNGMD21                                     *)         
(*    Internet:  GRZ027@MVS.GMD.DE                                   *)         
(*    DFN-X.400: S=GRZ027;OU=MVS;P=GMD;A=DBP;C=DE                    *)         
(*                                                                   *)         
(*CREDITS:                                                           *)         
(*                                                                   *)         
(*    Daniele Bovio created many test cases for GR.                  *)         
(*                                                                   *)         
(*    Stefan Fassbender suggested many implementation details.       *)         
(*                                                                   *)         
(*    Ed Skochiski tested early versions of the program on a CDC     *)         
(*    system and provided the information for NJEF tables.           *)         
(*                                                                   *)         
(*    Wolfgang Strobl was very helpful porting the program to C.     *)         
(*                                                                   *)         
(*                                                                   *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
PROGRAM gr                                                                      
  (database,                                                                    
   sysprint,                                                                    
   commands,                                                                    
   routetab,                                                                    
   xmailer  ) ;                                                                 
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   A bug is a bug, never try to recover (only in GRIBM).           *)         
(*                                                                   *)         
(*********************************************************************)         
%INCLUDE ONERROR;                                                               
PROCEDURE Onerror ;                                                             
BEGIN                                                                           
  Faction := Faction + (.XHALT.)                                                
END ;                                                                           
(*********************************************************************)         
(*                                                                   *)         
(*   Ok, let us define the rules of the game...                      *)         
(*                                                                   *)         
(*********************************************************************)         
LABEL 9999 ;                                                                    
CONST isnodatabase          = 1 ;                                               
      programstupid         = 2 ;                                               
      invalidalias          = 3 ;                                               
      nodehashoverflow      = 4 ;                                               
      toomanynetworks       = 5 ;                                               
      unknowndirective      = 6 ;                                               
      directivenotsupported = 7 ;                                               
      invalidnode           = 8 ;                                               
      nodatabase            = 9 ;                                               
      morethanonedatabase   = 10;                                               
      treeerror             = 11;                                               
      toomanytypes          = 12;                                               
      invalidrouttab        = 13;                                               
      databaselate          = 14;                                               
      myhighesterror        = 14;                                               
                                                                                
      thousand       = 1000 ;                                                   
      million        = 1000000 ;                                                
      billion        = 1000000000 ;                                             
      stringpoolsize = 50000 ;                                                  
      heapsize       = 20000 ;                                                  
      maxnodes       = 5123 ;                                                   
      maxconn        = 33000 ;    (* 4*maxnodes + some *)                       
      maxnetworks    = 100 ;                                                    
      maxtypes       = 20  ;                                                    
                                                                                
      maxcommandops=90;                                                         
      tagvaluesize = 260 ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   The following values are defined to keep some compatibility     *)         
(*   of new format resistancies with old format adjacency info.      *)         
(*   The values are not identical to the values used in the old      *)         
(*   program. They have been adjusted to the new environment.        *)         
(*                                                                   *)         
(*   The differences are:                                            *)         
(*                                                                   *)         
(*   The CC resistance is now set to 1.                              *)         
(*   The ZL-1L are now the correct figures. If there are two paths   *)         
(*   with equal resistance, the old program used the path with       *)         
(*   a smaller number of nodes.                                      *)         
(*   The values include a 1 for a node resistance.                   *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
      resistanceundefined  = -1 ;                                               
      resistanceCC         = 1000   ;                                           
      resistanceZL         = 15626  ;                                           
      resistance9L         = 104167 ;                                           
      resistance4L         = 208334 ;                                           
      resistance2L         = 416667 ;                                           
      resistance1L         = 833334 ;                                           
                                                                                
      resistanceVia        = 104167 ;                                           
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   The next constants define the type of link. We don't use this   *)         
(*   information so far.                                             *)         
(*   There is a possibility that a dial up line could close a ring   *)         
(*   and then .... no strategy has been defined for this link.       *)         
(*                                                                   *)         
(*********************************************************************)         
      permanentlink        = 0 ;                                                
      dialuplink           = -1 ;                                               
      virtualonelink       = 1 ;                                                
                                                                                
                                                                                
TYPE                                                                            
  NETWORKID = 0..maxnetworks ;                                                  
  TYPEID    = 0..maxtypes ;                                                     
  NODEID    = 0..maxnodes ;                                                     
  NODEARRAY =                                                                   
  PACKED                                                                        
  ARRAY(.NODEID.) OF NODEID ;                                                   
(*********************************************************************)         
(*                                                                   *)         
(*   The following type is used to categorize tags.                  *)         
(*                                                                   *)         
(*********************************************************************)         
  ATTRIBUTE =                                                                   
     (notinteresting,                                                           
     AttrNode,     (* both                                         *)           
     AttrAdjnodes, (* old                                          *)           
     AttrAdjtypes, (* old                                          *)           
     AttrAlias,    (* old                                          *)           
     AttrAliasnum, (* old                                          *)           
     AttrCountry,  (* both                                         *)           
     AttrGenrouts, (* both                                         *)           
     AttrLinks,    (* new                                          *)           
     AttrMailer,   (* old                                          *)           
     AttrNet,      (* both                                         *)           
     AttrNetsoft,  (* both                                         *)           
     AttrNodenum,  (* both                                         *)           
     AttrNoderesist, (* new                                        *)           
     AttrRouttab,  (* new                                          *)           
     Attrservers,  (* new                                          *)           
     AttrType,     (* new                                          *)           
     AttrVia ) ;   (* old                                          *)           
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   A TYPE for directives.                                          *)         
(*                                                                   *)         
(*********************************************************************)         
  DIRECTIVE =                                                                   
     (DirComment,                                                               
      DirUnknown,                                                               
      DirGetnodes,                                                              
      DirXMAILER,                                                               
      DirGenroute,                                                              
      DirCtryStat,                                                              
      DirGenPaths,                                                              
      DirTSOCommand,                                                            
      DirCMSCommand);                                                           
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   The various routing table format that we know.                  *)         
(*                                                                   *)         
(*********************************************************************)         
  ROUTTABFORMAT =                                                               
    (unknownformat,                                                             
     noneformat,                                                                
     formatnotdone,                                                             
     rscsformat,                                                                
     jes2format,                                                                
     jes2asmformat, jes2pliformat, jes2esaformat,                               
     jes3format,                                                                
     jnetformat,                                                                
     urepformat,                                                                
     njefformat,                                                                
     ntfformat,                                                                 
     pathformat) ;                                                              
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   Some like different orders.                                     *)         
(*                                                                   *)         
(*********************************************************************)         
  SORTTYPE =                                                                    
    (sortasinput, (* list nodes as in GETNODES input         *)                 
     sortjes2) ;  (* list nodes in JES2 node number sequence *)                 
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   A type that is a left over from initial coding. I'm too lazy    *)         
(*   to change this into two arrays (or to change other arrays       *)         
(*   into parts of this type.                                        *)         
(*                                                                   *)         
(*********************************************************************)         
  NODENTRY =                                                                    
    RECORD                                                                      
      name       :INTEGER  ;               (* index into string pool *)         
      links      :INTEGER  ;               (* index into connections *)         
    END ;                                                                       
                                                                                
  TAGLINE  = PACKED ARRAY (.0..tagvaluesize.) OF CHAR ;                         
                                                                                
VAR                                                                             
                                                                                
  tagnamelength  : INTEGER ;                                                    
  tagvaluelength : INTEGER ;                                                    
  cmdvaluelength : INTEGER ;                                                    
                                                                                
  genroutsversion : ARRAY (.0..8.) OF CHAR ;                                    
                                                                                
  tagvalue :        TAGLINE ;                                                   
  cmdvalue :        TAGLINE ;                                                   
  tagname  :        ARRAY (.0..64.) OF CHAR ;                                   
                                                                                
  database : TEXT ;                                                             
  nodesok  : BOOLEAN ;                                                          
  nodesverified : BOOLEAN ;                                                     
  upperminuslower : INTEGER ;                                                   
  ebcdic   : BOOLEAN ;                                                          
  htab     : CHAR ;                                                             
  prologuenotdone : BOOLEAN ;                                                   
  linksnode :     INTEGER ;                                                     
  versnode  :     INTEGER ;                                                     
  defaultnoderesist : INTEGER ;                                                 
  defaultlinkrest : INTEGER ;                                                   
                                                                                
  sysprint : TEXT ;                                                             
                                                                                
  commandoptions : INTEGER;                                                     
  commandop : ARRAY(.0..maxcommandops.) OF INTEGER;                             
                                                                                
  commands : TEXT ;                                                             
  commandop1 : INTEGER ;                                                        
  commandop1len : INTEGER ;                                                     
  commandop2 : INTEGER ;                                                        
  commandop2len : INTEGER ;                                                     
                                                                                
  routetab : TEXT ;                                                             
  firstroutedone : BOOLEAN ;                                                    
  asmsyntax  : BOOLEAN ;                                                        
  startofcomment : CHAR ;                                                       
                                                                                
  xmailer  : TEXT ;                                                             
  genxmailer : BOOLEAN ;                                                        
  xmtype,xmnode,xmaliaslength,                                                  
  xmnetsoftlength,xmmailerlength :  INTEGER ;                                   
  xmalias,xmnetsoft,xmmailer: TAGLINE ;                                         
                                                                                
                                                                                
  currentoffset  : INTEGER ;                                                    
                                                                                
  strings  : PACKED ARRAY(.0..stringpoolsize.) OF CHAR ;                        
                                                                                
  tagoccurrence  : INTEGER ;                                                    
                                                                                
  majortag,majortaglength : INTEGER ;                                           
                                                                                
  i : INTEGER ;                                                                 
  j : INTEGER ;                                                                 
                                                                                
  numberofnetworks : INTEGER ;                                                  
  numberoftypes    : INTEGER ;                                                  
  njetype          : INTEGER ;                                                  
  numberofnodes    : INTEGER ;                                                  
  sortednode       : INTEGER ;                                                  
  lidlists         : INTEGER ;                                                  
  lidassociations: INTEGER ;                                                    
                                                                                
  stringfree : INTEGER ;                                                        
                                                                                
  linkfree : INTEGER ;                                                          
  maxjes2num : INTEGER ;                                                        
  inventedjes2num : INTEGER ;                                                   
                                                                                
                                                                                
  sermon   : BOOLEAN ;                                                          
  debug    : BOOLEAN ;                                                          
                                                                                
  networks : ARRAY(.0..maxnetworks.) OF INTEGER ;                               
  types    : ARRAY(.0..maxtypes.) OF INTEGER ;                                  
  links    : ARRAY(.0..maxconn.) OF INTEGER ;                                   
  heap     : ARRAY(.0..heapsize.) OF INTEGER  ;                                 
                                                                                
  nodes    : ARRAY(.NODEID.) OF NODENTRY ;                                      
  sorted   : NODEARRAY ;                                                        
  jes2nodes: ARRAY(.NODEID.) OF INTEGER ;                                       
  sorttypes  : PACKED ARRAY(.NODEID.) OF SORTTYPE ;                             
  currentsort : SORTTYPE ;                                                      
                                                                                
  spanningheap   : NODEARRAY;                                                   
  dists          : ARRAY(.NODEID.) OF INTEGER       ;                           
  paths          : NODEARRAY ;                                                  
  via            : NODEARRAY ;                                                  
  marked         : ARRAY(.NODEID.) OF BOOLEAN       ;                           
 sources         : ARRAY(.NODEID.) OF BOOLEAN       ;                           
  isbettercall   : INTEGER ;                                                    
                                                                                
                                                                                
  jes2numbers    : ARRAY(.NODEID.) OF INTEGER ;                                 
  routtabformats : PACKED ARRAY(.NODEID.) OF ROUTTABFORMAT ;                    
  offsets        : ARRAY(.NODEID.) OF INTEGER ;                                 
                                                                                
  domain   : PACKED ARRAY(.NODEID.) OF RECORD                                   
                                    c1 : CHAR ;                                 
                                    c2 : CHAR ;                                 
                                    network : NETWORKID ;                       
                                    entrytype  :TYPEID  ;                       
                                  END ;                                         
                                                                                
  base36tochar : ARRAY(.0..35.) OF CHAR ;                                       
  chartobase36 : ARRAY(.0..255.) OF INTEGER ;                                   
                                                                                
  VAR                                                                           
    date,dateiso,time : ALFA ;                                                  
PROCEDURE setbase36(decimal:INTEGER;base36c:CHAR);                              
BEGIN                                                                           
  base36tochar(.decimal.) := base36c ;                                          
  chartobase36(.Ord(base36c).) := decimal ;                                     
  chartobase36(.Ord(base36c)+Ord('a')-Ord('A').) := decimal                     
END (* base36 *) ;                                                              
                                                                                
PROCEDURE initbase36;                                                           
VAR i:INTEGER ;                                                                 
BEGIN                                                                           
  FOR i := 0 TO 255 DO                                                          
  chartobase36(.i.) := -1 ;                                                     
                                                                                
  setbase36(00,'0'); setbase36(01,'1'); setbase36(02,'2');                      
  setbase36(03,'3'); setbase36(04,'4'); setbase36(05,'5');                      
  setbase36(06,'6'); setbase36(07,'7'); setbase36(08,'8');                      
  setbase36(09,'9'); setbase36(10,'A'); setbase36(11,'B');                      
  setbase36(12,'C'); setbase36(13,'D'); setbase36(14,'E');                      
  setbase36(15,'F'); setbase36(16,'G'); setbase36(17,'H');                      
  setbase36(18,'I'); setbase36(19,'J'); setbase36(20,'K');                      
  setbase36(21,'L'); setbase36(22,'M'); setbase36(23,'N');                      
  setbase36(24,'O'); setbase36(25,'P'); setbase36(26,'Q');                      
  setbase36(27,'R'); setbase36(28,'S'); setbase36(29,'T');                      
  setbase36(30,'U'); setbase36(31,'V'); setbase36(32,'W');                      
  setbase36(33,'X'); setbase36(34,'Y'); setbase36(35,'Z');                      
                                                                                
END (* initbase36 *) ;                                                          
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   Write a four digit number.                                      *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putintrt(i : INTEGER);                                                
VAR c : CHAR ;                                                                  
BEGIN                                                                           
                                                                                
  IF i < 0 THEN                                                                 
  BEGIN                                                                         
     write(routetab,'-','1')                                                    
  END                                                                           
  ELSE                                                                          
  BEGIN                                                                         
  write(routetab,Chr(Ord('0')+i DIV 1000));                                     
  i := i MOD 1000 ;                                                             
  write(routetab,Chr(Ord('0')+i DIV  100));                                     
  i := i MOD  100 ;                                                             
  write(routetab,Chr(Ord('0')+i DIV  10));                                      
  i := i MOD   10 ;                                                             
  write(routetab,Chr(Ord('0')+i     ))                                          
  END                                                                           
                                                                                
END (* putintrt *) ;                                                            
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   An empty line or a comment line.                                *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE emptyline ;                                                           
BEGIN                                                                           
  IF asmsyntax THEN Writeln(routetab,startofcomment)                            
  ELSE Writeln(routetab,' ')                                                    
END (* emptyline *) ;                                                           
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(* This routines are either dummy (ANSI83) or a call to Close (IBM). *)         
(* It is used to close the ROUTTAB dataset after creating it in      *)         
(* order to be able to manipulate it with a CMS/TSO command.         *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE closeroutetab ;                                                       
BEGIN                                                                           
    Close(routetab) ;                                                           
END (* closeroutetab *)  ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(* This routine is only used in the IBM version. It creates a        *)         
(* string from a tag value. A single asterisk is replaced by an      *)         
(* empty string. The function is intended to be used as a parameter  *)         
(* for a file specification for the PASCAL Rewrite/Reset routines.   *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION tagvaluetostr(start:INTEGER;endpos:INTEGER):STRING(255);               
VAR i:INTEGER;                                                                  
    s:STRING(255) ;                                                             
BEGIN                                                                           
  s := '' ;                                                                     
  FOR i:=start TO endpos DO                                                     
  s := s || str(tagvalue(.i.));                                                 
  IF s = '*' THEN                                                               
    tagvaluetostr := ''                                                         
  ELSE                                                                          
    tagvaluetostr := s                                                          
END ;                                                                           
(*********************************************************************)         
(*                                                                   *)         
(*  A little utility to simplify some tagvalue parsing.              *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE eattagop(operands :INTEGER) ;                                         
VAR i,j,prefix: INTEGER;                                                        
BEGIN                                                                           
  IF operands > commandoptions + 1 THEN                                         
    operands := commandoptions + 1 ;                                            
                                                                                
  prefix:= commandop(.operands.) - 1;                                           
                                                                                
  FOR  i := 0 TO commandoptions DO                                              
    commandop(.i.) := commandop(.i+operands.) - j ;                             
  commandoptions := commandoptions - operands ;                                 
  j := 1;                                                                       
  FOR  i := prefix+1 TO tagvaluelength DO                                       
  BEGIN                                                                         
    tagvalue(.j.) := tagvalue(.i.);                                             
    j := j + 1                                                                  
  END ;                                                                         
  FOR  i := j + 1 TO tagvaluelength DO tagvalue(.i.) := ' ';                    
  tagvaluelength := tagvaluelength - prefix                                     
END (* eattagop *) ;                                                            
(*********************************************************************)         
(*                                                                   *)         
(*   This routine executes a CMS command (IBM).                      *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION CMScommand : INTEGER ;                                                 
VAR returncode : INTEGER ;                                                      
  cmdstr : string(255) ;                                                        
                                                                                
PROCEDURE cms(CONST s : STRING; VAR returncode: INTEGER); external ;            
BEGIN                                                                           
  returncode := -1 ;                                                            
  eattagop(1);                                                                  
  WHILE tagvalue(.tagvaluelength.) = ' ' DO                                     
    tagvaluelength:= tagvaluelength - 1;                                        
  cms(tagvaluetostr(1,tagvaluelength),returncode) ;                             
  IF (returncode <> 0) OR debug THEN BEGIN                                      
    Writeln(sysprint);                                                          
    Writeln(sysprint,'CMS command returned with ',returncode:4)                 
  END ;                                                                         
  CMScommand := returncode ;                                                    
                                                                                
END (* CMScommand *)  ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   Execute a TSO command or clist. TSO/E rel 4. is a prereq.       *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION TSOcommand : INTEGER ;                                                 
                                                                                
VAR                                                                             
  functionflags    : INTEGER ;                                                  
  lastcc           : INTEGER;                                                   
  reasoncode       : INTEGER;                                                   
  abendcode        : INTEGER;                                                   
PROCEDURE tsolnk( VAR functionflags : INTEGER;                                  
                  VAR command       : CHAR ;                                    
                  VAR commandlength : INTEGER;                                  
                  VAR lastcc        : INTEGER;                                  
                  VAR reasoncode    : INTEGER;                                  
                  VAR abendcode     : INTEGER); FORTRAN;                        
BEGIN                                                                           
                                                                                
   functionflags :=  0*256+      (* 1=take dump, 0 = don't take dump *)         
                     5      ;    (* 1=TSO cmd, 2=pgm , 5 = clists/cmd *)        
   lastcc := -1;                                                                
   reasoncode := 0;                                                             
   abendcode  := 0;                                                             
                                                                                
   eattagop(1);                                                                 
   WHILE tagvalue(.tagvaluelength.) = ' ' DO                                    
     tagvaluelength:= tagvaluelength - 1;                                       
                                                                                
   tsolnk(functionflags,                                                        
          tagvalue(.1.),                                                        
          tagvaluelength,                                                       
          lastcc,                                                               
          reasoncode,                                                           
          abendcode);                                                           
   IF (lastcc <> 0) OR debug THEN                                               
   BEGIN                                                                        
     Writeln(sysprint,'TSO command returned with ',lastcc:4,                    
         ',reasoncode: ',reasoncode:5,' abendcode: ',abendcode:4)               
   END ;                                                                        
   TSOcommand := lastcc                                                         
END (* TSOcommand *)  ;                                                         
                                                                                
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Some error messages. This is rather poor so far, if more         *)         
(*  validation is necessary, some extra tests and messages are       *)         
(*  needed.                                                          *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putmsg(messagenumber:INTEGER) ;                                       
  VAR i,k: INTEGER ;                                                            
  BEGIN                                                                         
    Write(sysprint,messagenumber:5,': ') ;                                      
    IF (messagenumber > 0) AND (messagenumber <= myhighesterror) THEN           
    CASE messagenumber OF                                                       
      isnodatabase         :Writeln(sysprint,                                   
        'The GETNODES directive detected an invalid data base. ');              
      programstupid        :Writeln(sysprint,                                   
        'An old GR version is used. ');                                         
      invalidalias         :Writeln(sysprint,                                   
        'The data base contains an invalid alias. ');                           
      nodehashoverflow     :Writeln(sysprint,                                   
        'Node hash table overflow ');                                           
      toomanynetworks      :Writeln(sysprint,                                   
        'Data base containes too many networks. ');                             
      unknowndirective     :Writeln(sysprint,                                   
        'An unknown directive was requested. ');                                
      directivenotsupported:Writeln(sysprint,                                   
        'Missing directive handler (this should never occur). ');               
      invalidnode          :Writeln(sysprint,                                   
        'An invalid node was given on a directive. ');                          
      nodatabase           :Writeln(sysprint,                                   
        'A GETNODES directive was not used. ');                                 
      morethanonedatabase  :Writeln(sysprint,                                   
        'Only one GETNODES directive may be currently used.');                  
      treeerror            :Writeln(sysprint,                                   
        'A topology error was detected. ');                                     
      toomanytypes         :Writeln(sysprint,                                   
        'Data base contains too many types. ');                                 
      invalidrouttab       :Writeln(sysprint,                                   
        'Routtab value is incorrect. ');                                        
      databaselate         :Writeln(sysprint,                                   
        'GETNODES used after routing table directive.');                        
    END                                                                         
    ELSE                                                                        
      Writeln(sysprint,'Internal error, unknown error message. ');              
  END (* putmsg *) ;                                                            
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Fatal error handler:                                             *)         
(*  put out a message number and terminate the program.              *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE fatalerror(messagenumber:INTEGER) ;                                   
  BEGIN                                                                         
    Write(sysprint,'Fatal error:') ;                                            
    putmsg(messagenumber);                                                      
    Writeln(sysprint,'GR terminates.') ;                                        
    retcode(messagenumber) ;                                                    
    GOTO 9999                                                                   
  END (* fatalerror *) ;                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  String handling is not supported in ANSI83                       *)         
(*  As we only need strings in some global stringpool, and to        *)         
(*  get them either from a tagname or tagvalue, we have special      *)         
(*  routines for copying strings.                                    *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
PROCEDURE stringfromname(target:INTEGER) ;                                      
  VAR i:INTEGER ;                                                               
  BEGIN                                                                         
    FOR i := 1 TO tagnamelength DO                                              
    BEGIN                                                                       
      strings(.target.) := tagname(.i.) ;                                       
      target := Succ(target)                                                    
    END                                                                         
  END (* stringfromname *) ;                                                    
                                                                                
FUNCTION stringfromvalue(start:INTEGER ;len:INTEGER):INTEGER ;                  
  VAR i,j:INTEGER ;                                                             
  BEGIN                                                                         
    stringfromvalue := stringfree ;                                             
    strings(.stringfree.) := chr(len - 1) ;                                     
    j := start + len - 1 ;                                                      
    FOR i := start TO j DO                                                      
    BEGIN                                                                       
      stringfree := Succ(stringfree) ;                                          
      strings(.stringfree.) := tagvalue(.i.)                                    
    END ;                                                                       
    stringfree:= Succ(stringfree) ;                                             
  END (* stringfromvalue *) ;                                                   
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  We want to compare values (like nodenames) with the contents     *)         
(*  of the string pool.                                              *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION stringequalvalue(str:INTEGER ;                                         
         start:INTEGER ; len:INTEGER):BOOLEAN ;                                 
  LABEL 9 ;                                                                     
  VAR i,j:INTEGER ;                                                             
  BEGIN                                                                         
    stringequalvalue := False ;                                                 
    len := Pred(len) ;                                                          
    IF (Ord(strings(.str.)) = len) THEN                                         
    BEGIN                                                                       
      j := start + len ;                                                        
      FOR i := start TO j DO                                                    
      BEGIN                                                                     
        str := Succ(str) ;                                                      
        IF  strings(.str.) <> tagvalue(.i.) THEN GOTO 9                         
      END ;                                                                     
      stringequalvalue := True                                                  
    END ;                                                                       
    9:                                                                          
  END (* stringequalvalue *) ;                                                  
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  A variable length string from the pool is sent to a file         *)         
(*  and padded with blanks.      .                                   *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putv(str: INTEGER;VAR tofile:TEXT;pad:INTEGER);                       
  VAR i,j: INTEGER ;                                                            
  BEGIN                                                                         
    j := Ord(strings(.str.)) + 1 + str;                                         
    FOR i := str+1 TO j DO Write(tofile,strings(.i.)) ;                         
    IF j < str+pad THEN                                                         
      FOR j := j+1 TO str+pad DO Write(tofile,' ')                              
  END (* putv *) ;                                                              
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  A variable length string from the pool is sent to a file         *)         
(*  and truncated or padded.                                          *)        
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putvtrunc(str: INTEGER;VAR tofile:TEXT;pad:INTEGER);                  
  VAR i,j: INTEGER ;                                                            
  BEGIN                                                                         
    j := Ord(strings(.str.)) + 1 + str;                                         
    IF j > str+pad THEN j := str+pad;                                           
    FOR i := str+1 TO j DO Write(tofile,strings(.i.)) ;                         
  END (* putvtrunc *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Almost the same thing but all upper case characters are          *)         
(*  translated to lower case.                                        *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putvlc(str: INTEGER;VAR tofile:TEXT;pad:INTEGER);                     
  VAR i,j: INTEGER ; c: CHAR ;                                                  
  BEGIN                                                                         
    j := Ord(strings(.str.)) + 1 + str;                                         
    FOR i := str+1 TO j DO                                                      
    BEGIN                                                                       
      c := strings(.i.) ;                                                       
      IF (c >= 'A') AND (c <= 'Z') THEN                                         
         c := Chr(Ord(c)-upperminuslower) ;                                     
      Write(tofile,c) ;                                                         
    END ;                                                                       
    IF j < str+pad THEN                                                         
      FOR j := j+1 TO str+pad DO Write(tofile,' ')                              
  END (* putvlc *) ;                                                            
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  There are two formats of comments in a routing table. Either     *)         
(*  assembler syntax for many tables and PLI for some JES2 tables.   *)         
(*  so comments are a bit different. The JNET comment char also      *)         
(*  differs from the IBM *                                           *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE begincomment ;                                                        
  BEGIN                                                                         
    IF NOT asmsyntax THEN Write(routetab,'/');                                  
    Write(routetab,startofcomment,' ') ;                                        
  END (* begincomment *) ;                                                      
                                                                                
PROCEDURE endcomment ;                                                          
  BEGIN                                                                         
    IF NOT asmsyntax THEN                                                       
       Write(routetab,' */')                                                    
  END (* begincomment *) ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  These utilities makes the coding a bit more readable.            *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE prtnode(node : INTEGER;pad:INTEGER);                                  
BEGIN                                                                           
  putv(nodes(.node.).name,routetab,pad)                                         
END (* prtnode *);                                                              
                                                                                
PROCEDURE prtnodelc(node : INTEGER;pad:INTEGER);                                
BEGIN                                                                           
  putvlc(nodes(.node.).name,routetab,pad)                                       
END (* prtnode *);                                                              
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function is called after a positioning to a new record      *)         
(*  to database, i.e. after the first reset or some Readln.          *)         
(*  It returns True if the next character on input is a start of     *)         
(*  new tagname. The routines skips thru continuation lines and      *)         
(*  empty lines.                                                     *)         
(*                                                                   *)         
(*  I'm not sure whether comments or empty lines are allowed         *)         
(*  between continuation lines but at least they don't hurt and      *)         
(*  it doesn't make much sense not to skip them.                     *)         
(*                                                                   *)         
(*  Mr. Jackson please close your eyes.                              *)         
(*                                                                   *)         
(*  The routine should be considered as an internal routine of       *)         
(*  initdatabase and gettag. These routines will be available in     *)         
(*  assembler. The pascal version needs 16 seconds on a 3090/150E    *)         
(*  for all tags in BITEARN NODES and it seems possible that this    *)         
(*  value can be reduced very much.                                  *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION isnewtagname : BOOLEAN ;                                               
  LABEL 1,9 ;                                                                   
  VAR                                                                           
    currentchar  : CHAR ;                                                       
    currentxindex : INTEGER ;                                                   
  BEGIN                                                                         
    isnewtagname := False ;                                                     
    currentchar := ' ' ;                                                        
  1:WHILE (NOT Eof(database)) DO                                                
    BEGIN                                                                       
      IF Eoln(database) THEN BEGIN                                              
         Readln(database) ;                                                     
         GOTO 1                                                                 
      END ;                                                                     
      Read(database,currentchar) ;                                              
      IF currentchar = '*' THEN Readln(database)                                
      ELSE BEGIN                                                                
        WHILE (currentchar = ' ') DO                                            
        BEGIN                                                                   
          IF Eoln(database) THEN BEGIN                                          
             Readln(database) ;                                                 
             GOTO 1                                                             
          END ;                                                                 
          Read(database,currentchar) ;                                          
        END ;                                                                   
        isnewtagname := (currentchar = ':') ;                                   
        GOTO 9                                                                  
      END                                                                       
    END ;                                                                       
9:END (* isnewtagname *) ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This routine initializes the database. The file is opened and    *)         
(*  the start of the first tagname is searched.                      *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE initdatabase ;                                                        
  BEGIN                                                                         
    IF commandop1len > 0 THEN                                                   
      Reset(database,                                                           
            tagvaluetostr(commandop1,commandop1+commandop1len-1))               
    ELSE                                                                        
    Reset(database) ;                                                           
    IF NOT isnewtagname THEN fatalerror(isnodatabase)                           
  END (* initdatabase *) ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This gets a tagname.                                             *)         
(*                                                                   *)         
(*  The routine sets 4 variables. The tag name and the tag value are *)         
(*  set into tagname, tagnamelength, tagvalue, and tagvaluelength.   *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
PROCEDURE gettag ;                                                              
  LABEL 1,9 ;                                                                   
  VAR                                                                           
    currentchar  : CHAR ;                                                       
    previouswasblank : BOOLEAN ;                                                
  BEGIN                                                                         
    tagnamelength := 0 ;                                                        
    tagvaluelength := 0 ;                                                       
    IF NOT Eof(database) THEN                                                   
    BEGIN                                                                       
      Read(database,currentchar);                                               
      WHILE  currentchar <> '.' DO                                              
      BEGIN                                                                     
        IF currentchar = ' ' THEN fatalerror(isnodatabase);                     
        tagnamelength := Succ(tagnamelength) ;                                  
        IF (currentchar >= 'A') AND (currentchar <= 'Z') THEN                   
        currentchar := Chr(Ord(Currentchar)-upperminuslower) ;                  
        tagname(.tagnamelength.) := currentchar ;                               
        Read(database,currentchar);                                             
      END ;                                                                     
                                                                                
      previouswasblank := False ;                                               
                                                                                
    1:WHILE (NOT Eof(database)) DO                                              
      BEGIN                                                                     
        WHILE NOT Eoln(database) DO                                             
        BEGIN                                                                   
          Read(database,currentchar);                                           
          IF previouswasblank AND (currentchar = ':') THEN GOTO 9 ;             
          previouswasblank := currentchar = ' ';                                
          tagvaluelength := Succ(tagvaluelength) ;                              
          IF (currentchar >= 'a') AND (currentchar <= 'z') THEN                 
          currentchar := Chr(Ord(Currentchar)+upperminuslower) ;                
          tagvalue(.tagvaluelength .) := currentchar ;                          
        END ;                                                                   
        Readln(database) ;                                                      
        IF NOT isnewtagname  THEN                                               
        BEGIN                                                                   
          tagvaluelength := Succ(tagvaluelength);                               
          tagvalue(.tagvaluelength .) := ' ' ;                                  
          GOTO 1                                                                
        END ;                                                                   
        GOTO 9                                                                  
      END ;                                                                     
   9:                                                                           
      WHILE (tagvaluelength > 0)                                                
        AND (tagvalue(.tagvaluelength.) = ' ') DO                               
      tagvaluelength := Pred(tagvaluelength) ;                                  
      tagvalue(.tagvaluelength+1.) := ' '                                       
    END ;                                                                       
  END (* gettag *) ;                                                            
                                                                                
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  A debugging tool used in the putpaths directive.                 *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putnodename(node:INTEGER;VAR tofile:TEXT) ;                           
  BEGIN                                                                         
    IF nodes(.node.).name > 0 THEN                                              
    BEGIN                                                                       
      Write(tofile,' ');                                                        
      putv(nodes(.node.).name,tofile,9);                                        
      WITH domain(.node.) DO BEGIN                                              
        IF debug THEN                                                           
           Write(tofile,network,' ') ;                                          
        IF network > 0 THEN                                                     
          putv(networks(.network.),tofile,9)                                    
        ELSE Write(tofile,'* net *  ') ;                                        
        IF entrytype > 0 THEN                                                   
          putv(types(.entrytype.),tofile,9)                                     
        ELSE Write(tofile,'* type * ') ;                                        
        Write(tofile,c1,c2) ;                                                   
      END ;                                                                     
      Write(tofile,jes2numbers(.node.));                                        
      IF debug THEN                                                             
        Write(tofile,node)                                                      
    END                                                                         
  END (* putnodename *) ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function reads a nodes database and extract tags.           *)         
(*  The first tag name that is found in the database is used         *)         
(*  as a naming tag.                                                 *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE initmajorname ;                                                       
  VAR i:INTEGER ;                                                               
  BEGIN                                                                         
    majortag := stringfree ;                                                    
    majortaglength := tagnamelength ;                                           
    stringfromname(stringfree) ;                                                
    stringfree := stringfree + majortaglength ;                                 
  END (* initmajorname *) ;                                                     
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  See whether the next tag is a major tag.                         *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION ismajortag : BOOLEAN ;                                                 
  LABEL 9 ;                                                                     
  VAR i,mti:INTEGER ;                                                           
  BEGIN                                                                         
    ismajortag := False ;                                                       
    mti := majortag ;                                                           
    IF tagnamelength = majortaglength THEN                                      
    BEGIN                                                                       
      FOR i := 1 TO majortaglength DO                                           
      BEGIN                                                                     
        IF strings(.mti.) <> tagname(.i.) THEN GOTO 9 ;                         
          mti:= Succ(mti);                                                      
      END ;                                                                     
      ismajortag := True                                                        
    END ;                                                                       
9:END (* ismajortag *) ;                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Get tagname modifier. Multiple tags are distinguished by         *)         
(*  numbers.                                                         *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getintfromtag(pos:INTEGER) : BOOLEAN ;                                 
  LABEL 9 ;                                                                     
  VAR i,j : INTEGER ;                                                           
      c : CHAR    ;                                                             
  BEGIN                                                                         
    tagoccurrence := 0 ;                                                        
    getintfromtag := False ;                                                    
    FOR  i := pos TO tagnamelength DO                                           
    BEGIN                                                                       
      c := tagname(.i.) ;                                                       
      IF (c >= '0') AND (c <= '9') THEN                                         
        tagoccurrence := 10 * tagoccurrence + ORD(c) - ORD('0')                 
      ELSE GOTO 9                                                               
    END ;                                                                       
  getintfromtag := True ;                                                       
9:END (* getintfromtag *) ;                                                     
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function uses a subset of the tagvalue as a name to allocate*)         
(*  a string in the string pool and a nodename pointer in the        *)         
(*  nodes table.                                                     *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION findnode(start:INTEGER ;len:INTEGER):INTEGER ;                         
  LABEL 8,9 ;                                                                   
  VAR i,j,hash          :INTEGER ;                                              
      c1,c3,c5,c6 : INTEGER ;                                                   
  BEGIN                                                                         
    IF len > 5 THEN BEGIN                                                       
      c1 := Ord(tagvalue(.start+1.)) ;                                          
      c3 := Ord(tagvalue(.start+3.)) ;                                          
      c5 := Ord(tagvalue(.start+5.)) ;                                          
      IF len > 6 THEN c6 := Ord(tagvalue(.start+6.)) ELSE c6 := 0               
    END ELSE BEGIN                                                              
       c6 := 0 ; c5 := 0 ;                                                      
       IF len <= 3 THEN BEGIN                                                   
         c3 := 0 ;                                                              
         IF len > 1 THEN c1 := Ord(tagvalue(.start+1.)) ELSE c1 := 0            
       END ELSE BEGIN                                                           
         c3 := Ord(tagvalue(.start+3.)) ;                                       
         c1 := Ord(tagvalue(.start+1.))                                         
       END                                                                      
    END ;                                                                       
                                                                                
    hash := ((c5 + 243*c6 + 3017*c3 + 6013*c1) MOD heapsize)+1 ;                
    i := hash ;                                                                 
    WHILE i <= heapsize DO                                                      
    BEGIN                                                                       
      j := heap(.i.) ;                                                          
      IF j = 0 THEN GOTO 8 ;                                                    
      IF stringequalvalue(nodes(.j.).name,start,len)                            
         THEN GOTO 9 ;                                                          
      i := Succ(i);                                                             
    END ;                                                                       
    i := 1 ;                                                                    
    WHILE i <= hash DO                                                          
    BEGIN                                                                       
      j := heap(.i.) ;                                                          
      IF j = 0 THEN GOTO 8 ;                                                    
      j := nodes(.j.).name ;                                                    
      IF stringequalvalue(nodes(.j.).name,start,len)                            
         THEN GOTO 9 ;                                                          
      i := Succ(i) ;                                                            
    END ;                                                                       
                                                                                
    fatalerror(nodehashoverflow) ;                                              
  8:numberofnodes := numberofnodes + 1 ;                                        
    heap(.i.) := numberofnodes ;                                                
    j := numberofnodes ;                                                        
    nodes(.j.).name := stringfromvalue(start,len) ;                             
  9:findnode := j ;                                                             
  END (* findnode *) ;                                                          
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function allocates a number to a network name.              *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION findnetwork(start:INTEGER ;len:INTEGER):NETWORKID ;                    
  LABEL 9 ;                                                                     
  VAR i :NETWORKID ;                                                            
  BEGIN                                                                         
    i := 1 ;                                                                    
    WHILE i <= numberofnetworks DO                                              
    BEGIN                                                                       
      j := networks(.i.) ;                                                      
      IF stringequalvalue(j,start,len) THEN GOTO 9 ;                            
      i := Succ(i);                                                             
    END ;                                                                       
                                                                                
    IF i > maxnetworks THEN fatalerror(toomanynetworks);                        
    networks(.i.) := stringfromvalue(start,len) ;                               
    numberofnetworks := i ;                                                     
  9:findnetwork := i                                                            
  END (* findnetwork *) ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function allocates a number to a TYPE.                      *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION findtype(start:INTEGER ;len:INTEGER):TYPEID ;                          
  LABEL 9 ;                                                                     
  VAR i :TYPEID ;                                                               
  BEGIN                                                                         
    i := 1 ;                                                                    
    WHILE i <= numberoftypes DO                                                 
    BEGIN                                                                       
      j := types(.i.) ;                                                         
      IF stringequalvalue(j,start,len) THEN GOTO 9 ;                            
      i := Succ(i);                                                             
    END ;                                                                       
                                                                                
    IF i > maxtypes THEN fatalerror(toomanytypes);                              
    types(.i.) := stringfromvalue(start,len) ;                                  
    numberoftypes := i ;                                                        
  9:findtype := i                                                               
  END (* findtype *) ;                                                          
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  The following functions are used while parsing the NODES format. *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  The function matches a non negative integer. On invalid input    *)         
(*  -1 will be returned.                                             *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getintFromValue(pos:INTEGER;endpos:INTEGER) : INTEGER ;                
  LABEL 9 ;                                                                     
  VAR i,j,tv : INTEGER ;                                                        
      c : CHAR    ;                                                             
  BEGIN                                                                         
    getintfromvalue := - 1 ;                                                    
    tv := 0 ;                                                                   
    FOR  i := pos TO endpos DO                                                  
    BEGIN                                                                       
      c := tagvalue(.i.) ;                                                      
      IF (c >= '0') AND (c <= '9') THEN                                         
        tv := 10 * tv + ORD(c) - ORD('0')                                       
      ELSE GOTO 9                                                               
    END ;                                                                       
  getintfromvalue := tv ;                                                       
9:END (* getintfromvalue *) ;                                                   
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  The function matches a big number. This is a non negative        *)         
(*  number that may have the suffixes M or K to indicate scalars     *)         
(*  of 1000000 or 1000 resp. The number may contain fractionals      *)         
(*  which are used only up to a precision of 0.001.                  *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getbigintFromValue(pos:INTEGER;                                        
                            endpos:INTEGER):INTEGER ;                           
  LABEL 9 ;                                                                     
  VAR i,j,tv : INTEGER ;                                                        
      c : CHAR    ;                                                             
      modifier,ep : INTEGER ;                                                   
      rightofdot : BOOLEAN ;                                                    
  BEGIN                                                                         
    getbigintfromvalue := -1 ;                                                  
    rightofdot := False ;                                                       
    tv := 0 ;                                                                   
    IF (tagvalue(.endpos.) = 'K')                                               
    OR (tagvalue(.endpos.) = 'k') THEN                                          
    BEGIN                                                                       
      modifier := 1000 ;                                                        
      IF endpos = pos THEN tv := 1 ;                                            
      ep := endpos - 1;                                                         
    END                                                                         
    ELSE IF (tagvalue(.endpos.) = 'M')                                          
         OR (tagvalue(.endpos.) = 'm') THEN                                     
    BEGIN                                                                       
      modifier := million ;                                                     
      IF endpos = pos THEN tv := 1 ;                                            
      ep := endpos - 1;                                                         
    END                                                                         
    ELSE BEGIN                                                                  
      ep := endpos ;                                                            
      modifier := 1 ;                                                           
    END ;                                                                       
    FOR i := pos to ep DO                                                       
    BEGIN                                                                       
      c := tagvalue(.i.) ;                                                      
      IF c = '.' THEN                                                           
      BEGIN                                                                     
        IF rightofdot THEN GOTO 9 ;                                             
        rightofdot := True                                                      
      END                                                                       
      ELSE IF (c < '0') OR (c > '9') THEN GOTO 9                                
      ELSE IF NOT rightofdot THEN                                               
        tv := 10 * tv + ORD(c) - ORD('0')                                       
      ELSE IF modifier > 1 THEN                                                 
      BEGIN                                                                     
        modifier := modifier DIV 10 ;                                           
        tv := 10 * tv + ORD(c) - ORD('0')                                       
      END                                                                       
    END ;                                                                       
    IF tv >  million / modifier THEN                                            
       getbigintfromvalue := million                                            
    ELSE                                                                        
       getbigintfromvalue := tv * modifier ;                                    
9:END (* getintfromvalue *) ;                                                   
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  The function matches number that is implcitely scaled by 1000.   *)         
(*  A dot may separate a fractional part.                            *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getKintFromValue(pos:INTEGER;                                          
                          endpos:INTEGER):INTEGER ;                             
  LABEL 9 ;                                                                     
  VAR i,j,tv : INTEGER ;                                                        
      c : CHAR    ;                                                             
      modifier : INTEGER ;                                                      
      rightofdot : BOOLEAN ;                                                    
  BEGIN                                                                         
    rightofdot := False ;                                                       
    getKintfromvalue := -1 ;                                                    
    tv := 0 ;                                                                   
    modifier := 1000 ;                                                          
    FOR  i := pos TO endpos DO                                                  
    BEGIN                                                                       
      c := tagvalue(.i.) ;                                                      
      IF c = '.' THEN                                                           
      BEGIN                                                                     
        IF rightofdot THEN GOTO 9 ;                                             
        rightofdot := True                                                      
      END                                                                       
      ELSE IF (c < '0') OR (c > '9') THEN GOTO 9                                
      ELSE IF NOT rightofdot THEN                                               
        tv := 10 * tv + ORD(c) - ORD('0')                                       
      ELSE IF modifier > 1 THEN                                                 
      BEGIN                                                                     
        tv := 10 * tv + ORD(c) - ORD('0') ;                                     
        modifier := modifier DIV 10                                             
      END                                                                       
    END ;                                                                       
  getKintfromvalue := tv * modifier ;                                           
9:END (* getKintfromvalue *) ;                                                  
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses a JES2 node number. As a side effect the    *)         
(*  highest used JES2 nodenumber is calculated.                      *)         
(*  A result -1 from getintfromvalue is allowed. This may happen     *)         
(*  when a node number attribute contains PEND as a value.           *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getnodenum(pos:INTEGER;endpos:INTEGER) : INTEGER ;                     
  VAR i : INTEGER ;                                                             
  BEGIN                                                                         
    i := getintfromvalue(pos,endpos);                                           
    IF maxjes2num < i THEN maxjes2num := i ;                                    
    getnodenum := i                                                             
  END (* getnodenum *) ;                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function calculates a resistance value for the old format   *)         
(*  adjtype.                                                         *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION calcrest(start:INTEGER ;len:INTEGER):INTEGER ;                         
  VAR t1,t2 :CHAR ;                                                             
  BEGIN                                                                         
    calcrest := resistanceundefined ;                                           
    IF len <= 2 THEN                                                            
    BEGIN                                                                       
      t1 := tagvalue(.start.) ;                                                 
      t2 := tagvalue(.start+1.) ;                                               
      IF (t2 = 'L') OR (t2 = 'S') OR (t2 = 'P') OR (t2 = 'X') THEN              
      BEGIN                                                                     
        IF t1 = '9' THEN                                                        
          calcrest := resistance9L                                              
        ELSE IF t1 = 'Z' THEN                                                   
          calcrest := resistanceZL                                              
        ELSE IF t1 = '4' THEN                                                   
          calcrest := resistance4L                                              
        ELSE IF t1 = '2' THEN                                                   
          calcrest := resistance2L                                              
        ELSE IF t1 = '1' THEN                                                   
          calcrest := resistance1L                                              
      END                                                                       
      ELSE IF (t1 = 'C') AND (t2 = 'C') THEN                                    
        calcrest := resistancecc                                                
    END                                                                         
  END (* calcrest *) ;                                                          
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses the remainder of a Netsoft tag for JES2.    *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION jes2netsoft : ROUTTABFORMAT ;                                          
  VAR i : INTEGER ;                                                             
  BEGIN                                                                         
    jes2netsoft := jes2pliformat ;                                              
    i := 5 ;                                                                    
    WHILE (i < tagvaluelength) AND (tagvalue(.i.) = ' ') DO                     
      i := i+1 ;                                                                
    IF (tagvaluelength > 4)                                                     
    AND (tagvalue(.i.) = '3') THEN                                              
      jes2netsoft := jes2esaformat                                              
    ELSE IF (i + 4) <= tagvaluelength THEN                                      
    BEGIN                                                                       
      IF tagvalue(.i.) = '1' THEN                                               
      BEGIN                                                                     
        IF tagvalue(.i+4.) <> '6' THEN                                          
          jes2netsoft := jes2asmformat                                          
      END                                                                       
      ELSE IF tagvalue(.i.) = '2' THEN                                          
      BEGIN                                                                     
        IF (tagvalue(.i+2.) = '1')                                              
        AND (tagvalue(.i+4.) < '5') THEN                                        
          jes2netsoft := jes2asmformat                                          
      END                                                                       
    END                                                                         
  END (* jes2netsoft *) ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses the remainder of a routtab for JES2.        *)         
(*  A side effect the global variable currentoffset will be set      *)         
(*  to the value of the optional O= operand, and the currentsort     *)         
(*  will be set to sortjes2num if S=NUM was detected.                *)         
(*                                                                   *)         
(*       JES2 <" "* version > <" "* jes2parm* >                      *)         
(*       version  : V1 | V2 | V3 | V4                                *)         
(*       jes2parm : offset | sort                                    *)         
(*       offset   : "O=" <number>                                    *)         
(*       sort     : "S=" <NUM"                                       *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION jes2routtab : ROUTTABFORMAT ;                                          
  VAR i,j : INTEGER ;                                                           
  BEGIN                                                                         
    jes2routtab := jes2pliformat ;                                              
    i := 5 ;                                                                    
    WHILE (i < tagvaluelength) AND (tagvalue(.i.) = ' ') DO                     
      i := i+1 ;                                                                
    IF (tagvaluelength > i) THEN                                                
    BEGIN                                                                       
      IF  (tagvalue(.i.) = 'V') THEN                                            
      BEGIN                                                                     
        IF  (tagvalue(.i+1.) = '2') THEN                                        
        ELSE IF  (tagvalue(.i+1.) = '1') THEN                                   
          jes2routtab := jes2asmformat                                          
        ELSE IF (tagvalue(.i+1.) = '3') THEN                                    
          jes2routtab := jes2esaformat                                          
        ELSE IF (tagvalue(.i+1.) = '4') THEN                                    
          jes2routtab := jes2esaformat ;                                        
        i := i + 2 ;                                                            
      END ;                                                                     
    END ;                                                                       
    j := i;                                                                     
    WHILE (i < tagvaluelength) DO                                               
    BEGIN                                                                       
      WHILE (i < tagvaluelength) AND (tagvalue(.i.) = ' ') DO                   
        i := i+1 ;                                                              
      IF (tagvaluelength > i) THEN BEGIN                                        
        j := i;                                                                 
        IF  (tagvalue(.i.) = 'O')                                               
        AND (tagvalue(.i+1.) = '=') THEN                                        
        BEGIN                                                                   
          currentoffset := 0 ;                                                  
          j := i + 2 ;                                                          
          WHILE (j <= tagvaluelength)                                           
            AND (tagvalue(.j.) >= '0')                                          
            AND (tagvalue(.j.) <= '9') DO                                       
            j := j+1 ;                                                          
          currentoffset := getintfromvalue(i+2,j-1)                             
        END                                                                     
        ELSE IF  (tagvalue(.i.) = 'S')                                          
        AND (tagvalue(.i+1.) = '=') THEN                                        
        BEGIN                                                                   
          currentsort := sortasinput ;                                          
          j := i + 2 ;                                                          
          WHILE (j <= tagvaluelength)                                           
            AND (tagvalue(.j.) <> ' ') DO                                       
            j := j+1 ;                                                          
          IF i+5 = j THEN                                                       
          BEGIN                                                                 
            IF  (tagvalue(.i+2.) = 'N')                                         
            AND (tagvalue(.i+3.) = 'U')                                         
            AND (tagvalue(.i+4.) = 'M')                                         
            THEN                                                                
              currentsort := sortjes2                                           
          END                                                                   
        END                                                                     
        ELSE WHILE (j <= tagvaluelength)                                        
        AND (tagvalue(.j.) <> ' ') DO                                           
          j := j+1 ;                                                            
      END ;                                                                     
      i := j                                                                    
    END ;                                                                       
                                                                                
  END (* jes2routtab *) ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  These routines parse the remainder of a routtab for NJEF.        *)         
(*  As a side effect the LidLists are updated.                       *)         
(*                                                                   *)         
(*  Optional parameters are node=lid associations. The associations  *)         
(*  will also be stored in the connection table (from the end),      *)         
(*  The are basically a list of pairs (node,lid), where node         *)         
(*  is either the internal id, or the negative JES2 number of the    *)         
(*  node. Each list of pairs is started by a pair (node,number)      *)         
(*  containing the internal id of the node that has this list        *)         
(*  in its routtab, and the number of (node,lid) associations.       *)         
(*                                                                   *)         
(*       NJEF <" "* node "=" lid>*                                   *)         
(*       node     : name or number                                   *)         
(*       lid      : three characters                                 *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getlid(startpos:INTEGER;endpos:INTEGER):INTEGER ;                      
VAR l1,l2,l3 : INTEGER ;                                                        
BEGIN                                                                           
  getlid := -1 ;                                                                
  IF startpos+2 = endpos THEN                                                   
                                                                                
  BEGIN                                                                         
    l1 := chartobase36(.Ord(tagvalue(.startpos.)).);                            
    l2 := chartobase36(.Ord(tagvalue(.startpos+1.)).);                          
    l3 := chartobase36(.Ord(tagvalue(.startpos+2.)).);                          
    IF (l1 >= 0) AND (l2 >= 0) AND (l3 >= 0) THEN                               
      getlid := (l1*36 + l2)*36 +l3                                             
  END ;                                                                         
END (* getlid *) ;                                                              
                                                                                
PROCEDURE NJEFrouttab;                                                          
VAR i,j,nodenum,lid :INTEGER ;                                                  
  BEGIN                                                                         
    i := 5 ;                                                                    
    WHILE (i < tagvaluelength) DO                                               
    BEGIN                                                                       
      WHILE (i < tagvaluelength) AND (tagvalue(.i.) = ' ') DO                   
        i := i+1 ;                                                              
      IF (tagvaluelength > i) THEN                                              
      BEGIN                                                                     
        j := i + 1;                                                             
        WHILE (j < tagvaluelength) AND (tagvalue(.j.) <> '=') DO                
          j := j + 1 ;                                                          
        nodenum := -getintfromvalue(i,j-1);                                     
        IF nodenum > 0 THEN                                                     
          nodenum := findnode(i,j-i) ;                                          
        i := j + 1 ;                                                            
        WHILE (i <= tagvaluelength) AND (tagvalue(.i.) <> ' ') DO               
          i := i+1 ;                                                            
        lidlists := lidlists - 2;                                               
        links(.lidlists.) := nodenum ;                                          
        links(.lidlists+1.) := getlid(j+1,i-1);                                 
        lidassociations := lidassociations + 1 ;                                
      END                                                                       
    END ;                                                                       
    IF lidassociations > 0 THEN                                                 
    BEGIN                                                                       
      lidlists := lidlists - 2;                                                 
      links(.lidlists+1.) := lidassociations                                    
    END ;                                                                       
  END (* NJEFrouttab *) ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses the first part of the routtab tag.          *)         
(*                                                                   *)         
(*   routtab : grip <" " parameters> <<" "*> "(" other-parameters>   *)         
(*                                                                   *)         
(*   grip :  RSCS  | JES3  | JNET  | UREP | JES2 | NJEF              *)         
(*                                                                   *)         
(*  The routine first looks for the occurance of a open bracket (    *)         
(*  and sets the tag value length to the position before.            *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION doroutbas : ROUTTABFORMAT ;                                            
  VAR rt : ROUTTABFORMAT ;                                                      
      i  : INTEGER ;                                                            
  BEGIN                                                                         
    i := 1;                                                                     
    WHILE (i <= tagvaluelength) AND (tagvalue(.i.) <> '(') DO                   
      i := i+1 ;                                                                
    tagvaluelength := i-1;                                                      
    WHILE (tagvaluelength > 0)                                                  
      AND (tagvalue(.tagvaluelength.) = ' ') DO                                 
      tagvaluelength := tagvaluelength-1 ;                                      
                                                                                
    rt := unknownformat;                                                        
    IF tagvaluelength = 3 THEN                                                  
    BEGIN                                                                       
      IF  (tagvalue(.1.) = 'N')                                                 
      AND (tagvalue(.2.) = 'T')                                                 
      AND (tagvalue(.3.) = 'F')                                                 
      THEN rt := ntfformat                                                      
    END                                                                         
    ELSE IF tagvaluelength >= 4 THEN                                            
    BEGIN                                                                       
      IF tagvalue(.1.) = 'R' THEN                                               
      BEGIN                                                                     
        IF  (tagvalue(.2.) = 'S')                                               
        AND (tagvalue(.3.) = 'C')                                               
        AND (tagvalue(.4.) = 'S')                                               
        THEN rt := rscsformat                                                   
      END                                                                       
      ELSE IF tagvalue(.2.) = 'E' THEN                                          
      BEGIN                                                                     
        IF  (tagvalue(.1.) = 'J')                                               
        AND (tagvalue(.3.) = 'S')                                               
        THEN BEGIN                                                              
          IF  tagvalue(.4.) = '3' THEN                                          
            rt := jes3format                                                    
          ELSE IF  tagvalue(.4.) = '2' THEN                                     
            rt := jes2format                                                    
        END                                                                     
      END                                                                       
      ELSE IF tagvalue(.2.) = 'N' THEN                                          
      BEGIN                                                                     
        IF  (tagvalue(.1.) = 'J')                                               
        AND (tagvalue(.3.) = 'E')                                               
        AND (tagvalue(.4.) = 'T')                                               
        THEN rt := jnetformat                                                   
      END                                                                       
      ELSE IF tagvalue(.1.) = 'U' THEN                                          
      BEGIN                                                                     
        IF  (tagvalue(.2.) = 'R')                                               
        AND (tagvalue(.3.) = 'E')                                               
        AND (tagvalue(.4.) = 'P')                                               
        THEN rt := urepformat                                                   
      END                                                                       
      ELSE IF tagvalue(.1.) = 'P' THEN                                          
      BEGIN                                                                     
        IF  (tagvalue(.2.) = 'A')                                               
        AND (tagvalue(.3.) = 'T')                                               
        AND (tagvalue(.4.) = 'H')                                               
        THEN rt := pathformat                                                   
      END                                                                       
      ELSE IF tagvalue(.1.) = 'N' THEN                                          
      BEGIN                                                                     
        IF  (tagvalue(.2.) = 'J') THEN                                          
        BEGIN                                                                   
          IF  (tagvalue(.3.) = 'E')                                             
          AND (tagvalue(.4.) = 'F')                                             
          THEN rt := njefformat                                                 
        END                                                                     
        ELSE IF  (tagvalue(.2.) = 'O')                                          
        AND (tagvalue(.3.) = 'N')                                               
        AND (tagvalue(.4.) = 'E')                                               
        THEN rt := noneformat                                                   
      END                                                                       
    END ;                                                                       
                                                                                
    IF tagvaluelength > 4 THEN                                                  
    BEGIN                                                                       
      IF (tagvalue(.5.) <> ' ') THEN rt := unknownformat                        
    END ;                                                                       
                                                                                
    doroutbas := rt                                                             
  END (* doroutbas *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses a netsoft tag. A routing table format       *)         
(*  is detected and for JES2 the detailed routine JES2netsoft        *)         
(*  is called for further differentiation.                           *)         
(*                                                                   *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION donetsoft : ROUTTABFORMAT ;                                            
  VAR rt : ROUTTABFORMAT ;                                                      
  BEGIN                                                                         
    rt := doroutbas ;                                                           
    IF rt = jes2format THEN                                                     
       donetsoft := jes2netsoft                                                 
    ELSE                                                                        
       donetsoft := rt                                                          
  END (* donetsoft *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses a routtab tag. A routing table format       *)         
(*  is detected and for JES2 the detailed routine JES2routtab        *)         
(*  is called for further differentiation.                           *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION dorouttab : ROUTTABFORMAT ;                                            
  VAR rt : ROUTTABFORMAT ;                                                      
  BEGIN                                                                         
    lidassociations := 0 ;                                                      
    rt := doroutbas  ;                                                          
    IF rt = njefformat THEN njefrouttab                                         
    ELSE IF rt = jes2format THEN                                                
       rt := jes2routtab ;                                                      
    IF rt = unknownformat THEN                                                  
    BEGIN                                                                       
      IF debug THEN                                                             
        putmsg(invalidrouttab) ;                                                
      dorouttab := rscsformat                                                   
    END                                                                         
    ELSE                                                                        
      dorouttab := rt                                                           
  END (* dorouttab *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This procedure dumps one tag to the XMAILER file.                *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putxmailertag(currentattribute:ATTRIBUTE;node:INTEGER);               
VAR i,j,k,l,mtype: INTEGER ;                                                    
    c : CHAR ;                                                                  
BEGIN                                                                           
  IF genxmailer THEN                                                            
  BEGIN                                                                         
    IF currentattribute = Attrservers THEN                                      
    BEGIN                                                                       
      i := 1;                                                                   
      WHILE (i < tagvaluelength) DO                                             
      BEGIN                                                                     
        WHILE (tagvalue(.i.) = ' ') DO i := Succ(i) ;                           
        j := i;                                                                 
        REPEAT                                                                  
          i := Succ(i) ;                                                        
          c := tagvalue(.i.) ;                                                  
        UNTIL (i = tagvaluelength)                                              
          OR (c = ' ') OR (c = '(') ;                                           
        IF (i = tagvaluelength) THEN k := i ELSE                                
        k := i - 1;                                                             
        IF (c = '(') THEN                                                       
        BEGIN                                                                   
          WHILE (i < tagvaluelength) AND (c <> ')') DO                          
          BEGIN                                                                 
            i := Succ(i) ;                                                      
            c := tagvalue(.i.)                                                  
          END ;                                                                 
          IF i = tagvaluelength THEN i:=Succ(i) ;                               
          l := k+6 ;                                                            
          IF (l <= i) THEN                                                      
          BEGIN                                                                 
            IF  (tagvalue(.k+2.) = 'M')                                         
            AND (tagvalue(.k+3.) = 'A')                                         
            AND (tagvalue(.k+4.) = 'I')                                         
            AND (tagvalue(.k+5.) = 'L')                                         
            AND ((tagvalue(.l.) = ',')                                          
             OR  (tagvalue(.l.) = ')'))                                         
            THEN                                                                
            BEGIN                                                               
              mtype := 3 ;                                                      
              l := succ(l) ;                                                    
              WHILE (l < i)                                                     
              AND (tagvalue(.l.) <> ',') DO                                     
                l := succ(l) ;                                                  
              IF l < i THEN                                                     
                l := succ(l) ;                                                  
              WHILE (l < i)                                                     
              AND (tagvalue(.l.) <> ',') DO                                     
                l := succ(l) ;                                                  
              IF l < i THEN                                                     
              BEGIN                                                             
              c := tagvalue(.succ(l).) ;                                        
                IF c = 'B' THEN                                                 
                ELSE IF c = 'O' THEN mtype := 1                                 
                ELSE IF c = 'R' THEN mtype := 2                                 
              END ;                                                             
              IF xmtype >= mtype THEN                                           
              BEGIN                                                             
                Writeln(sysprint,'Multiple mailer definitions for ');           
                putnodename(node,sysprint) ;                                    
                Writeln(sysprint);                                              
                FOR j := j TO k DO                                              
                  Write(sysprint,tagvalue(.j.));                                
                Writeln(sysprint,' ignored.');                                  
              END ELSE BEGIN                                                    
                IF xmmailerlength > 0 THEN                                      
                BEGIN                                                           
                  Writeln(sysprint,'Multiple mailer definitions for ');         
                  putnodename(node,sysprint) ;                                  
                  Writeln(sysprint);                                            
                  FOR l := 1 TO xmmailerlength DO                               
                    Write(sysprint,xmmailer(.l.));                              
                  Writeln(sysprint,' ignored.');                                
                END ;                                                           
                xmtype := mtype ;                                               
                xmmailerlength := 0 ;                                           
                FOR j := j TO k DO                                              
                BEGIN                                                           
                  xmmailerlength := xmmailerlength + 1 ;                        
                  xmmailer(.xmmailerlength.) := tagvalue(.j.)                   
                END                                                             
              END                                                               
            END                                                                 
          END                                                                   
        END ;                                                                   
        i := succ(i);                                                           
      END                                                                       
    END                                                                         
    ELSE IF currentattribute = attrnode THEN                                    
    BEGIN                                                                       
      IF domain(.xmnode.).entrytype = njetype THEN                              
      BEGIN                                                                     
        Write(xmailer,':nick.') ;                                               
        putv(nodes(.xmnode.).name,xmailer,0);                                   
        Write(xmailer,' :alias.') ;                                             
        IF xmaliaslength > 0 THEN                                               
          FOR i := 1 TO xmaliaslength DO                                        
            Write(xmailer,xmalias(.i.))                                         
        ELSE                                                                    
        putv(nodes(.xmnode.).name,xmailer,0);                                   
                                                                                
        Write(xmailer,' :mailer.') ;                                            
        IF xmmailerlength > 0 THEN                                              
          FOR i:=1 TO xmmailerlength DO                                         
            Write(xmailer,xmmailer(.i.));                                       
        IF xmtype = 3 THEN                                                      
          Write(xmailer,' BSMTP 3')                                             
        ELSE IF xmtype = 1 THEN                                                 
          Write(xmailer,' DEFRT 1 TRUNCATE')                                    
        ELSE IF xmtype = 2 THEN                                                 
          Write(xmailer,' BITNET 2') ;                                          
                                                                                
        Write(xmailer,' :net.') ;                                               
        putv(networks(.domain(.xmnode.).network.),xmailer,0) ;                  
                                                                                
        Write(xmailer,' :netsoft.') ;                                           
        If xmnetsoftlength > 0 THEN                                             
          FOR i := 1 TO xmnetsoftlength DO                                      
            Write(xmailer,xmnetsoft(.i.));                                      
        Writeln(xmailer,' ');                                                   
      END ;                                                                     
      xmmailerlength := 0 ;                                                     
      xmaliaslength := 0 ;                                                      
      xmnetsoftlength := 0 ;                                                    
      xmnode    := node  ;                                                      
      xmtype    := 0                                                            
    END                                                                         
    ELSE IF currentattribute = attrnetsoft THEN                                 
    BEGIN                                                                       
      xmnetsoftlength := tagvaluelength ;                                       
      FOR i := 1 to tagvaluelength DO                                           
        xmnetsoft(.i.) := tagvalue(.i.)                                         
    END                                                                         
    ELSE IF currentattribute = attralias   THEN                                 
    BEGIN                                                                       
      xmaliaslength := tagvaluelength ;                                         
      FOR i := 1 to tagvaluelength DO                                           
        xmalias(.i.) := tagvalue(.i.)                                           
    END                                                                         
    ELSE IF xmmailerlength > 0 THEN                                             
    BEGIN                                                                       
      Writeln(sysprint,'Multiple mailer definitions for ');                     
      putnodename(node,sysprint) ;                                              
      Writeln(sysprint);                                                        
      FOR j := 1 TO tagvaluelength DO                                           
         Write(sysprint,tagvalue(.j.));                                         
      Writeln(sysprint,' ignored.')                                             
    END                                                                         
    ELSE                                                                        
    BEGIN                                                                       
      FOR i := 1 TO tagvaluelength DO                                           
        xmmailer(.i.) := tagvalue(.i.) ;                                        
      xmmailerlength := i                                                       
    END                                                                         
  END                                                                           
END (* putxmailertag *) ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(* This routine matches tag names and turns them into an ATTRIBUTE.  *)         
(* The ugly layout is a result of an attempt to find a minimal       *)         
(*                                                                   *)         
(* We do not use some table driven logic. The reader should assume   *)         
(* that some parser generator could have generated the following.    *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION categorizetag : ATTRIBUTE ;                                            
                                                                                
  VAR c:CHAR ;                                                                  
      i:INTEGER ;                                                               
  BEGIN                                                                         
    categorizetag := notinteresting ;                                           
    tagoccurrence := 0 ;                                                        
    c := tagname(.1.) ;                                                         
    IF c = 'a' THEN                                                             
    BEGIN                                                                       
      IF tagnamelength = 5 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'l')                                                
        AND (tagname(.3.) = 'i')                                                
        AND (tagname(.4.) = 'a')                                                
        AND (tagname(.5.) = 's') THEN                                           
          categorizetag := attralias                                            
      END                                                                       
      ELSE IF tagnamelength = 8 THEN                                            
      BEGIN                                                                     
        IF  (tagname(.2.) = 'l')                                                
        AND (tagname(.3.) = 'i')                                                
        AND (tagname(.4.) = 'a')                                                
        AND (tagname(.5.) = 's')                                                
        AND (tagname(.6.) = 'n')                                                
        AND (tagname(.7.) = 'u')                                                
        AND (tagname(.8.) = 'm') THEN                                           
          categorizetag := attraliasnum                                         
      END                                                                       
      ELSE IF (tagnamelength > 8)                                               
          AND (tagname(.2.) = 'd')                                              
          AND (tagname(.3.) = 'j')                                              
          AND (tagname(.7.) = 'e')                                              
          AND (tagname(.8.) = 's') THEN                                         
      BEGIN                                                                     
        IF  (tagname(.4.) = 'n')                                                
        AND (tagname(.5.) = 'o')                                                
        AND (tagname(.6.) = 'd') THEN                                           
        BEGIN                                                                   
          IF getintfromtag(9) THEN                                              
          categorizetag := attradjnodes                                         
        END                                                                     
        ELSE                                                                    
        IF  (tagname(.4.) = 't')                                                
        AND (tagname(.5.) = 'y')                                                
        AND (tagname(.6.) = 'p') THEN                                           
        BEGIN                                                                   
          IF getintfromtag(9) THEN                                              
          categorizetag := attradjtypes                                         
        END                                                                     
      END                                                                       
    END                                                                         
    ELSE IF c = 'c' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength = 7 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'o')                                                
        AND (tagname(.3.) = 'u')                                                
        AND (tagname(.4.) = 'n')                                                
        AND (tagname(.5.) = 't')                                                
        AND (tagname(.6.) = 'r')                                                
        AND (tagname(.7.) = 'y') THEN                                           
          categorizetag := attrcountry                                          
      END                                                                       
    END                                                                         
    ELSE IF c = 'g' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength = 8 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'e')                                                
        AND (tagname(.3.) = 'n')                                                
        AND (tagname(.4.) = 'r')                                                
        AND (tagname(.5.) = 'o')                                                
        AND (tagname(.6.) = 'u')                                                
        AND (tagname(.7.) = 't')                                                
        AND (tagname(.8.) = 's') THEN                                           
          categorizetag := attrgenrouts                                         
      END                                                                       
    END                                                                         
    ELSE IF c = 'l' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength >= 5 THEN                                                
      BEGIN                                                                     
        IF  (tagname(.2.) = 'i')                                                
        AND (tagname(.3.) = 'n')                                                
        AND (tagname(.4.) = 'k')                                                
        AND (tagname(.5.) = 's') THEN                                           
        BEGIN                                                                   
          IF getintfromtag(6) THEN                                              
          categorizetag := attrlinks                                            
        END                                                                     
      END                                                                       
    END                                                                         
    ELSE IF c = 'm' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength = 6 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'a')                                                
        AND (tagname(.3.) = 'i')                                                
        AND (tagname(.4.) = 'l')                                                
        AND (tagname(.5.) = 'e')                                                
        AND (tagname(.6.) = 'r') THEN                                           
          categorizetag := attrmailer                                           
      END                                                                       
    END                                                                         
    ELSE IF c = 'n' THEN                                                        
    BEGIN                                                                       
      IF (tagnamelength = 3) THEN                                               
      BEGIN                                                                     
        IF  (tagname(.2.) = 'e')                                                
        AND (tagname(.3.) = 't') THEN                                           
          categorizetag := attrnet                                              
      END                                                                       
      ELSE                                                                      
      IF (tagnamelength = 7) THEN                                               
      BEGIN                                                                     
        IF  (tagname(.2.) = 'e') THEN                                           
        BEGIN                                                                   
          IF  (tagname(.3.) = 't')                                              
          AND (tagname(.4.) = 's')                                              
          AND (tagname(.5.) = 'o')                                              
          AND (tagname(.6.) = 'f')                                              
          AND (tagname(.7.) = 't') THEN                                         
            categorizetag := attrnetsoft                                        
        END                                                                     
        ELSE IF (tagname(.2.) = 'o') THEN                                       
        BEGIN                                                                   
          IF  (tagname(.3.) = 'd')                                              
          AND (tagname(.4.) = 'e')                                              
          AND (tagname(.5.) = 'n')                                              
          AND (tagname(.6.) = 'u')                                              
          AND (tagname(.7.) = 'm') THEN                                         
            categorizetag := attrnodenum                                        
        END                                                                     
      END                                                                       
      ELSE IF tagnamelength = 10 THEN                                           
      BEGIN                                                                     
        IF  (tagname(.2.) = 'o')                                                
        AND (tagname(.3.) = 'd')                                                
        AND (tagname(.4.) = 'e')                                                
        AND (tagname(.5.) = 'r')                                                
        AND (tagname(.6.) = 'e')                                                
        THEN                                                                    
        BEGIN (* the C code shows why I have added a block here *)              
          IF  (tagname(.7.) = 's')                                              
          AND (tagname(.8.) = 'i')                                              
          AND (tagname(.9.) = 's')                                              
          AND (tagname(.10.) = 't') THEN                                        
            categorizetag := attrnoderesist                                     
        END                                                                     
      END                                                                       
    END                                                                         
    ELSE IF c = 'r' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength = 7 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'o')                                                
        AND (tagname(.3.) = 'u')                                                
        AND (tagname(.4.) = 't')                                                
        AND (tagname(.5.) = 't')                                                
        AND (tagname(.6.) = 'a')                                                
        AND (tagname(.7.) = 'b') THEN                                           
          categorizetag := attrrouttab                                          
      END                                                                       
    END                                                                         
    ELSE IF c = 's' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength >= 7 THEN                                                
      BEGIN                                                                     
        IF  (tagname(.2.) = 'e')                                                
        AND (tagname(.3.) = 'r')                                                
        AND (tagname(.4.) = 'v')                                                
        AND (tagname(.5.) = 'e')                                                
        AND (tagname(.6.) = 'r')                                                
        AND (tagname(.7.) = 's') THEN                                           
        BEGIN                                                                   
          IF getintfromtag(8) THEN                                              
          categorizetag := Attrservers ;                                        
        END                                                                     
      END                                                                       
    END                                                                         
    ELSE IF c = 't' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength = 4 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'y')                                                
        AND (tagname(.3.) = 'p')                                                
        AND (tagname(.4.) = 'e') THEN                                           
          categorizetag := attrtype                                             
      END                                                                       
    END                                                                         
    ELSE IF c = 'v' THEN                                                        
    BEGIN                                                                       
      IF tagnamelength = 3 THEN                                                 
      BEGIN                                                                     
        IF  (tagname(.2.) = 'i')                                                
        AND (tagname(.3.) = 'a') THEN                                           
          categorizetag := attrvia                                              
      END                                                                       
    END                                                                         
  END (* categorizetag *) ;                                                     
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function determines whether the current node in prgress     *)         
(*  is either the VERSxxxx or LINKSxxx node and if the occur at the  *)         
(*  beginning of the nodes file.                                     *)         
(*                                                                   *)         
(*  The function uses the current tagvalue as nodename. The only     *)         
(*  allowed place to call it is in the getnodes routine.             *)         
(*  The routine is coded as a separate function in order to          *)         
(*  make the getnodes routine a bit more readable.                   *)         
(*                                                                   *)         
(*  The two variables linksnode and versnode are set.                *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE setprologue(node:INTEGER);                                            
  BEGIN                                                                         
    IF tagvaluelength <> 8 THEN                                                 
      prologuenotdone := False                                                  
    ELSE                                                                        
    BEGIN                                                                       
      IF tagvalue(.1.) = 'V' THEN                                               
      BEGIN                                                                     
        IF versnode <> 0 THEN  prologuenotdone := False                         
        ELSE IF (tagvalue(.2.) = 'E')                                           
            AND (tagvalue(.3.) = 'R')                                           
            AND (tagvalue(.4.) = 'S')                                           
        THEN                                                                    
        BEGIN                                                                   
          prologuenotdone := linksnode = 0 ;                                    
          versnode := node                                                      
        END                                                                     
        ELSE prologuenotdone := False                                           
      END                                                                       
      ELSE IF tagvalue(.1.) = 'L' THEN                                          
      BEGIN                                                                     
        IF linksnode <> 0 THEN  prologuenotdone := False                        
        ELSE IF (tagvalue(.2.) = 'I')                                           
            AND (tagvalue(.3.) = 'N')                                           
            AND (tagvalue(.4.) = 'K')                                           
        THEN                                                                    
        BEGIN                                                                   
          prologuenotdone := versnode = 0 ;                                     
          linksnode := node                                                     
        END                                                                     
        ELSE prologuenotdone := False                                           
      END                                                                       
      ELSE prologuenotdone := False                                             
    END                                                                         
  END (* setprologue *);                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This function parses the nodes file.                             *)         
(*                                                                   *)         
(*  getnodes recognizes both the old and the new bitearn nodes       *)         
(*  format and creates some tables containing sufficient information *)         
(*  for routing table generation.                                    *)         
(*                                                                   *)         
(*  The routine consists of a loop reading all tags. When a major    *)         
(*  tag is found a new internal entry is created in the various      *)         
(*  data structures. The main idea of the rest of the routin is      *)         
(*  that all information of an entry is complete at each minor       *)         
(*  entry so no special processing to finish an entry. (This is      *)         
(*  no quite true because routine checknodes does some extras.)      *)         
(*                                                                   *)         
(*  There are some conflicting tags in the new and the old format.   *)         
(*  Entries from the new format always take precedences, i.e.        *)         
(*  when a new tag is found like links then any adjnodes/adjtypes    *)         
(*  information is reset and further occurences of old tags will     *)         
(*  be ignored.                                                      *)         
(*                                                                   *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
PROCEDURE getnodes ;                                                            
  LABEL 8;                                                                      
  VAR                                                                           
    node            : INTEGER ; (* internal number of this entry     *)         
    alias           : INTEGER ; (* internal number of alias entry    *)         
                                                                                
    nodenum         : INTEGER ; (* JES2 node number of this node     *)         
    aliasnum        : INTEGER ; (* JES2 node number of alias node    *)         
                                                                                
    noderesist      : INTEGER ; (* outgoing node resistance          *)         
                                                                                
    thereislinkstag : BOOLEAN ; (* true = new link information       *)         
    ignoreentry     : BOOLEAN ; (* duplicate nodeentry               *)         
    link            : INTEGER ; (* start of link information         *)         
    linktype        : INTEGER ; (* linktype of entry                 *)         
    adjrest,adjnode : INTEGER ; (* status of adjtype/adjnode list    *)         
                                                                                
    linkrest        : INTEGER ; (* used to compute link resistance   *)         
    i,k,j           : INTEGER ; (* not the TSO IKJ*                  *)         
    cc1,cc2         : CHAR    ; (* 2 letter ISO country code         *)         
    c               : CHAR    ; (* a variable used for parsing       *)         
  BEGIN                                                                         
    initdatabase;                                                               
    gettag ;                                                                    
    initmajorname ;                                                             
    adjnode := linkfree ;                                                       
    adjrest := linkfree + 1 ;                                                   
    WHILE tagnamelength > 0 DO                                                  
    BEGIN                                                                       
      IF ismajortag THEN                                                        
      BEGIN                                                                     
        ignoreentry := false;                                                   
        node := findnode(1,tagvaluelength) ;                                    
        IF nodes(.node.).links <> 0 THEN                                        
          ignoreentry := true                                                   
        ELSE BEGIN                                                              
          putxmailertag(attrnode,node);                                         
          thereislinkstag := False ;                                            
          IF prologuenotdone THEN setprologue(node) ;                           
          nodes(.node.).links := linkfree ;                                     
                                                                                
          sortednode := sortednode + 1;                                         
          sorted(.sortednode.) := node ;                                        
                                                                                
          link := linkfree ;                                                    
          links(.link.) := 0 ;                                                  
          links(.link+1.) := 0 ;                                                
          linkfree := link + 2 ;                                                
          adjnode := linkfree ;                                                 
          adjrest := linkfree + 1 ;                                             
                                                                                
          noderesist := 1 ;                                                     
          routtabformats(.node.) := formatnotdone ;                             
                                                                                
          alias := 0 ;                                                          
          aliasnum := 0 ;                                                       
          nodenum  := 0 ;                                                       
          jes2numbers(.node.) := 0 ;                                            
          currentoffset := 0 ;                                                  
          currentsort := sortasinput ;                                          
                                                                                
          WITH domain(.node.) DO                                                
          BEGIN                                                                 
            c1 := ' '; c2 := ' '; network := 0 ; entrytype := 0 ;               
          END                                                                   
        END                                                                     
      END                                                                       
      ELSE IF ignoreentry THEN gettag                                           
      ELSE CASE categorizetag OF                                                
      notinteresting : GOTO 8 ;                                                 
                                                                                
      AttrGenrouts   :                                                          
        IF tagvaluelength >= 8 THEN                                             
        BEGIN                                                                   
          i := 1 ;                                                              
          WHILE i <= 8 DO                                                       
          BEGIN                                                                 
            IF tagvalue(.i.) > genroutsversion(.i.) THEN                        
               fatalerror(programstupid)                                        
            ELSE IF tagvalue(.i.) < genroutsversion(.i.) THEN                   
               i := 8 ;                                                         
            i := Succ(i) ;                                                      
            IF (i = 3) OR (i = 6) THEN i := Succ(i)                             
          END                                                                   
        END (* Genrouts tag *) ;                                                
                                                                                
      AttrAdjnodes   :                                                          
                                                                                
        (* The old genrouts just concatenated the values. *)                    
        (* No order check, same is done here              *)                    
                                                                                
        IF NOT thereislinkstag THEN                                             
        BEGIN                                                                   
          i := 0 ;                                                              
          j := 1 ;                                                              
          REPEAT                                                                
            i := i + 1 ;                                                        
            IF tagvalue(.i.) = ' ' THEN                                         
            BEGIN                                                               
              IF i = j THEN j := j + 1                                          
              ELSE BEGIN                                                        
                links(.adjnode.) := findnode(j,i-j) ;                           
                IF links(.link.) = links(.adjnode.) THEN                        
                BEGIN                                                           
                  linkfree := linkfree - 2 ;                                    
                  links(.link.) := 0 ;                                          
                  links(.link+1.) := links(.link+1.) - 1 ;                      
                END ;                                                           
                adjnode := adjnode + 2 ;                                        
                j := i + 1                                                      
              END                                                               
            END                                                                 
          UNTIL i > tagvaluelength ;                                            
        END ;                                                                   
                                                                                
      AttrAdjtypes   :                                                          
        IF NOT thereislinkstag THEN BEGIN                                       
          i := 0 ;                                                              
          j := 1 ;                                                              
          REPEAT                                                                
            i := i + 1 ;                                                        
            IF tagvalue(.i.) = ' ' THEN                                         
            BEGIN                                                               
              IF i = j THEN j := j + 1                                          
              ELSE BEGIN                                                        
                links(.adjrest.) :=  calcrest(j,i-j) ;                          
                adjrest := adjrest + 2 ;                                        
                linkfree := linkfree + 2 ;                                      
                links(.link+1.) := links(.link+1.) + 1 ;                        
                j := i + 1                                                      
              END                                                               
            END                                                                 
          UNTIL i > tagvaluelength ;                                            
          IF links(.link.) <> 0 THEN                                            
          BEGIN                                                                 
            IF adjnode < linkfree THEN BEGIN                                    
              links(.linkfree-2.) := links(.link.) ;                            
              links(.linkfree-1.) := resistanceVia                              
            END                                                                 
          END ;                                                                 
        END ;                                                                   
                                                                                
      AttrAlias      :                                                          
        BEGIN                                                                   
          putxmailertag(Attralias,node) ;                                       
          alias := findnode(1,tagvaluelength) ;                                 
          IF nodes(.alias.).links = 0 THEN BEGIN                                
            nodes(.alias.).links :=  - node ;                                   
            sortednode := sortednode + 1;                                       
            sorted(.sortednode.) := alias;                                      
                                                                                
            IF aliasnum <> 0 THEN                                               
               jes2numbers(.alias.):= aliasnum ;                                
            WITH domain(.alias.) DO                                             
            BEGIN                                                               
              c1 := cc1 ; c2 := cc2 ;                                           
              network := domain(.node.).network;                                
              entrytype := domain(.node.).entrytype                             
            END                                                                 
          END                                                                   
          ELSE IF alias <> node THEN                                            
             putmsg(invalidalias)                                               
        END ;                                                                   
                                                                                
      AttrAliasnum   :                                                          
        BEGIN                                                                   
          aliasnum := getnodenum(1,tagvaluelength) ;                            
          IF alias <> 0 THEN                                                    
            jes2numbers(.alias.) := aliasnum ;                                  
        END ;                                                                   
                                                                                
      AttrNetsoft    :                                                          
        BEGIN                                                                   
          putxmailertag(Attrnetsoft,node) ;                                     
          IF routtabformats(.node.) = formatnotdone THEN                        
            routtabformats(.node.) := donetsoft ;                               
        END ;                                                                   
                                                                                
      AttrVia        :                                                          
        IF NOT thereislinkstag THEN                                             
        BEGIN                                                                   
          i := findnode(1,tagvaluelength) ;                                     
          IF i <> 0 THEN                                                        
          BEGIN                                                                 
            via(.node.) := i ;                                                  
            j := link + 2 ;                                                     
            WHILE j <= adjnode DO                                               
            BEGIN                                                               
              IF i = links(.j.) THEN GOTO 8 ;                                   
              j := j + 2                                                        
            END ;                                                               
            links(.link.) := i ;                                                
            links(.linkfree.) := i ;                                            
            links(.linkfree+1.) := resistancevia ;                              
            linkfree := linkfree + 2 ;                                          
            links(.link+1.) := links(.link+1.) + 1 ;                            
          END                                                                   
        END (* Via tag *) ;                                                     
      AttrNodenum    :                                                          
        BEGIN                                                                   
          nodenum := getnodenum(1,tagvaluelength) ;                             
          jes2numbers(.node.) := nodenum ;                                      
          IF (alias <> 0) AND (aliasnum <> 0) THEN                              
             jes2numbers(.alias.) := nodenum ;                                  
        END ;                                                                   
                                                                                
      AttrLinks      :                                                          
        BEGIN                                                                   
          IF NOT thereislinkstag THEN                                           
          BEGIN                                                                 
            links(.link.) := 0 ;                                                
            links(.link+1.) := 0 ;                                              
            linkfree := link + 2 ;                                              
            thereislinkstag := True ;                                           
          END ;                                                                 
          i := 1 ;                                                              
          j := 1 ;                                                              
                                                                                
          linktype := permanentlink ;                                           
                                                                                
          REPEAT                                                                
                                                                                
            WHILE tagvalue(.i.) = ' ' DO i := i + 1;                            
            j := i ;                                                            
            REPEAT                                                              
              i := Succ(i) ;                                                    
              c := tagvalue(.i.) ;                                              
            UNTIL (i = tagvaluelength)                                          
               OR (c = ' ') OR (c = '(') ;                                      
            IF i = tagvaluelength THEN i := Succ(i);                            
                                                                                
            links(.linkfree.) := findnode(j,i-j) ;                              
            linkrest  := resistancevia - 1 ;                                    
            IF c = '(' THEN                                                     
            BEGIN                                                               
                                                                                
              j := Succ(i) ;                                                    
              REPEAT                                                            
              i := Succ(i) ;                                                    
              c := tagvalue(.i.) ;                                              
              UNTIL (c = ')') OR ( c = ',') ;                                   
                                                                                
              IF i > j THEN BEGIN                                               
                IF tagvalue(.j.) = 'D' THEN                                     
                  linktype := dialuplink                                        
                ELSE IF tagvalue(.j.) = 'V' THEN                                
                BEGIN                                                           
                  linktype := virtualonelink ;                                  
                  IF i > j + 1 THEN                                             
                     linktype := getintfromvalue(j+1,i-1)                       
                END                                                             
              END ;                                                             
                                                                                
              IF c = ',' THEN BEGIN                                             
                j := Succ(i) ;                                                  
                REPEAT                                                          
                i := Succ(i) ;                                                  
                c := tagvalue(.i.) ;                                            
                UNTIL (c = ')') OR ( c = ',') ;                                 
                                                                                
                IF i > j THEN BEGIN                                             
                   linkrest :=                                                  
                      getbigintfromvalue(j,i-1);                                
                   IF linkrest < 0 THEN linkrest := defaultlinkrest             
                   ELSE IF linkrest > 0 THEN                                    
                   linkrest := (billion DIV linkrest) ;                         
                END ;                                                           
                                                                                
                IF c = ',' THEN BEGIN                                           
                  j := Succ(i) ;                                                
                  REPEAT                                                        
                  i := Succ(i) ;                                                
                  c := tagvalue(.i.) ;                                          
                  UNTIL (c = ')') OR ( c = ',') ;                               
                  IF (i > j) AND (linkrest > 0) THEN                            
                  BEGIN                                                         
                    linkrest := getKintfromvalue(j,i-1);                        
                    IF linkrest < 0 THEN                                        
                      linkrest := defaultlinkrest                               
                  END ;                                                         
                                                                                
                  WHILE tagvalue(.i.) <> ')' DO                                 
                    i := Succ(i)                                                
                END                                                             
              END ;                                                             
              IF c <> ')' THEN (* ????? *) ;                                    
            END ;                                                               
            IF linkrest > 0 THEN                                                
            BEGIN                                                               
              links(.linkfree+1.) := linkrest + noderesist ;                    
              linkfree := linkfree + 2 ;                                        
              links(.link+1.) := links(.link+1.) + 1 ;                          
              IF (node = versnode) AND                                          
                 (links(.link+1.) = 1)                                          
              THEN defaultlinkrest := linkrest ;                                
            END ;                                                               
            i := i + 1;                                                         
                                                                                
          UNTIL i > tagvaluelength ;                                            
        END (* Links tag *) ;                                                   
                                                                                
      AttrNoderesist :                                                          
        BEGIN                                                                   
          noderesist := getKintfromvalue(1,tagvaluelength) ;                    
          if noderesist <= 0 THEN                                               
             noderesist := defaultnoderesist ;                                  
          IF node = versnode THEN defaultnoderesist := noderesist ;             
          IF thereislinkstag THEN                                               
          BEGIN                                                                 
            i := link + 3 ;                                                     
                                                                                
            WHILE i < linkfree DO                                               
            BEGIN                                                               
              links(.i.) := links(.i.) + noderesist ;                           
              i := i + 2                                                        
            END                                                                 
          END                                                                   
        END (* Noderesist tag *) ;                                              
                                                                                
      AttrRouttab    :                                                          
        BEGIN                                                                   
          routtabformats(.node.) := dorouttab ;                                 
          offsets(.node.) := currentoffset ;                                    
          sorttypes(.node.) := currentsort ;                                    
          IF lidassociations > 0 THEN                                           
             links(.lidlists.) := node                                          
        END ;                                                                   
                                                                                
      AttrCountry    :                                                          
        BEGIN                                                                   
          cc1 := tagvalue(.1.) ;                                                
          cc2 := tagvalue(.2.) ;                                                
          WITH domain(.node.) DO                                                
          BEGIN                                                                 
            c1 := cc1 ; c2 := cc2                                               
          END ;                                                                 
          IF alias <> 0 THEN                                                    
          WITH domain(.alias.) DO                                               
          BEGIN                                                                 
            c1 := cc1 ; c2 := cc2                                               
          END ;                                                                 
        END (* Country tag *) ;                                                 
                                                                                
      AttrNet        :                                                          
        BEGIN                                                                   
          domain(.node.).network := findnetwork(1,tagvaluelength) ;             
          IF alias <> 0 THEN                                                    
          domain(.alias.).network := domain(.node.).network;                    
        END ;                                                                   
                                                                                
      AttrMailer     :                                                          
        putxmailertag(Attrmailer,node) ;                                        
                                                                                
      AttrServers    :                                                          
        putxmailertag(Attrservers,node) ;                                       
                                                                                
      AttrType       :                                                          
        BEGIN                                                                   
          domain(.node.).entrytype :=                                           
                                  findtype(1,tagvaluelength) ;                  
          IF alias <> 0 THEN                                                    
          domain(.alias.).entrytype := domain(.node.).entrytype ;               
                                                                                
                                                                                
        END ;                                                                   
      END ;                                                                     
    8:gettag                                                                    
    END ;                                                                       
    putxmailertag(attrnode,node)                                                
  END (* getnodes *) ;                                                          
(*********************************************************************)         
(*                                                                   *)         
(*  This function does some validation and post processing of a      *)         
(*  nodes data structure after calling getnodes.                     *)         
(*                                                                   *)         
(*  Nodes without links will be removed.                             *)         
(*  Aliases to undefined nodes will be deleted.                      *)         
(*                                                                   *)         
(*  Missing JES2 nodenumber will be assigned.                        *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE checknodes ;                                                          
  LABEL 1 ;                                                                     
  VAR i,j,k,l: INTEGER ;                                                        
  BEGIN                                                                         
                                                                                
                                                                                
    FOR i := 1 TO numberofnodes DO                                              
    IF  nodes(.i.).links = 0 THEN                                               
    BEGIN                                                                       
      IF debug THEN BEGIN                                                       
        Write(sysprint,                                                         
        'The following node is referenced but not defined: ') ;                 
        putnodename(i,sysprint) ;                                               
        Writeln(sysprint)                                                       
      END ;                                                                     
      nodes(.i.).name := 0                                                      
    END ;                                                                       
                                                                                
  1:FOR i := 1 TO numberofnodes DO                                              
    BEGIN                                                                       
      j := nodes(.i.).links ;                                                   
      IF j < 0 THEN BEGIN                                                       
        IF nodes(.-j.).name = 0 THEN                                            
        BEGIN                                                                   
          Write(sysprint,                                                       
          'The following alias points to an undefined node: ') ;                
          putnodename(i,sysprint) ;                                             
          Writeln(sysprint) ;                                                   
          nodes(.i.).name := 0 ;                                                
          nodes(.i.).links := 0                                                 
        END                                                                     
      END ;                                                                     
    END ;                                                                       
                                                                                
    FOR i := 1 TO numberofnodes DO                                              
    BEGIN                                                                       
      j := nodes(.i.).links ;                                                   
      IF j > 0 THEN BEGIN                                                       
        links(.j.) := i ;                                                       
        l := links(.j+1.)*2 + j ;                                               
        j := j + 2 ;                                                            
        WHILE j <= l DO                                                         
        BEGIN                                                                   
          k := links(.j.) ;                                                     
          IF k > 0 THEN BEGIN                                                   
            IF nodes(.k.).links = 0 THEN                                        
            BEGIN                                                               
              IF debug THEN BEGIN                                               
                Write(sysprint,                                                 
    'The following node contains a reference to an undefined node: ') ;         
                putnodename(i,sysprint) ;                                       
                Writeln(sysprint)                                               
              END ;                                                             
              links(.j.) := 0                                                   
            END                                                                 
            ELSE IF nodes(.k.).links < 0 THEN                                   
            BEGIN                                                               
              k := - nodes(.k.).links ;                                         
              links(.j.) := k ;                                                 
              Write(sysprint,                                                   
              'A link to an alias is replaced by node: ') ;                     
              putnodename(i,sysprint) ;                                         
              Writeln(sysprint) ;                                               
              Write(sysprint,'  ') ;                                            
              putnodename(k,sysprint) ;                                         
              Writeln(sysprint)                                                 
            END                                                                 
          END ;                                                                 
          j := j + 2                                                            
        END ;                                                                   
        j := nodes(.i.).links ;                                                 
        k := links(.j+1.) ; l := k*2 + j  ; j := j + 2  ;                       
        WHILE j <= l DO                                                         
        BEGIN                                                                   
          IF links(.j.) = 0 THEN k := k - 1 ;                                   
          j := j + 2                                                            
        END ;                                                                   
        IF k <= 0 THEN                                                          
        BEGIN                                                                   
          IF debug OR (domain(.i.).entrytype = njetype) THEN                    
          BEGIN                                                                 
            Write(sysprint,                                                     
            'The following node has no links: ') ;                              
            putnodename(i,sysprint) ;                                           
            Writeln(sysprint)                                                   
          END ;                                                                 
          nodes(.i.).links := 0 ;                                               
          nodes(.i.).name := 0 ;                                                
          GOTO 1                                                                
        END ;                                                                   
      END                                                                       
    END ;                                                                       
                                                                                
    (* apply pending node name *)                                               
                                                                                
    FOR j := 1 TO sortednode DO                                                 
    BEGIN                                                                       
      i := sorted(.j.) ;                                                        
      IF nodes(.i.).name <> 0 THEN                                              
      BEGIN                                                                     
        IF jes2numbers(.i.) = -1 THEN                                           
        BEGIN                                                                   
          maxjes2num := maxjes2num + 1;                                         
          inventedjes2num := inventedjes2num + 1;                               
          jes2numbers(.i.) := maxjes2num                                        
        END ;                                                                   
        jes2nodes(.jes2numbers(.i.).) := i                                      
      END                                                                       
    END ;                                                                       
                                                                                
  END (* checknodes *) ;                                                        
(*********************************************************************)         
(*                                                                   *)         
(*  Here's the heart of routing table generation. A minimal spanning *)         
(*  tree of paths seen TO a given node will be constructed.          *)         
(*                                                                   *)         
(*  This tree is later used (and probably destroyed) in the routing  *)         
(*  table formatter.                                                 *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
FUNCTION spanningtree(node: INTEGER) : BOOLEAN ;                                
  LABEL 1;                                                                      
  CONST indefinite = -1 ;                                                       
  VAR                                                                           
    heapstart     : INTEGER ;                                                   
    i,j,l,k       : INTEGER ;                                                   
    h1,h2,h3,h4   : INTEGER ;                                                   
    dx,dy,x,y,dyn : INTEGER ;                                                   
                                                                                
  BEGIN                                                                         
                                                                                
    spanningtree := True;  (* There are no errors *)                            
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Initialize our spanning tree and the heap.                       *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
    FOR i := 1 TO numberofnodes DO BEGIN                                        
      dists(.i.) := indefinite ;                                                
      marked(.i.) := False ;                                                    
     sources(.i.) := False ;                                                    
      spanningheap(.i.) := 0 ;                                                  
      paths(.i.) := 0 ;                                                         
    END ;                                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  The distance to the our node is 0, what else, and our node       *)         
(*  is the first (and only) node on the heap.                        *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
    dists(.node.) := 0 ;                                                        
    heapstart := node ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Take the "best" node on the heap, i.e. the one with the          *)         
(*  shortest dist, inspect all its outgoing links, calculate the     *)         
(*  distance to this node.                                           *)         
(*                                                                   *)         
(*********************************************************************)         
    isbettercall := 0 ;                                                         
    REPEAT                                                                      
      dx := dists(.heapstart.) ;                                                
      i := nodes(.heapstart.).links ;                                           
      j := links(.i+1.)*2 + i ;                                                 
      l := i + 2 ;                                                              
      k := i + 3 ;                                                              
      WHILE l <= j DO BEGIN                                                     
        y := links(.l.) ;                                                       
        IF y <> 0 THEN BEGIN                                                    
          dyn := links(.k.) + dx ;                                              
          dy  := dists(.y.) ;                                                   
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  We now test for better paths, and if so we reorganize our        *)         
(*  paths tree and the "yet unprocessed nodes" heap.                 *)         
(*                                                                   *)         
(*  The first test is the classical one. The second comparison       *)         
(*  is done also with the equal because we might want to prefer      *)         
(*  a path with equal resistance for other reasons.                  *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
          IF (dy = indefinite) OR (dy >= dyn) THEN                              
          BEGIN                                                                 
                                (* shorter path found *)                        
(*********************************************************************)         
(*                                                                   *)         
(*  Now we check the "direct path prefered" rule.                    *)         
(*  It is possible that the topology is not eucledian, i.e. a        *)         
(*  direct connection may be "longer" than an indirect path. In      *)         
(*  order to utilize such a link and in order to have transitive     *)         
(*  routes we do not want to take the "better"indirect path if there *)         
(*  is a direct connection.                                          *)         
(*                                                                   *)         
(*  If we find a direct link then we mark all nodes on the           *)         
(*  'shorter path'.                                                  *)         
(*                                                                   *)         
(*                                                                   *)         
(*********************************************************************)         
            h1 := paths(.heapstart.) ;                                          
            WHILE (h1 <> 0) DO                                                  
            BEGIN                                                               
              h3 := nodes(.h1.).links ;                                         
              FOR h2 := 1 TO links(.h3+1.) DO                                   
              BEGIN                                                             
                h3 := h3 + 2;                                                   
                IF links(.h3.) = y THEN                                         
                BEGIN                                                           
                  marked(.y.) := True ;                                         
                  h3 := heapstart ;                                             
                  WHILE (h1 <> h3) DO                                           
                  BEGIN                                                         
                                                                                
                     marked(.h3.) := True ;                                     
                     h3 := paths(.h3.) ;                                        
                  END ;                                                         
                  GOTO 1                                                        
                END                                                             
              END ;                                                             
              h1 := paths(.h1.) ;                                               
            END ;                                                               
                                                                                
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  When there are equally weighted paths one has to select one.     *)         
(*                                                                   *)         
(*  First, we try to use the one with the minimal number of hops.    *)         
(*                                                                   *)         
(*  The same path must be selected from both ends. We use the        *)         
(*  lowest JES2 node number on the two paths.                        *)         
(*                                                                   *)         
(*  The implementation is not optimal in time but we do not want to  *)         
(*  waste storage. We simply follow the two paths and look for the   *)         
(*  minimal value. If both minima are identical then the two paths   *)         
(*  have already joined before the minimal value. We repeat the      *)         
(*  process with the minimal node as end point.                      *)         
(*                                                                   *)         
(*********************************************************************)         
            IF (dy = dyn) AND (paths(.y.) <> heapstart) THEN                    
            BEGIN                                                               
              h4 := node ;                                                      
              h1 := heapstart ;                                                 
              h2 := 0 ;                                                         
              WHILE (h1 <> h4) DO                                               
              BEGIN                                                             
                h2 := h2 + 1;                                                   
                h1 := paths(.h1.)                                               
              END ;                                                             
              h1 := paths(.y.) ;                                                
              WHILE (h1 <> h4) DO                                               
              BEGIN                                                             
                h2 := h2 - 1;                                                   
                h1 := paths(.h1.)                                               
              END ;                                                             
              IF h2 > 0 THEN GOTO 1 ;                                           
              IF h2 = 0 THEN BEGIN                                              
                h4 := node ;                                                    
                REPEAT                                                          
                  h2 := maxjes2num ;                                            
                  h1 := heapstart ;                                             
                  WHILE (h1 <> h4) DO                                           
                  BEGIN                                                         
                    IF  jes2numbers(.h1.) < h2 THEN                             
                      h2 := jes2numbers(.h1.);                                  
                    h1 := paths(.h1.)                                           
                  END ;                                                         
                  h3 := maxjes2num ;                                            
                  h1 := paths(.y.) ;                                            
                  WHILE (h1 <> h4) DO                                           
                  IF  jes2numbers(.h1.) > h3 THEN                               
                    h1 := paths(.h1.)                                           
                  ELSE BEGIN                                                    
                    h3 := jes2numbers(.h1.);                                    
                    IF h3 > h2 THEN h1 := paths(.h1.)                           
                    ELSE IF h3 = h2 THEN h4 := h1                               
                    ELSE GOTO 1                                                 
                END ;                                                           
                UNTIL (h3 <> h2)                                                
              END ;                                                             
            END ;                                                               
(*********************************************************************)         
(*                                                                   *)         
(*  We now have found a "better" path.                               *)         
(*                                                                   *)         
(*********************************************************************)         
            dists(.y.) := dyn ;                                                 
            paths(.y.) := heapstart ;                                           
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Place node on the heap, if it is already there, the chain         *)        
(*  will be reorganized such that it is always sorted. Thus          *)         
(*  the node with the smallest distance is always the first in the   *)         
(*  chain.                                                           *)         
(*  This implementation of a heap is not the fastest, Mr. Tarjan,    *)         
(*  please forgive.                                                  *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
            h1  := heapstart ; h2 := spanningheap(.h1.) ;                       
                                                                                
            WHILE (h2 <> 0) AND (dyn > dists(.h2.)) DO                          
            BEGIN                                                               
              h1 := h2 ; h2 := spanningheap(.h1.) ;                             
            END ;                                                               
                                                                                
            IF y <> h2 THEN BEGIN                                               
              spanningheap(.h1.) := y ;                                         
              h3 := h2 ;                                                        
              h1 := y;                                                          
              WHILE (h3 <> y) AND (h3 <> 0) DO BEGIN                            
                h1 := h3 ;                                                      
                h3 := spanningheap(.h3.) ;                                      
              END ;                                                             
              spanningheap(.h1.) := spanningheap(.y.) ;                         
              spanningheap(.y.) := h2                                           
            END ;                                                               
        1:END                                                                   
        END ;                                                                   
(*********************************************************************)         
(*                                                                   *)         
(*  Take next link.                                                  *)         
(*                                                                   *)         
(*********************************************************************)         
        l := l + 2 ;                                                            
        k := k + 2 ;                                                            
      END ;                                                                     
(*********************************************************************)         
(*                                                                   *)         
(*  This node is done, pop the next node from the heap.              *)         
(*                                                                   *)         
(*********************************************************************)         
      h1 := heapstart ;                                                         
      heapstart := spanningheap(.heapstart.) ;                                  
      spanningheap(.h1.) := 0                                                   
    UNTIL heapstart = 0 ;                                                       
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Now we have at least a partial spanning tree. The missing pieces *)         
(*  are ALIASes and VIA links that go in the "wrong" direction.      *)         
(*  The simple solution for the tree is now to take the vias and     *)         
(*  aliases as path.                                                 *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
    FOR i := 1 TO numberofnodes DO                                              
    IF i <> node THEN                                                           
    BEGIN                                                                       
      IF via(.i.) = node THEN                                                   
        paths(.i.) := via(.i.)                                                  
      ELSE IF paths(.i.) = 0 THEN                                               
      BEGIN                                                                     
        IF  nodes(.i.).links > 0 THEN                                           
          paths(.i.) := via(.i.)                                                
        ELSE IF  nodes(.i.).links < 0 THEN                                      
          paths(.i.) := -nodes(.i.).links                                       
      END                                                                       
    END                                                                         
                                                                                
  END (* spanningtree *) ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This is a debugging formatter. If gives a list of the spanning   *)         
(*  tree showing the resitancies, node names, number, and domain     *)         
(*  info.                                                            *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putpaths(VAR tofile:TEXT);                                            
  VAR                                                                           
    i,j,pl,maxpath : INTEGER ;                                                  
  BEGIN                                                                         
    maxpath := 0 ;                                                              
    FOR i := 1 TO numberofnodes DO                                              
    BEGIN                                                                       
      Write(tofile,'Path  -');                                                  
      IF debug THEN Write(tofile,dists(.i.)) ;                                  
      putnodename(i,tofile) ;                                                   
      Writeln(tofile) ;                                                         
      IF nodes(.i.).links < 0 THEN                                              
      BEGIN                                                                     
        Write(tofile,' alias-') ;                                               
        putnodename(-nodes(.i.).links,tofile) ;                                 
        Writeln(tofile) ;                                                       
      END ELSE BEGIN                                                            
        pl := 1 ;                                                               
        j := paths(.i.) ;                                                       
        WHILE j <> 0 DO                                                         
        BEGIN                                                                   
          Write(tofile,'      -');                                              
          IF debug THEN Write(tofile,dists(.j.)) ;                              
          putnodename(j,tofile);                                                
          Writeln(tofile) ;                                                     
          j := paths(.j.) ;                                                     
          pl := pl + 1 ;                                                        
        END ;                                                                   
                                                                                
        IF pl > maxpath THEN BEGIN                                              
          maxpath := pl ;                                                       
        END ;                                                                   
      END ;                                                                     
      Writeln(tofile)                                                           
    END ;                                                                       
    Writeln(tofile,'Maximum path length: ',maxpath) ;                           
  END (* putpaths *) ;                                                          
                                                                                
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This procedure is a common routine for all formatters to create  *)         
(*  some identification at the beginning of a table.                 *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE genid(node:INTEGER);                                                  
  VAR                                                                           
    i,j,k : INTEGER ;                                                           
  BEGIN                                                                         
    emptyline ;                                                                 
    begincomment;                                                               
    Write(routetab,'GR version ');                                              
    FOR i := 1 TO 8 DO Write(routetab,genroutsversion(.i.));                    
    Write(routetab,', date=',dateiso,', time=',time,' ');                       
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    emptyline ;                                                                 
                                                                                
    begincomment;                                                               
    Write(routetab,'Table generation parameters: ') ;                           
    FOR i := 1 TO cmdvaluelength DO                                             
    BEGIN                                                                       
      IF i MOD 37 = 0 THEN                                                      
      BEGIN                                                                     
        endcomment ;                                                            
        Writeln(routetab) ;                                                     
        begincomment;                                                           
        Write(routetab,'                           : ') ;                       
      END ;                                                                     
      Write(routetab,cmdvalue(.i.));                                            
    END ;                                                                       
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    emptyline ;                                                                 
                                                                                
    begincomment;                                                               
    Write(routetab,'Routing table information for ') ;                          
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
                                                                                
    begincomment;                                                               
    Write(routetab,'node: ') ;                                                  
    prtnode(node,9) ;                                                           
    Write(routetab,'nodenum: ') ;                                               
    putintrt(jes2numbers(.node.)+currentoffset) ;                               
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    emptyline ;                                                                 
                                                                                
  END (* genid *) ;                                                             
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This procedure is a common routine for all formatters to create  *)         
(*  a list of directly attached neighbours as a comment at the       *)         
(*  beginning of each node file.                                     *)         
(*                                                                   *)         
(*  The two paths() statements work because we have paths(.0.) = 0   *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE neighbours(node:INTEGER);                                             
  VAR                                                                           
    i,j,k : INTEGER ;                                                           
    firstdone : BOOLEAN ;                                                       
  BEGIN                                                                         
                                                                                
    firstdone := False ;                                                        
    emptyline ;                                                                 
    begincomment;                                                               
    Write(routetab,'The routing table neither contains neighbours ');           
    Write(routetab,'nor the local node.');                                      
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    begincomment;                                                               
    Write(routetab,'List of neighbours:');                                      
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
                                                                                
    emptyline ;                                                                 
    FOR i := 1 TO numberofnodes DO                                              
    IF nodes(.i.).name > 0 THEN                                                 
    BEGIN                                                                       
      j := paths(.i.) ;                                                         
      k := paths(.j.) ;                                                         
                                                                                
      IF (k = 0) AND (j <> 0) THEN                                              
      BEGIN                                                                     
        begincomment ;                                                          
        IF firstdone THEN BEGIN                                                 
           Write(routetab,'      ') ;                                           
           prtnode(i,18)                                                        
        END ELSE BEGIN                                                          
           Write(routetab,'node: ') ;                                           
           prtnode(i,9) ;                                                       
           Write(routetab,'nodenum: ')                                          
        END ;                                                                   
        firstdone := True ;                                                     
        putintrt(jes2numbers(.i.)+currentoffset) ;                              
        endcomment ;                                                            
        Writeln(routetab)                                                       
      END ;                                                                     
    END ;                                                                       
    emptyline ;                                                                 
  END (* neighbours *) ;                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This procedure inspects the spanning tree (and destroys it)      *)         
(*  to create a RSCS like routing table. This class of tables can    *)         
(*  be characterised that for each node the next neighbour that      *)         
(*  leads to that node has to be determined.                         *)         
(*                                                                   *)         
(*  The ugly use of the prt routine is done to simplify the C port.  *)         
(*  I will eventually change this soon.                              *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE RSCStable(node:INTEGER; vers:ROUTTABFORMAT);                          
  VAR                                                                           
    i,j,k,l,si,ni : INTEGER ;                                                   
    firstpass : BOOLEAN ;                                                       
  BEGIN                                                                         
    asmsyntax := True ;                                                         
    IF vers = JNETformat THEN                                                   
      startofcomment := '!'                                                     
    ELSE                                                                        
      startofcomment := '*' ;                                                   
    genid(node);                                                                
    neighbours(node);                                                           
    IF vers = RSCSformat THEN BEGIN                                             
    begincomment;                                                               
    Write(routetab,'* The position of the network name and country *');         
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    begincomment;                                                               
    Write(routetab,'* assumes that the RSCS ROUTE command does not *');         
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    begincomment;                                                               
    Write(routetab,'* have more than two parameters.               *');         
    endcomment ;                                                                
    Writeln(routetab) ;                                                         
    END ;                                                                       
    firstpass := True;                                                          
    ni := 0 ;                                                                   
    FOR si := 1 TO sortednode DO                                                
    BEGIN                                                                       
      REPEAT                                                                    
        ni := ni+1 ;                                                            
        IF ni > sortednode THEN                                                 
        BEGIN                                                                   
           ni := 1 ;                                                            
           firstpass := False                                                   
        END ;                                                                   
        i := sorted(.ni.) ;                                                     
      UNTIL                                                                     
        (firstpass =                                                            
        ((i = versnode) OR (i = linksnode) OR                                   
          ((domain(.node.).C1 = domain(.i.).C1) AND                             
           (domain(.node.).C2 = domain(.i.).C2))));                             
      IF nodes(.i.).name > 0 THEN                                               
      BEGIN                                                                     
        l := i ;                                                                
        k := paths(.i.) ;                                                       
(*********************************************************************)         
(*                                                                   *)         
(*  We are only inspecting nodes which are not the local node and    *)         
(*  not a direct neighbour.                                          *)         
(*                                                                   *)         
(*  We follow the path to the next neighbour. While following        *)         
(*  the chain the spanning tree will be destroyed. The paths of      *)         
(*  all intermediate nodes will be set to the node where we started. *)         
(*  This will shorten the chains for all nodes that join the same    *)         
(*  chain.                                                           *)         
(*                                                                   *)         
(*********************************************************************)         
        IF k <> 0 THEN (* not ownnode *)                                        
        BEGIN                                                                   
          j := k ;                                                              
          k := paths(.k.) ;                                                     
          IF k <> 0 THEN (* not neighbour *)                                    
          BEGIN                                                                 
            REPEAT                                                              
              paths(.l.) := i ;                                                 
              l := j ;                                                          
              j := k ;                                                          
              k := paths(.k.) ;                                                 
            UNTIL k = 0 ;                                                       
            paths(.i.) := l ;                                                   
(*********************************************************************)         
(*                                                                   *)         
(*  We have the direct neighbour and will now generate a ROUTE       *)         
(*  definition statement.                                            *)         
(*                                                                   *)         
(*  The hardcoded formats seem to be ugly, but we do not expect too  *)         
(*  many new NJE emulators be developped in the next future.         *)         
(*                                                                   *)         
(*********************************************************************)         
            IF vers = RSCSformat THEN                                           
            BEGIN                                                               
              Write(routetab,'ROUTE ');                                         
              prtnode(i,9) ;                                                    
              prtnode(l,9) ;                                                    
              WITH domain(.i.) DO                                               
              BEGIN                                                             
                IF network = 0 THEN Write(routetab,'?        ')                 
                ELSE putv(networks(.network.),routetab,9) ;                     
                Write(routetab,c1,c2) ;                                         
              END                                                               
            END                                                                 
            ELSE IF vers = JNETformat THEN                                      
            BEGIN                                                               
              Write(routetab,'DEFINE ');                                        
              prtnode(i,9) ;                                                    
              Write(routetab,'/ROUTE=');                                        
              prtnode(l,9) ;                                                    
              Write(routetab,'! ');                                             
              WITH domain(.i.) DO                                               
              BEGIN                                                             
                IF network = 0 THEN Write(routetab,'?        ')                 
                ELSE putv(networks(.network.),routetab,9) ;                     
                Write(routetab,c1,c2) ;                                         
              END                                                               
            END                                                                 
            ELSE IF vers = UREPformat THEN                                      
            BEGIN                                                               
              prtnode(i,9) ;                                                    
              prtnode(l,9) ;                                                    
              Write(routetab,'*')                                               
            END                                                                 
            ELSE IF vers = JES3format THEN                                      
            BEGIN                                                               
              Write(routetab,'NJERMT,NAME=');                                   
              prtnode(i,0) ;                                                    
              Write(routetab,',MAXLINE=0,PATH=');                               
              prtnode(l,0)                                                      
            END ;                                                               
            Writeln(routetab)                                                   
          END ;                                                                 
        END                                                                     
      END                                                                       
    END ;                                                                       
  END (* RSCSTable *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This procedures create a unix map file as requested by M.Gettes. *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE writepath(p:INTEGER) ;                                                
  VAR                                                                           
    np : INTEGER ;                                                              
  BEGIN                                                                         
    np := paths(.p.) ;                                                          
    IF np <> 0 THEN BEGIN                                                       
      writepath(np) ;                                                           
      prtnodelc(p,0) ;                                                          
      write(routetab,'!')                                                       
    END ;                                                                       
  END (* writepath *) ;                                                         
                                                                                
PROCEDURE PATHtable;                                                            
  VAR                                                                           
    i,j : INTEGER ;                                                             
    firstpass : BOOLEAN ;                                                       
  BEGIN                                                                         
    FOR i := 1 TO numberofnodes DO                                              
    IF nodes(.i.).name > 0 THEN                                                 
    BEGIN                                                                       
      j := paths(.i.) ;                                                         
      prtnodelc(i,0) ;                                                          
      write(routetab,htab) ;                                                    
      IF j <> 0 THEN (* not ownnode *)                                          
        writepath(i) ;                                                          
      Writeln(routetab,'%s')                                                    
    END ;                                                                       
  END (* PATHTable *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  We have to determine a lid for a node.                           *)         
(*                                                                   *)         
(*  node is node number, lidlist may point to an association list.   *)         
(*  The default processing is to add the nodenumber to "NJE".        *)         
(*  If the node is not in the list the default lid is created, too.  *)         
(*                                                                   *)         
(*  The lid list either contains the internal node ids or the        *)         
(*  negative JES2 node number.                                       *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE putlidrt(node:INTEGER; lidlist:INTEGER);                              
VAR lid:INTEGER ;                                                               
    numberoflids,lidassociation:INTEGER ;                                       
    nodenumber :INTEGER ;                                                       
BEGIN                                                                           
  lid := 0 ;                                                                    
  nodenumber:=-jes2numbers(.node.);                                             
  IF lidlist > 0 THEN                                                           
  BEGIN                                                                         
    numberoflids := links(.lidlist+1.);                                         
    lidassociation := lidlist+2 ;                                               
    WHILE numberoflids > 0 DO                                                   
    BEGIN                                                                       
      IF (links(.lidassociation.) = node)                                       
      OR (links(.lidassociation.) = nodenumber) THEN                            
      BEGIN                                                                     
        lid:=links(.lidassociation+1.) ;                                        
        numberoflids := 0                                                       
      END ELSE BEGIN                                                            
        numberoflids := numberoflids - 1;                                       
        lidassociation := lidassociation + 2                                    
      END                                                                       
    END                                                                         
  END ;                                                                         
                                                                                
  IF lid = 0 THEN lid := 30506 - nodenumber ; (* "NJE"+number *)                
                                                                                
  write(routetab,base36tochar(.lid DIV 1296.));                                 
  lid := lid MOD 1296 ;                                                         
  write(routetab,base36tochar(.lid DIV 36.));                                   
  lid := lid MOD 36 ;                                                           
  write(routetab,base36tochar(.lid.))                                           
                                                                                
END (* putlidrt *) ;                                                            
(*********************************************************************)         
(*                                                                   *)         
(*  This procedure inspects the spanning tree (and destroys it)      *)         
(*  to create an NJEF routing table. The table is similar to the     *)         
(*  RSCS tables.                                                     *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE NJEFtable(node:INTEGER);                                              
  VAR                                                                           
    i,j,k,l,si : INTEGER ;                                                      
    currentlidlist : INTEGER ;                                                  
  BEGIN                                                                         
    currentlidlist := 0 ;                                                       
    i := lidlists ;                                                             
    WHILE (i < maxconn) AND (currentlidlist = 0) DO                             
    BEGIN                                                                       
      IF links(.i.) = node THEN                                                 
        currentlidlist := i                                                     
      ELSE                                                                      
        i := i + links(.i+1.)*2+2 ;                                             
    END ;                                                                       
                                                                                
    asmsyntax := True ;                                                         
    startofcomment := '*' ;                                                     
    genid(node);                                                                
    neighbours(node);                                                           
    FOR si := 1 TO sortednode DO                                                
    BEGIN                                                                       
      i := sorted(.si.) ;                                                       
      IF nodes(.i.).name > 0 THEN                                               
      BEGIN                                                                     
        l := i ;                                                                
        k := paths(.i.) ;                                                       
        IF k <> 0 THEN (* not ownnode *)                                        
        BEGIN                                                                   
          j := k ;                                                              
          k := paths(.k.) ;                                                     
          IF k <> 0 THEN (* not neighbour *)                                    
          BEGIN                                                                 
            REPEAT                                                              
              paths(.l.) := i ;                                                 
              l := j ;                                                          
              j := k ;                                                          
              k := paths(.k.) ;                                                 
            UNTIL k = 0 ;                                                       
            paths(.i.) := l ;                                                   
                                                                                
            Write(routetab,'FAMILY,');                                          
            prtnode(i,8) ;                                                      
            Write(routetab,',NOD');                                             
            putintrt(jes2numbers(.i.)) ;                                        
            Write(routetab,',');                                                
                                                                                
            putlidrt(l,currentlidlist);                                         
            Write(routetab,',');                                                
            prtnode(l,0) ;                                                      
            Write(routetab,'.');                                                
            Writeln(routetab)                                                   
          END ;                                                                 
        END                                                                     
      END                                                                       
    END ;                                                                       
  END (* NJEFTable *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This procedure inspects the spanning tree and creates a JES2     *)         
(*  connection table. For each non adjacent node the last neighbour  *)         
(*  on the path to the given node is determined.                     *)         
(*                                                                   *)         
(*  Both a node definition statement and a connect statement are     *)         
(*  created.                                                         *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE JES2table(node:INTEGER; vers:ROUTTABFORMAT);                          
  VAR                                                                           
    i,j,k,l,si,ni : INTEGER ;                                                   
  BEGIN                                                                         
                                                                                
    asmsyntax := vers = JES2asmformat ;                                         
    startofcomment := '*' ;                                                     
    genid(node);                                                                
                                                                                
    begincomment;                                                               
    Write(routetab,'Offset for JES2 node numbers      :') ;                     
    Write(routetab,currentoffset:6);                                            
    endcomment ;                                                                
    Writeln(routetab);                                                          
                                                                                
    begincomment;                                                               
    Write(routetab,'Highest JES2 node number in table :') ;                     
    Write(routetab,maxjes2num+currentoffset:6) ;                                
    endcomment ;                                                                
    Writeln(routetab);                                                          
                                                                                
    begincomment;                                                               
    Write(routetab,'Number of invented JES2 numbers   :') ;                     
    Write(routetab,inventedjes2num:6) ;                                         
    endcomment ;                                                                
    Writeln(routetab);                                                          
    emptyline ;                                                                 
                                                                                
    neighbours(node);                                                           
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  We have to take as many nodes as we have in the sortednode       *)         
(*  array independant of how we sort the node.                       *)         
(*                                                                   *)         
(*  All non NJE node names are 'deleted' from the table, the         *)         
(*  name pointer is 0. In addition the jes2nodes value is 0,         *)         
(*  and the node 0 also has no name.                                 *)         
(*                                                                   *)         
(*********************************************************************)         
    ni := 0 ;                                                                   
    FOR si := 1 TO sortednode DO                                                
    BEGIN                                                                       
      IF currentsort = sortasinput THEN                                         
        i := sorted(.si.)                                                       
      ELSE                                                                      
      BEGIN (* sortjes2 *)                                                      
          ni := ni+1;                                                           
          i := jes2nodes(.ni.) ;                                                
      END ;                                                                     
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Now we may have a node that we want to use in a node definition  *)         
(*  and connect statement. Determine path to the local node.         *)         
(*  If we end up with 0, then this is either the own node or a       *)         
(*  neighbour. name may be 0 because the node entry could have       *)         
(*  be invalid on input.                                             *)         
(*                                                                   *)         
(*********************************************************************)         
      IF nodes(.i.).name > 0 THEN                                               
      BEGIN                                                                     
        k := paths(.i.) ;                                                       
        l := paths(.k.) ;                                                       
                                                                                
        IF l <> 0 THEN                                                          
        BEGIN                                                                   
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  There are a couple different syntaxes that have to be supported  *)         
(*  for JES2 systems.                                                *)         
(*                                                                   *)         
(*  I wonder whether 3.1.2 will have a new format.                   *)         
(*                                                                   *)         
(*********************************************************************)         
          Write(routetab,'N');                                                  
          IF vers = JES2esaformat THEN                                          
          BEGIN                                                                 
            Write(routetab,'ODE(');                                             
            putintrt(jes2numbers(.i.) + currentoffset) ;                        
            Write(routetab,') NAME=');                                          
            prtnode(i,0) ;                                                      
          END                                                                   
          ELSE                                                                  
          BEGIN                                                                 
            putintrt(jes2numbers(.i.) +currentoffset) ;                         
            Write(routetab,' NAME=');                                           
            prtnode(i,0) ;                                                      
            Write(routetab,                                                     
                 ',NONETATH,NODEVATH,NOJOBATH,NOSYSATH,SNA')                    
          END ;                                                                 
                                                                                
          Writeln(routetab) ;                                                   
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  At least the CONNECT is the same for all.                        *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
          Write(routetab,'CONNECT NODEA=');                                     
          putintrt(jes2numbers(.i.)+currentoffset) ;                            
                                                                                
          Write(routetab,',NODEB=');                                            
          putintrt(jes2numbers(.k.)+currentoffset) ;                            
                                                                                
          Write(routetab,',REST=2,MEMBA=1,MEMBB=1 /* ');                        
          prtnode(k,9) ;                                                        
          Write(routetab,'*/');                                                 
                                                                                
          Writeln(routetab)                                                     
        END                                                                     
      END                                                                       
    END ;                                                                       
  END (* JES2Table *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  This formatter creates a list for the CTRYSTAT program.          *)         
(*  The list contains all nodes of the network.                      *)         
(*                                                                   *)         
(*  This procedure inspects the spanning tree (and destroys it)      *)         
(*  the table.                                                       *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
PROCEDURE ctrystatlist(node:INTEGER);                                           
  VAR                                                                           
    i,j,k,l,si : INTEGER ;                                                      
  BEGIN                                                                         
    asmsyntax := True;                                                          
    startofcomment := '*' ;                                                     
                                                                                
    genid(node);                                                                
    begincomment ;                                                              
    Write(routetab,' ROUTING STATEMENTS FOR NODE ');                            
    prtnode(node,0) ;                                                           
    Writeln(routetab) ;                                                         
    Write(routetab,' LOCAL ') ;                                                 
    prtnode(node,9) ;                                                           
    Write(routetab,'*        ') ;                                               
    WITH domain(.node.) DO                                                      
    BEGIN                                                                       
      IF network = 0 THEN Write(routetab,'?')                                   
      ELSE putv(networks(.network.),routetab,0) ;                               
      Write(routetab,' ',c1,c2) ;                                               
    END ;                                                                       
    Writeln(routetab) ;                                                         
                                                                                
    FOR si := 1 TO sortednode DO                                                
    BEGIN                                                                       
      i := sorted(.si.) ;                                                       
      IF nodes(.i.).name > 0 THEN                                               
      BEGIN                                                                     
        j := paths(.i.) ;                                                       
        k := paths(.j.) ;                                                       
                                                                                
        IF (k = 0) AND (j <> 0) THEN                                            
        BEGIN                                                                   
          IF nodes(.i.).links > 0 THEN                                          
          BEGIN                                                                 
            Write(routetab,' LINK  ') ;                                         
            prtnode(i,9) ;                                                      
            prtnode(i,9)                                                        
          END                                                                   
          ELSE                                                                  
          BEGIN                                                                 
            Write(routetab,' ALIAS ') ;                                         
            prtnode(i,9) ;                                                      
            prtnode(j,9)                                                        
          END ;                                                                 
          WITH domain(.i.) DO                                                   
          BEGIN                                                                 
            IF network = 0 THEN Write(routetab,'?')                             
            ELSE putv(networks(.network.),routetab,0) ;                         
            Write(routetab,' ');                                                
            Write(routetab,c1,c2)                                               
          END ;                                                                 
          Writeln(routetab)                                                     
        END                                                                     
      END                                                                       
    END ;                                                                       
                                                                                
    FOR si := 1 TO sortednode DO                                                
    BEGIN                                                                       
      i := sorted(.si.) ;                                                       
      IF nodes(.i.).name > 0 THEN                                               
      BEGIN                                                                     
        l := i ;                                                                
        j := paths(.i.) ;                                                       
        k := paths(.j.) ;                                                       
        IF k <> 0 THEN (* not neighbour, not own node *)                        
        BEGIN                                                                   
          REPEAT                                                                
            paths(.l.) := i ;                                                   
            l := j ;                                                            
            j := k ;                                                            
            k := paths(.k.) ;                                                   
          UNTIL k = 0 ;                                                         
          paths(.i.) := l ;                                                     
          Write(routetab,' ROUTE ') ;                                           
          prtnode(i,9) ;                                                        
          IF l <> 0 THEN prtnode(l,9)                                           
          ELSE Write(routetab,'?        ') ;                                    
          WITH domain(.i.) DO                                                   
          BEGIN                                                                 
            IF network = 0 THEN Write(routetab,'?')                             
            ELSE putv(networks(.network.),routetab,0) ;                         
            Write(routetab,' ') ;                                               
            Write(routetab,c1,c2) ;                                             
          END ;                                                                 
          Writeln(routetab)                                                     
        END                                                                     
      END                                                                       
    END ;                                                                       
  END (* ctrystatlist *) ;                                                      
(*********************************************************************)         
(*                                                                   *)         
(*  Do a directive:                                                  *)         
(*                                                                   *)         
(*  The directive operands are parsed and apprpriate routines        *)         
(*  are called:                                                      *)         
(*                                                                   *)         
(*  For GETNODES the nodes table is read and checked.                *)         
(*                                                                   *)         
(*  For GENROUTS the node, filespec, and grip operands are parsed,   *)         
(*  spanningtree routine, and an appropriate formatter is invoked.   *)         
(*                                                                   *)         
(*  For TSO and CMS the commands are executed.                       *)         
(*                                                                   *)         
(*  GENPATHS is a debugging directive to create a detailed list      *)         
(*  of a spanning tree.                                              *)         
(*                                                                   *)         
(*  Although it sometimes seems that we do some error checking this  *)         
(*  is not the case; we assume a high degree of correctness of       *)         
(*  the data base.                                                   *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE docommand(currentdirective : DIRECTIVE) ;                             
  VAR currentnode,                                                              
      i,                                                                        
      safelidlists  : INTEGER ;                                                 
      f:              ROUTTABFORMAT ;                                           
      genrouttab    : BOOLEAN ;                                                 
  BEGIN                                                                         
    IF (currentdirective = DirGenRoute) OR                                      
       (currentdirective = DirCtryStat) OR                                      
       (currentdirective = DirGenPaths) THEN                                    
    BEGIN                                                                       
      IF NOT nodesverified THEN checknodes ;                                    
      nodesverified := True ;                                                   
      currentnode := findnode(commandop1,commandop1len) ;                       
                                                                                
      IF NOT nodesok THEN                                                       
        fatalerror(nodatabase)                                                  
      ELSE IF currentnode <= 0 THEN                                             
        fatalerror(invalidnode)                                                 
      ELSE IF nodes(.currentnode.).links <= 0 THEN                              
        fatalerror(invalidnode)                                                 
      ELSE IF NOT Spanningtree(currentnode) THEN                                
        fatalerror(treeerror)                                                   
      ELSE IF currentdirective = DirGenPaths THEN                               
        Putpaths(sysprint)                                                      
      ELSE                                                                      
      BEGIN                                                                     
        genrouttab := True ;                                                    
        IF commandop2len > 0 THEN                                               
        BEGIN                                                                   
          firstroutedone := False ;                                             
          Rewrite(routetab,                                                     
          tagvaluetostr(commandop2,commandop2+commandop2len-1))                 
        END                                                                     
        ELSE                                                                    
        IF (routtabformats(.currentnode.) = noneformat) THEN                    
        BEGIN                                                                   
          writeln(sysprint,'Format NONE, no table generated') ;                 
          genrouttab := False ;                                                 
        END                                                                     
        ELSE                                                                    
          Rewrite(routetab);                                                    
        IF genrouttab THEN                                                      
        BEGIN                                                                   
                                                                                
          eattagop(3);                                                          
          IF currentdirective = DirCtrystat THEN                                
            ctrystatlist(currentnode)                                           
          ELSE                                                                  
          BEGIN                                                                 
            safelidlists := lidlists;                                           
            IF commandoptions >= 0 THEN                                         
            BEGIN                                                               
              f := dorouttab ;                                                  
              IF lidassociations > 0 THEN                                       
                links(.lidlists.) := currentnode                                
            END ELSE BEGIN                                                      
              f := routtabformats(.currentnode.) ;                              
              currentoffset := offsets(.currentnode.);                          
              currentsort  := sorttypes(.currentnode.)                          
            END ;                                                               
                                                                                
            IF f = RSCSformat THEN RSCSTable(currentnode,f)                     
            ELSE IF f = JES3format THEN RSCSTable(currentnode,f)                
            ELSE IF f = JNETformat THEN RSCSTable(currentnode,f)                
            ELSE IF f = UREPformat THEN RSCSTable(currentnode,f)                
            ELSE IF f = NJEFformat THEN NJEFTable(currentnode)                  
            ELSE IF f = NTFformat  THEN ctrystatlist(currentnode)               
            ELSE IF f = PATHformat THEN PATHTable                               
            ELSE IF f = noneformat THEN                                         
            BEGIN                                                               
              writeln(sysprint,'Format NONE, RSCS table generated') ;           
              RSCSTable(currentnode,rscsformat)                                 
            END                                                                 
            ELSE IF f = unknownformat THEN                                      
            BEGIN                                                               
              writeln(sysprint,'Format unknown, RSCS table generated') ;        
              RSCSTable(currentnode,rscsformat)                                 
            END                                                                 
            ELSE JES2table(currentnode,f) ;                                     
            lidlists := safelidlists;                                           
          END ;                                                                 
          IF f <> PATHformat THEN BEGIN                                         
            begincomment ;                                                      
            Write(routetab,'End of table') ;                                    
            endcomment ;                                                        
            Writeln(routetab)                                                   
          END ;                                                                 
          closeroutetab                                                         
        END                                                                     
      END                                                                       
    END                                                                         
    ELSE IF currentdirective = DirGetNodes THEN                                 
    IF nodesverified THEN                                                       
      fatalerror(databaselate)                                                  
    ELSE                                                                        
    BEGIN                                                                       
      getnodes ;                                                                
      nodesok := True ;                                                         
    END                                                                         
    ELSE IF currentdirective = DirXmailer    THEN                               
    BEGIN                                                                       
      IF commandop1len > 0 THEN                                                 
        Rewrite(xmailer,                                                        
        tagvaluetostr(commandop1,commandop1+commandop1len-1))                   
      ELSE                                                                      
      Rewrite(Xmailer) ;                                                        
      genxmailer := True                                                        
    END                                                                         
    ELSE IF currentdirective = DirTSOcommand THEN                               
    BEGIN                                                                       
      IF TSOcommand = 0 THEN                                                    
    END                                                                         
    ELSE IF currentdirective = DirCMScommand THEN                               
    BEGIN                                                                       
      IF CMScommand = 0 THEN                                                    
    END                                                                         
    ELSE IF currentdirective = DirUnknown THEN                                  
      fatalerror(unknowndirective)                                              
    ELSE IF currentdirective <> Dircomment THEN                                 
      fatalerror(directivenotsupported)                                         
  END (* docommand *) ;                                                         
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  An input line from the COMMAND file is read operands are         *)         
(*  detected. An array of operand positions is created. Blanks       *)         
(*  are compressed to one blank. Users of the CMS and TSO command    *)         
(*  should take care!!                                               *)         
(*                                                                   *)         
(*********************************************************************)         
FUNCTION getcommand : DIRECTIVE ;                                               
  VAR                                                                           
    currentchar,ch  : CHAR ;                                                    
    i : INTEGER ;                                                               
    skipblanks   : BOOLEAN ;                                                    
  BEGIN                                                                         
    IF sermon THEN Write(sysprint,'GR directive: ');                            
    IF Eoln(commands) THEN                                                      
      getcommand := DirComment                                                  
    ELSE                                                                        
    BEGIN                                                                       
      tagvaluelength := 0 ;                                                     
      commandoptions:=-1;                                                       
      skipblanks := True;                                                       
      WHILE NOT Eoln(commands) DO                                               
      BEGIN                                                                     
        Read(commands,ch);                                                      
        IF sermon THEN Write(sysprint,ch);                                      
        CASE skipblanks OF                                                      
        False:                                                                  
          BEGIN                                                                 
            IF ch = ' ' THEN                                                    
              skipblanks := True ;                                              
            tagvaluelength:=tagvaluelength + 1;                                 
            tagvalue(.tagvaluelength.):=ch                                      
          END;                                                                  
        True:                                                                   
          IF ch <> ' ' THEN                                                     
          BEGIN                                                                 
            skipblanks := False ;                                               
            tagvaluelength:=tagvaluelength + 1;                                 
            tagvalue(.tagvaluelength.):=ch;                                     
            commandoptions:=commandoptions + 1;                                 
            commandop(.commandoptions.):=tagvaluelength                         
          END                                                                   
        END (* CASE *)                                                          
      END (* WHILE *) ;                                                         
      IF ch <> ' ' THEN                                                         
        tagvaluelength:=tagvaluelength + 1;                                     
      IF tagvaluelength <= tagvaluesize THEN                                    
      FOR i := tagvaluelength TO tagvaluesize DO                                
        tagvalue(.i.):= ' ' ;                                                   
      commandop(.commandoptions + 1.):=tagvaluelength + 1;                      
      FOR i := 1 TO tagvaluesize DO                                             
        cmdvalue(.i.):= tagvalue(.i.);                                          
      cmdvaluelength := tagvaluelength ;                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Now we set a few variables which describe the first two tokens.  *)         
(*  These parameters are a kind of leftover from a previous code.    *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
      commandop1:=commandop(.1.);                                               
      commandop1len:=commandop(.2.) - commandop(.1.) - 1;                       
      commandop2:=commandop(.2.);                                               
      commandop2len:=commandop(.3.) - commandop(.2.) - 1;                       
                                                                                
      IF sermon OR debug THEN Writeln(sysprint) ;                               
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Now we have to parse the first token.                            *)         
(*                                                                   *)         
(*       GETNODES <filespec>                                         *)         
(*       TSO command                                                 *)         
(*       CMS command                                                 *)         
(*       GENROUTE node <filespec> <grip>                             *)         
(*       GENPATHS node <filespec>                                    *)         
(*       CTRYSTAT node <filespec>                                    *)         
(*       XMAILER <filespec>                                          *)         
(*       QUIET                                                       *)         
(*       SERMON                                                      *)         
(*       DEBUG                                                       *)         
(*                                                                   *)         
(*  The following is the output of a poor man's parser generator.    *)         
(*                                                                   *)         
(*********************************************************************)         
      IF tagvalue(.1.) = '*' THEN                                               
        getcommand := Dircomment                                                
      ELSE                                                                      
      BEGIN                                                                     
        getcommand := DirUnknown;                                               
        IF commandop1 = 10 THEN                                                 
        BEGIN                                                                   
          IF    (tagvalue(.1.) = 'G')                                           
            AND (tagvalue(.2.) = 'E') THEN                                      
          BEGIN                                                                 
            IF tagvalue(.3.) = 'T' THEN                                         
            BEGIN                                                               
              IF (tagvalue(.4.) = 'N') AND                                      
                 (tagvalue(.5.) = 'O') AND                                      
                 (tagvalue(.6.) = 'D') AND                                      
                 (tagvalue(.7.) = 'E') AND                                      
                 (tagvalue(.8.) = 'S')                                          
              THEN getcommand  := DirGetNodes                                   
            END                                                                 
            ELSE IF tagvalue(.3.) = 'N' THEN                                    
            BEGIN                                                               
              IF (tagvalue(.4.) = 'R') AND                                      
                 (tagvalue(.5.) = 'O') AND                                      
                 (tagvalue(.6.) = 'U') AND                                      
                 (tagvalue(.7.) = 'T') AND                                      
                 (tagvalue(.8.) = 'E')                                          
              THEN getcommand  := DirGenRoute                                   
              ELSE IF (tagvalue(.4.) = 'P') AND                                 
                 (tagvalue(.5.) = 'A') AND                                      
                 (tagvalue(.6.) = 'T') AND                                      
                 (tagvalue(.7.) = 'H') AND                                      
                 (tagvalue(.8.) = 'S')                                          
              THEN getcommand  := DirGenPaths                                   
            END                                                                 
          END                                                                   
          ELSE IF (tagvalue(.1.) = 'C')                                         
            AND (tagvalue(.2.) = 'T')                                           
            AND (tagvalue(.3.) = 'R')                                           
            AND (tagvalue(.4.) = 'Y')                                           
            AND (tagvalue(.5.) = 'S')                                           
          THEN                                                                  
          BEGIN                                                                 
            IF  (tagvalue(.6.) = 'T')                                           
            AND (tagvalue(.7.) = 'A')                                           
            AND (tagvalue(.8.) = 'T')                                           
            THEN getcommand  := DirCtrystat                                     
          END                                                                   
        END                                                                     
        ELSE IF commandop1 = 5 THEN                                             
        BEGIN                                                                   
          IF tagvalue(.1.) = 'T' THEN                                           
          BEGIN                                                                 
            IF (tagvalue(.2.) = 'S') AND                                        
               (tagvalue(.3.) = 'O') THEN                                       
            getcommand := DirTSOcommand                                         
          END                                                                   
          ELSE IF tagvalue(.1.) = 'C' THEN                                      
          BEGIN                                                                 
            IF (tagvalue(.2.) = 'M') AND                                        
               (tagvalue(.3.) = 'S') THEN                                       
            getcommand := DirCMScommand                                         
          END                                                                   
        END                                                                     
        ELSE IF commandop1 = 7 THEN                                             
        BEGIN                                                                   
          IF tagvalue(.1.) = 'Q' THEN                                           
          BEGIN                                                                 
            IF (tagvalue(.2.) = 'U') AND                                        
               (tagvalue(.3.) = 'I') AND                                        
               (tagvalue(.4.) = 'E') AND                                        
               (tagvalue(.5.) = 'T') THEN                                       
            BEGIN                                                               
              getcommand := DirComment ;                                        
              sermon := False ;                                                 
              debug := False                                                    
            END                                                                 
          END                                                                   
          ELSE IF tagvalue(.1.) = 'D' THEN                                      
          BEGIN                                                                 
            IF (tagvalue(.2.) = 'E') AND                                        
               (tagvalue(.3.) = 'B') AND                                        
               (tagvalue(.4.) = 'U') AND                                        
               (tagvalue(.5.) = 'G') THEN                                       
            BEGIN                                                               
              getcommand := DirComment ;                                        
              debug := True                                                     
            END                                                                 
          END                                                                   
        END                                                                     
        ELSE IF commandop1 = 8 THEN                                             
        BEGIN                                                                   
          IF tagvalue(.1.) = 'S' THEN                                           
          BEGIN                                                                 
            IF (tagvalue(.2.) = 'E') AND                                        
               (tagvalue(.3.) = 'R') AND                                        
               (tagvalue(.4.) = 'M') AND                                        
               (tagvalue(.5.) = 'O') AND                                        
               (tagvalue(.6.) = 'N') THEN                                       
            BEGIN                                                               
              getcommand := DirComment ;                                        
              sermon := True                                                    
            END                                                                 
          END                                                                   
        END                                                                     
        ELSE IF commandop1 = 9 THEN                                             
        BEGIN                                                                   
          IF tagvalue(.1.) = 'X' THEN                                           
          BEGIN                                                                 
            IF (tagvalue(.2.) = 'M') AND                                        
               (tagvalue(.3.) = 'A') AND                                        
               (tagvalue(.4.) = 'I') AND                                        
               (tagvalue(.5.) = 'L') AND                                        
               (tagvalue(.6.) = 'E') AND                                        
               (tagvalue(.7.) = 'R') THEN                                       
            getcommand := DirXMAILER                                            
          END                                                                   
        END                                                                     
      END                                                                       
    END                                                                         
  END (* getcommand *) ;                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*  Read the COMMAND file and act on directives.                     *)         
(*                                                                   *)         
(*********************************************************************)         
PROCEDURE docommands ;                                                          
  LABEL 1,9 ;                                                                   
  VAR                                                                           
    currentchar  : CHAR ;                                                       
  BEGIN                                                                         
    Reset(commands) ;                                                           
  1:WHILE (NOT Eof(commands)) DO                                                
    BEGIN                                                                       
       IF NOT Eoln(commands) THEN                                               
          Docommand(getcommand) ;                                               
       Readln(commands) ;                                                       
    END ;                                                                       
9:END (* docommands *) ;                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   And here is the game...                                         *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
BEGIN                                                                           
                                                                                
  ebcdic := (' ' = chr(64)) ;                                                   
  upperminuslower := Ord('A') - Ord('a') ;                                      
  IF ebcdic THEN htab := chr(5) ELSE htab := chr(9) ;                           
  genroutsversion(.1.) := '9' ;                                                 
  genroutsversion(.2.) := '1' ;                                                 
  genroutsversion(.3.) := '-' ;                                                 
  genroutsversion(.4.) := '1' ;                                                 
  genroutsversion(.5.) := '1' ;                                                 
  genroutsversion(.6.) := '-' ;                                                 
  genroutsversion(.7.) := '1' ;                                                 
  genroutsversion(.8.) := '9' ;                                                 
                                                                                
  Rewrite(sysprint) ;                                                           
  Write(sysprint,'GR starting, version ');                                      
  FOR i := 1 TO 8 DO Write(sysprint,genroutsversion(.i.));                      
  datetime(date,time);                                                          
  dateiso(.1.) := date(.7.);                                                    
  dateiso(.2.) := date(.8.);                                                    
  dateiso(.3.) := '-' ;                                                         
  dateiso(.4.) := date(.1.);                                                    
  dateiso(.5.) := date(.2.);                                                    
  dateiso(.6.) := '-' ;                                                         
  dateiso(.7.) := date(.4.);                                                    
  dateiso(.8.) := date(.5.);                                                    
  Writeln(sysprint,', date=',dateiso,', time=',time);                           
  Writeln(sysprint,' ');                                                        
    Write(sysprint,'   (c) GMD Bonn 1989, 1990;');                              
  Writeln(sysprint,' EARN association 1990, 1991');                             
  Writeln(sysprint,'   Author Peter Sylvester ');                               
  Writeln(sysprint,' ');                                                        
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   I should not set all these vaiables here but rather have        *)         
(*   some initxxxx routines.                                         *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
  genxmailer := False ;                                                         
  xmtype := 0 ;                                                                 
  initbase36;                                                                   
  firstroutedone := False;                                                      
  sermon := True;                                                               
  debug := False ;                                                              
  stringfree := 1 ;                                                             
  linkfree := 1 ;                                                               
  numberofnetworks := 0 ;                                                       
  numberofnodes := 0 ;                                                          
  numberoftypes := 0 ;                                                          
  lidlists         := maxconn ;                                                 
  sortednode := 0 ;                                                             
  paths(.0.) := 0 ;                                                             
  defaultnoderesist := 1 ;                                                      
  defaultlinkrest := resistance9L - 1;                                          
                                                                                
  maxjes2num := 0 ;                                                             
  inventedjes2num := 0 ;                                                        
  dists(.0.) := 0 ;                                                             
                                                                                
  tagvalue(.1.) := 'N';                                                         
  tagvalue(.2.) := 'J';                                                         
  tagvalue(.3.) := 'E';                                                         
                                                                                
  njetype := findtype(1,3) ;                                                    
                                                                                
  prologuenotdone  := True ;                                                    
  linksnode := 0;                                                               
  versnode  := 0;                                                               
  FOR i := 1 TO heapsize DO                                                     
    heap(.i.) := 0 ;                                                            
  FOR i := 0 TO maxnodes DO                                                     
  BEGIN                                                                         
    WITH domain(.i.) DO                                                         
    BEGIN                                                                       
      c1 := '?' ; c2 := '?' ; network := 0 ;                                    
    END ;                                                                       
    via(.i.) := 0 ;                                                             
    routtabformats(.i.) := unknownformat ;                                      
    offsets(.i.) := 0 ;                                                         
    sorttypes(.i.) := sortasinput ;                                             
    jes2nodes(.i.) := 0 ;                                                       
    nodes(.i.).name := 0 ;                                                      
    nodes(.i.).links := 0 ;                                                     
                                                                                
                                                                                
  END ;                                                                         
                                                                                
  nodesok := False ;                                                            
  nodesverified := False ;                                                      
                                                                                
(*********************************************************************)         
(*                                                                   *)         
(*   Dear user, I'm ready to help you.                               *)         
(*                                                                   *)         
(*********************************************************************)         
                                                                                
  docommands ;                                                                  
                                                                                
  Writeln(sysprint,'GR successful.');                                           
9999 : ;                                                                        
  Writeln(sysprint,'GR done.');                                                 
END .                                                                           
